//---------------------------------------------------------------------------
// DBMaskEdit.cpp
// By: Malcolm Smith, April 2000
// Email: mjfreelancing@optusnet.com.au
// Web  : www.mjfreelancing.com
// Last modified 3rd May 2000
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DBMaskEdit.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TDBMaskEdit *)
{
  new TDBMaskEdit(NULL);
}
//---------------------------------------------------------------------------
__fastcall TDBMaskEdit::TDBMaskEdit(TComponent* Owner)
  : TMaskEdit(Owner)
{
FDataLink = new TFieldDataLink();
FDataLink->Control = this;
FDataLink->OnUpdateData = UpdateData;
FDataLink->OnDataChange = DataChange;
}
//---------------------------------------------------------------------------
__fastcall TDBMaskEdit::~TDBMaskEdit(void)
{
if(FDataLink)
  {
  FDataLink->Control = 0;
  FDataLink->OnUpdateData = 0;
  FDataLink->OnDataChange = 0;
  delete FDataLink;
  }
}
//---------------------------------------------------------------------------
namespace Dbmaskedit
{
  void __fastcall PACKAGE Register()
  {
    TComponentClass classes[1] = {__classid(TDBMaskEdit)};
    RegisterComponents("MJF Pack", classes, 0);
  }
}
//---------------------------------------------------------------------------
AnsiString __fastcall TDBMaskEdit::GetDataField(void)
{
return(FDataLink->FieldName);
}
//---------------------------------------------------------------------------
TDataSource * __fastcall TDBMaskEdit::GetDataSource(void)
{
return(FDataLink->DataSource);
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::SetDataField(AnsiString pDataField)
{
FDataLink->FieldName = pDataField;
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::SetDataSource(TDataSource *pDataSource)
{
if(pDataSource != NULL)
  pDataSource->FreeNotification(this);

FDataLink->DataSource = pDataSource;
}
//---------------------------------------------------------------------------
TField * __fastcall TDBMaskEdit::GetField(void)
{
return(FDataLink->Field);
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::DataChange(TObject *Sender)
{
if(!FDataLink->Field)
  {
  if(ComponentState.Contains(csDesigning))
    Text = Name;
  else
    Text = "";
  }
else
  Text = FDataLink->Field->AsString;
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::MouseDown(TMouseButton Button, TShiftState Shift, int X, int Y)
{
if(!ReadOnly && FDataLink->Edit())
  TMaskEdit::MouseDown(Button, Shift, X, Y);
else
  {
  if(OnMouseDown)
    OnMouseDown(this, Button, Shift, X , Y);
  }
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::KeyDown(unsigned short &Key, TShiftState Shift)
{
/*
// defined in winuser.h
#define VK_PRIOR          0x21
#define VK_NEXT           0x22
#define VK_END            0x23
#define VK_HOME           0x24
#define VK_LEFT           0x25
#define VK_UP             0x26
#define VK_RIGHT          0x27
#define VK_DOWN           0x28
*/

Set<unsigned short, VK_PRIOR, VK_DOWN> Keys;
Keys = Keys << VK_PRIOR << VK_NEXT << VK_END << VK_HOME << VK_LEFT << VK_UP << VK_RIGHT << VK_DOWN;

if(!ReadOnly && (Keys.Contains(Key)) && FDataLink->Edit())
  TMaskEdit::KeyDown(Key, Shift);
else
  {
  if(OnKeyDown)
    OnKeyDown(this, Key, Shift);
  }
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::UpdateData(TObject *Sender)
{
if(FDataLink->CanModify)
  FDataLink->Field->AsString = Text;
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::Change(void)
{
if(FDataLink)
  {
  // we need to see if the datasource is in edit mode
  // if not then we need to save the current value because
  // placing the datasource into edit mode will change the
  // current value to that already present in the table
  AnsiString ChangedValue = Text;

  // get cursor position too
  int CursorPosition = SelStart;

  if(FDataLink->CanModify && FDataLink->Edit())         // need to be in edit mode
    {
    Text = ChangedValue;        // just in case we were not in edit mode
    SelStart = CursorPosition;
    FDataLink->Modified();      // posting a change (the datasource is not put back into edit mode)
    }
  }

TMaskEdit::Change();
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::CMExit(TWMNoParams Message)
{
try
  {
  ValidateEdit();             // added this to validate the entry against the mask
  if(FDataLink && FDataLink->CanModify)
    FDataLink->UpdateRecord();
  }

catch(...)
  {
  SetFocus();
  throw;
  }
}
//---------------------------------------------------------------------------
void __fastcall TDBMaskEdit::CMGetDataLink(TMessage Message)
{
Message.Result = (int)FDataLink;
}
//---------------------------------------------------------------------------

