// ExampleButton.cpp
// By: Sean Rock
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ExampleButton.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//

static inline void ValidCtrCheck(TExampleButton *)
{
  new TExampleButton(NULL);
}
//---------------------------------------------------------------------------
__fastcall TExampleButton::TExampleButton(TComponent* Owner)
  : TGraphicControl(Owner)
{
  SetBounds(0,0,50,50);
  ControlStyle = ControlStyle << csReplicatable;
  FState = esFlat;

}
//---------------------------------------------------------------------------
namespace Examplebutton
{
  void __fastcall PACKAGE Register()
  {
     TComponentClass classes[1] = {__classid(TExampleButton)};
     RegisterComponents("Samples", classes, 0);
  }
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::CMMouseEnter(TMessage &Msg)
{
  if(Enabled)
    {
    FState = esUp;
    FMouseInControl = true;
    Invalidate();
    }
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::CMMouseLeave(TMessage &Msg)
{
  if(Enabled)
    {
    FState = esFlat;
    FMouseInControl = false;
    Invalidate();
    }
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::CMEnabledChanged(TMessage &Msg)
{
  FState = (Enabled) ? esFlat : esDisabled;
  Invalidate();
}

// ---------------------------------------------------------------------------
void __fastcall TExampleButton::MouseDown(TMouseButton Button, TShiftState
  Shift, int X, int Y)
{
  if(Button == mbLeft)
    {
    if(Enabled && FMouseInControl)
      {
      FState = esDown;
      Invalidate();
      }
    }
}

// ---------------------------------------------------------------------------
void __fastcall TExampleButton::MouseUp(TMouseButton Button, TShiftState
  Shift, int X, int Y)
{
  if(Button == mbLeft)
    {
    if(Enabled && FMouseInControl)
      {
      FState = esUp;
      Invalidate();
      if(FOnClick)
        FOnClick(this);
      }
    }
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::SetGlyph(Graphics::TBitmap * Value)
{
  if(Value == NULL)
    return;

  if(!FGlyph)
    FGlyph = new Graphics::TBitmap;
  FGlyph->Assign(Value);
  Invalidate();
}

// ---------------------------------------------------------------------------
void __fastcall TExampleButton::SetCaption(AnsiString Value)
{
  FCaption = Value;
  Invalidate();
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::SwapColors(TColor &Top, TColor &Bottom)
{
  if(ComponentState.Contains(csDesigning))
    {
    FState = esUp;
    }

  Top = (FState == esUp) ? clBtnHighlight : clBtnShadow;
  Bottom = (FState == esDown) ? clBtnHighlight : clBtnShadow;
}

// ---------------------------------------------------------------------------
void __fastcall TExampleButton::BeforeDestruction(void)
{
  if(FImage)
    delete FImage;

  if(FGlyph)
    delete FGlyph;
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::Paint(void)
{
  TRect cRect, tRect, gRect;
  TColor TopColor, BottomColor;

  Canvas->Brush->Color = clBtnFace;
  Canvas->FillRect(ClientRect);
  cRect = ClientRect;
  Graphics::TBitmap *bit = new Graphics::TBitmap;
  bit->Width = ClientWidth;
  bit->Height = ClientHeight;
  bit->Canvas->Brush->Color = clBtnFace;
  bit->Canvas->FillRect(cRect);

  if(FGlyph)
    if(!FGlyph->Empty)
      {
      CalcGlyphLayout(gRect);
      bit->Canvas->BrushCopy(gRect, FGlyph,
        Rect(0,0,FGlyph->Width,FGlyph->Height), FGlyph->TransparentColor);
      }
  if(!FCaption.IsEmpty())
    {
    CalcTextLayout(tRect);
    bit->Canvas->TextRect(tRect, tRect.Left,tRect.Top, FCaption);
    }

  if(FState == esUp || FState == esDown)
    {
    SwapColors(TopColor, BottomColor);
    Frame3D(bit->Canvas, cRect, TopColor, BottomColor, 1);
    }


  BitBlt(Canvas->Handle, 0, 0, ClientWidth, ClientHeight, bit->Canvas->Handle, 0, 0,
      SRCCOPY);

  delete bit;

}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::CalcGlyphLayout(TRect &r)
{

  int TotalHeight=0;
  int TextHeight=0;

  if(!FCaption.IsEmpty())
    TextHeight = Canvas->TextHeight(FCaption);

  // the added 5 could be a 'Spacing' property but for simplicity I've just
  // added the 5.
  TotalHeight = FGlyph->Height + TextHeight + 5;

  r = Rect((ClientWidth/2)-(FGlyph->Width/2),
          ((ClientHeight/2)-(TotalHeight/2)), FGlyph->Width +
          (ClientWidth/2)-(FGlyph->Width/2), FGlyph->Height +
          ((ClientHeight/2)-(TotalHeight/2)));
}
// ---------------------------------------------------------------------------
void __fastcall TExampleButton::CalcTextLayout(TRect &r)
{
  int TotalHeight=0;
  int TextHeight=0;
  int TextWidth=0;
  TRect temp;

  if(FGlyph)
    TotalHeight = FGlyph->Height;

  TextHeight = Canvas->TextHeight(FCaption);
  TextWidth = Canvas->TextWidth(FCaption);

  TotalHeight += TextHeight + 5;

  temp.Left = 0;
  temp.Top = (ClientHeight/2)-(TotalHeight/2);
  temp.Bottom = temp.Top + TotalHeight;
  temp.Right = ClientWidth;

  r = Rect(((ClientWidth/2) - (TextWidth/2)), temp.Bottom-TextHeight,
      ((ClientWidth/2)-(TextWidth/2))+TextWidth, temp.Bottom);
}
