// TSHFileListbox
// By:  Sean Rock
// roks@bigfoot.com
// http://www.theblack.co.uk
// last modified: 11 Aug 00
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "SHFileListBox.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
__fastcall TShellFileListItem::TShellFileListItem(LPITEMIDLIST lpidl, int Index)
  : TObject()
{
  // store a copy of the file's pidl
  Fpidl = CopyPIDL(lpidl);
  // and save its icon index
  FImageIndex = Index;
}
//----------------------------------------------------------------------------
__fastcall TShellFileListItem::~TShellFileListItem(void)
{
  LPMALLOC lpMalloc=NULL;
  if(SUCCEEDED(SHGetMalloc(&lpMalloc)))
    {
    // free the memory associated with the pidl
    lpMalloc->Free(Fpidl);
    lpMalloc->Release();
    }
}

//---------------------------------------------------------------------------
__fastcall TSHFileListBox::TSHFileListBox(TComponent* Owner)
  : TFileListBox(Owner)
{
  ItemHeight = 18;
  ShowGlyphs = true;
  FCanLaunch = true;
  FRightBtnSel = true;
}
//---------------------------------------------------------------------------
__fastcall TSHFileListBox::~TSHFileListBox(void)
{
  // free the images
  if(FImages)
    delete FImages;
  FImages = NULL;
}
//----------------------------------------------------------------------------
void __fastcall TSHFileListBox::DeleteString(int Index)
{
  // This method is called in response to the LB_DELETESTRING messeage
  // First delete the TShellFileListItem pointed to by the string's
  // Object property
  TShellFileListItem *ShellItem = reinterpret_cast<TShellFileListItem*>
    (Items->Objects[Index]);
  delete ShellItem;
  ShellItem = NULL;
  // now delete the item
  Items->Delete(Index);
}
//----------------------------------------------------------------------------
namespace Shfilelistbox
{
  void __fastcall PACKAGE Register()
  {
     TComponentClass classes[1] = {__classid(TSHFileListBox)};
     RegisterComponents("Samples", classes, 0);
  }
}
//---------------------------------------------------------------------------
void __fastcall TSHFileListBox::ReadFileNames(void)
{

  LPMALLOC g_pMalloc;
  LPSHELLFOLDER pisf;
  LPSHELLFOLDER sfChild;
  LPITEMIDLIST pidlDirectory;
  LPITEMIDLIST rgelt;
  LPENUMIDLIST ppenumIDList;
  int hExtent;

  try
    {
    try
      {
      if(HandleAllocated())
        {
        GetSysImages();
        // prohibit screen updates
        Items->BeginUpdate();
        // delete the items already in the list
        Items->Clear();
        // get the shell's global allocator
        if(SHGetMalloc(&g_pMalloc) != NOERROR)
          {
          return;
          }
        // get the desktop's IShellFolder interface
        if(SHGetDesktopFolder(&pisf) != NOERROR)
          {
          return;
          }

        // convert folder string to WideChar
        WideChar oleStr[MAX_PATH];
        FDirectory.WideChar(oleStr, MAX_PATH);
        unsigned long pchEaten;
        unsigned long pdwAttributes;
        // get pidl of current folder
        pisf->ParseDisplayName(Handle, 0, oleStr, &pchEaten,
                &pidlDirectory, &pdwAttributes);

        // get an IShellFolder interface for the current folder
        if(pisf->BindToObject(pidlDirectory,NULL,
            IID_IShellFolder, (void**)&sfChild) != NOERROR)
          {
          return;
          }
        // enumerate the objects withing the folder
        sfChild->EnumObjects(Handle, SHCONTF_NONFOLDERS |
          SHCONTF_INCLUDEHIDDEN, &ppenumIDList);

        // walk through the enumlist
        ULONG celt = 1;
        ULONG Fetched = 0;
        ppenumIDList->Next(celt, &rgelt, &Fetched);
        hExtent = 0;
        while(Fetched > 0)
          {
          // add the item to the listbox
          int l = AddItem(rgelt);
          if(l > hExtent)
            hExtent = l;
          ppenumIDList->Next(celt, &rgelt, &Fetched);
          }

        }
      }
    catch(Exception &E)
      {
      throw(E); // re-throw any exceptions
      }
    }
  __finally
    { // make sure we do this reguardless
    g_pMalloc->Free(rgelt);
    g_pMalloc->Free(ppenumIDList);
    g_pMalloc->Free(pidlDirectory);
    pisf->Release();
    sfChild->Release();
    g_pMalloc->Release();
    Items->EndUpdate();
    }
  // Show the horizontal scrollbar if necessary
  DoHorizontalScrollBar(hExtent);
}
// ---------------------------------------------------------------------------
void __fastcall TSHFileListBox::DoHorizontalScrollBar(int he)
{
  // add a little space for the margins
  he += 2;
  // if we're showing the images make room for it plus a bit more
  // for the space between the image and the text
  if(ShowGlyphs)
    he += 18;

  Perform(LB_SETHORIZONTALEXTENT, he, 0);
}

// ---------------------------------------------------------------------------
void __fastcall TSHFileListBox::GetSysImages(void)
{
  SHFILEINFO shfi;
  DWORD iHnd;
  if(!FImages)
    {
    FImages = new TImageList(this);
    FImages->ShareImages = true;
    FImages->Height = 16;
    FImages->Width = 16;
    iHnd = SHGetFileInfo("", 0, &shfi, sizeof(shfi), SHGFI_SYSICONINDEX |
      SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);
    if(iHnd != 0)
      FImages->Handle = iHnd;
    }
}
// ---------------------------------------------------------------------------
int __fastcall TSHFileListBox::AddItem(LPITEMIDLIST pidl)
{
  SHFILEINFO shfi;
  int Index;

  SHGetFileInfo((char*)pidl, 0, &shfi, sizeof(shfi), SHGFI_PIDL | SHGFI_SYSICONINDEX |
    SHGFI_SMALLICON | SHGFI_DISPLAYNAME | SHGFI_USEFILEATTRIBUTES);

  // fire the OnAddItem event to allow the user the choice to add the
  // file name or not
  bool FCanAdd = true;
  if(FOnAddItem)
    FOnAddItem(this, AnsiString(shfi.szDisplayName), FCanAdd);

  if(FCanAdd)
    {
    TShellFileListItem *ShellInfo = new TShellFileListItem(pidl, shfi.iIcon);
    Index = Items->AddObject(AnsiString(shfi.szDisplayName), (TObject*)ShellInfo);
    // return the length of the file name
    return Canvas->TextWidth(Items->Strings[Index]);
    }
  // return zero as the length as the file has not been added
  return 0;
}
// ---------------------------------------------------------------------------
void __fastcall TSHFileListBox::DrawItem(int Index, const TRect &Rect,
  TOwnerDrawState State)
{
  int Offset;

  Canvas->FillRect(Rect);
  Offset = 2;
  if(ShowGlyphs)
    {
    TShellFileListItem *ShellItem = reinterpret_cast<TShellFileListItem*>
      (Items->Objects[Index]);
    // draw the file's icon in the listbox
    FImages->Draw(Canvas, Rect.Left+2, Rect.Top+2, ShellItem->ImageIndex, true);
    Offset += 18;
    }
  int Texty = Canvas->TextHeight(Items->Strings[Index]);
  Texty = ((ItemHeight - Texty) / 2) + 1;
  // now draw the text
  Canvas->TextOut(Rect.Left + Offset, Rect.Top + Texty, Items->Strings[Index]);
}
//----------------------------------------------------------------------------
void __fastcall TSHFileListBox::DblClick(void)
{
  if(FCanLaunch)
    {
    int ii=0;
    // go through the list and find which item is selected
    for(ii=0; ii < Items->Count; ii++)
      {
      if(Selected[ii])
        {
        AnsiString str = Items->Strings[ii];
        ShellExecute(Handle, "open",	str.c_str(), 0, 0, SW_SHOWDEFAULT);
        }
      }
    }
  // fire the OnDblClick event
  if(FOnDblClick)
    FOnDblClick(this);
}
//----------------------------------------------------------------------------
void __fastcall TSHFileListBox::MouseUp(TMouseButton Button, TShiftState Shift,
  int X, int Y)
{
  if(!FRightBtnSel)
    return;

  TPoint ItemPos = Point(X,Y);
  // is there an item under the mouse ?
  int Index = ItemAtPos(ItemPos, true);
  // if not just return
  if(Index == -1)
    return;
  // else select the item
  Perform(LB_SETCURSEL, (WPARAM)Index, 0);
}
//----------------------------------------------------------------------------
// ValidCtrCheck is used to assure that the components created do not have
// any pure virtual functions.
//
static inline void ValidCtrCheck(TSHFileListBox *)
{
  new TSHFileListBox(NULL);
}
// ---------------------------------------------------------------------------
// HELPER FUNCTIONS
// The following functions were written by Damon Chandler
//----------------------------------------------------------------------------
LPITEMIDLIST CopyPIDL(LPITEMIDLIST lpidl)
{
  unsigned short size = GetPIDLSize(lpidl);
  LPITEMIDLIST lpidlCopy = CreatePIDL(size);
  MoveMemory(lpidlCopy, lpidl, size);
  return lpidlCopy;
}
//----------------------------------------------------------------------------
unsigned short GetPIDLSize(LPITEMIDLIST lpidl)
{
  unsigned short cb = 0;
  while (lpidl)
    {
    cb = (unsigned short)(cb + lpidl->mkid.cb);
    lpidl = GetNextItem(lpidl);
    }
  return (unsigned short)(cb + 2);
}
//----------------------------------------------------------------------------
LPITEMIDLIST CreatePIDL(unsigned short size)
{
  LPITEMIDLIST lpidlResult;
  LPMALLOC lpMalloc;
  if (SUCCEEDED(SHGetMalloc(&lpMalloc)))
    {
    lpidlResult = (LPITEMIDLIST)lpMalloc->Alloc(size);
    if (lpidlResult)
      ZeroMemory(lpidlResult, sizeof(size));
    lpMalloc->Release();
    }
  return lpidlResult;
}
//----------------------------------------------------------------------------
LPITEMIDLIST GetNextItem(LPITEMIDLIST pidl)
{
  unsigned short nLen = pidl->mkid.cb;
  if(nLen == 0)
    return NULL;
  return (LPITEMIDLIST)((LPBYTE)pidl + nLen);
}
