//------------------------------------------------------------------------------------------//
#include <vcl.h> 
#pragma hdrstop

#include "PropertyEditors.h"
#include "Comctrls.hpp"
#include "Consts.hpp"
#include <memory>
//------------------------------------------------------------------------------------------//
//#pragma package(smart_init,weak)

// You may wish to recompile the package with this unit weakly packaged to see the diiference
// in the *.bpi file generated. If the unit is weakly packaged the *.bpi file is substantially
// bigger as it contains the .obj of the unit. Aside from this the package will function as
// before except the name of the unit will not be an issue.

#pragma package(smart_init)
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TShapeTypePropertyEditor                                    //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

void __fastcall TShapeTypePropertyEditor::ListMeasureWidth(const AnsiString Value,
                                                           Graphics::TCanvas* ACanvas,
                                                           int& AWidth)
{
   AWidth += (ACanvas->TextHeight("Ag")+2) + 0; // 0 is the ImageWidthIncrease

   // This allows for vRight being the Right-Hand side of a SQUARE image.
   // The TextHeight("Ag")+2 ensures that actual Height of the Rect canvas is returned,
   // as this is the width of a SQUARE based image.

   // For a RECTANGULAR image a multiple number of SQUAREs (say 2) wide, the following
   // code would be used:
   //
   // AWidth = AWidth + (ACanvas->TextHeight("A")+2)*2 + ImageWidthIncrease;

   // ImageWidthIncrease is the number of pixels BEYOND the basic SQUARE (or multiple) image
   // that the image is.

   // Both the (ACanvas->TextHeight("Ag")+2) and ImageWidthIncrease variables could be
   // replaced by the exact pixel width of the image if it is known.
}
//------------------------------------------------------------------------------------------//

void __fastcall TShapeTypePropertyEditor::ListMeasureHeight(const AnsiString Value,
                                                            Graphics::TCanvas* ACanvas,
                                                            int& AHeight)
{
   AHeight += 0; // 0 is the ImageHeightIncrease

   // ImageHeightIncrease is the number of pixels BEYOND the default height for the list entry
   // that the image may be. If an exact figure is known for the height of the image then that
   // could be used directly. However remember that AHeight should not be less than :
   //
   // ACanvas->TextHeight("Ag")+2
   //
   // This is because this will allow enough room for the height of the text with a one pixel
   // border surrounding it
   //
   // If a direct assignment is used then the value to be assigned should be compared with
   // that obtained from  ACanvas->TextHeight("Ag")+2  and the larger of the two used.
}
//------------------------------------------------------------------------------------------//

void __fastcall TShapeTypePropertyEditor::ListDrawValue(const AnsiString Value,
                                                        Graphics::TCanvas* ACanvas,
                                                        const Types::TRect& ARect,
                                                        bool ASelected)

{
   // vRight declared here as it will be used in the __finally block as well
   int vRight = ARect.Bottom - ARect.Top + ARect.Left;
 
   try
   {
      // Step 1 - Save ACanvas properties that we are going to change

      TColor vOldPenColor = ACanvas->Pen->Color;
      TColor vOldBrushColor = ACanvas->Brush->Color;


      // Step 2 - Frame the area to be modified. This is required so that any previous
      //          rendering on the canvas is overwritten. For example when the IDE selection
      //          rendering is applied, i.e., the property value is surrounded by a dashed
      //          yellow and black line and the AnsiString representation is highlighted in
      //          clBlue, and focus then moves to another list value the modified parts of
      //          ACanvas are cleared, ready for the custom rendering. To see the effect of
      //          including this operation simply comment out the follwoing two lines. If the
      //          entire ACanvas is going to be changed then this operation is not required.

      ACanvas->Pen->Color = ACanvas->Brush->Color;
      ACanvas->Rectangle(ARect.Left, ARect.Top, vRight, ARect.Bottom);


      // Step 3 - Perform any preparation required. For example paint a background colour
      //          and place a highlight box around the image of the list value if ASelected
      //          is true.
      //
      //          To choose a colour to match the current text used by windows select
      //          clWindowText, this is useful as an image border, hence this is often
      //          selected as a suitable ACanvas->Pen colour.
      //
      //          To choose give the appearance of a clear background, clear border or both
      //          set the ACanvas->Brush and/or ACanvas->Pen colour to clWindow.
      //
      //          To use a colour the same as the Object Inspector choose clBtnFace.


      if(ASelected)                            // Choose a Pen colour
      {                                        // depending on whether
         ACanvas->Pen->Color = clYellow;       // the list value is
      }                                        // selected or not
      else
      {
         ACanvas->Pen->Color = clBtnFace;
      }

      ACanvas->Brush->Color = clBtnFace;       // Choose a background color to
                                               // match the Object Inspector


      ACanvas->Rectangle( ARect.Left + 1,      // Draw the background onto
                          ARect.Top + 1,       // the Canvas using the
                          vRight - 1,          // current Pen and the
                          ARect.Bottom - 1 );  // current Brush :-)


      // Step 4 - Determine the value of the current list item
      TShapeType ShapeType = TShapeType(GetEnumValue(GetPropType(), Value));

      // Line 150 and 630 in $(BCB)\Source\Vcl\typinfo.pas for GetEnumValue and other
      // typeinfo VCL procedures

      // Step 5 - Draw the required image onto ACanvas

      ACanvas->Pen->Color = clBlack;
      ACanvas->Brush->Color = clWhite;

      switch(ShapeType)
      {
         case stRectangle   : ACanvas->Rectangle(ARect.Left+2,
                                                  ARect.Top+4,
                                                  vRight-2,
                                                  ARect.Bottom-4);
                               break;
         case stSquare      : ACanvas->Rectangle(ARect.Left+2,
                                                  ARect.Top+2,
                                                  vRight-2,
                                                  ARect.Bottom-2);
                               break;
         case stRoundRect   : ACanvas->RoundRect(ARect.Left+2,
                                                  ARect.Top+4,
                                                  vRight-2,
                                                  ARect.Bottom-4,
                                                  (ARect.Bottom-ARect.Top-6)/2,
                                                  (ARect.Bottom-ARect.Top-6)/2);
                               break;
         case stRoundSquare : ACanvas->RoundRect(ARect.Left+2,
                                                  ARect.Top+2,
                                                  vRight-2,
                                                  ARect.Bottom-2,
                                                  (ARect.Bottom-ARect.Top)/3,
                                                  (ARect.Bottom-ARect.Top)/3);
                               break;
         case stEllipse     : ACanvas->Ellipse(ARect.Left+1,
                                                ARect.Top+2,
                                                vRight-1,
                                                ARect.Bottom-2);
                               break;
         case stCircle      : ACanvas->Ellipse(ARect.Left+1,
                                                ARect.Top+1,
                                                vRight-1,
                                                ARect.Bottom-1);
                               break;
         default : break;
      }

      // Step 6 - Restore modified ACanvas properties to their original values

      ACanvas->Pen->Color = vOldPenColor;
      ACanvas->Brush->Color = vOldBrushColor;

   }
   __finally
   {
      // Perform the following operation to render the AnsiString represnetation of
      // the current item, i.e. Value, onto ACanvas

      // 1. Either call the parents ListDrawValue method passing vRight as the ARect.Left
      //    parameter of the Rect variable, i.e.
      //
      //    TEnumProperty::ListDrawValue(Value,
      //                                 ACanvas,
      //                                 Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
      //                                 ASelected);
      //    which becomes:
      //
      //    inherited::ListDrawValue(Value,
      //                             ACanvas,
      //                             Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
      //                             ASelected);
      //
      //    using our typedef.

      // 2. Or perform this operation directly by calling the TextRect member function
      //    directly; removing the need to call the parent version of this virtual function.
      //    The basic behaviour used to draw an AnsiString Value onto the Canvas is:
      //
      //    ACanvas->TextRect( SomeRect, vRight + 1, ARect.Top + 1, Value );

      // Use Method 1
      /*
      inherited::ListDrawValue(Value,
                               ACanvas,
                               Rect(vRight,
                                    ARect.Top,
                                    ARect.Right,
                                    ARect.Bottom),
                               ASelected);
      */
      // Use method 2
      ACanvas->TextRect( Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
                         vRight+1,
                         ARect.Top+1,
                         Value );
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TShapeTypePropertyEditor::PropDrawValue(Graphics::TCanvas* ACanvas,
                                                    const Types::TRect& ARect,
                                                    bool ASelected)
{
   if( GetVisualValue() != "" )
   {
      ListDrawValue(GetVisualValue(), ACanvas, ARect, ASelected);
   }
   else
   {
      // Perform the following operation to render the AnsiString Value onto ACanvas

      // 1. Either call the parents PropDrawValue method , i.e.
      //
      //    TEnumProperty::PropDrawValue(ACanvas, ARect, ASelected);
      //
      //    which becomes:
      //
      //    inherited::PropDrawValue(ACanvas, ARect, ASelected);
      //
      //    using our typedef

      // 2. Or perform this operation directly by calling the TextRect member function
      //    directly removing the need to call the parent version of this virtual function. In
      //    C++ this is probably better as there is no proper mechanism for extending the
      //    functionality of virtual functions in a robust way, for example, if the name of
      //    the parent class is changed, all occurences of the name in this derived class must
      //    also be changed, though using a typedef as we have done is a reasoanble solution.
      //
      //    The basic behaviour used to draw an AnsiString Value onto the Canvas is:
      //
      //    ACanvas->TextRect( ARect, ARect.Left + 1, ARect.Top + 1, GetVisualValue() );


      // Use Method 1
      //inherited::PropDrawValue(ACanvas, ARect, ASelected);

      // Use Method 2
      ACanvas->TextRect( ARect,
                         ARect.Left+1,
                         ARect.Top+1,
                         GetVisualValue() );
   }
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                               TDisplayFontNameProperty                                   //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

TPropertyAttributes __fastcall TDisplayFontNameProperty::GetAttributes(void)
{
   return TPropertyAttributes() << paValueList
                                << paSortList
                                << paRevertable
                                << paMultiSelect;
}
//------------------------------------------------------------------------------------------//

void __fastcall TDisplayFontNameProperty::GetValues(Classes::TGetStrProc Proc)
{
   for(int i = 0; i<Screen->Fonts->Count; ++i) Proc(Screen->Fonts->Strings[i]);
}

//------------------------------------------------------------------------------------------//

void __fastcall TDisplayFontNameProperty::ListMeasureWidth(const AnsiString Value,
                                                       Graphics::TCanvas* ACanvas,
                                                       int& AWidth)
{
   // Save the current font
   AnsiString OldFontName = ACanvas->Font->Name;

   // Set the font of Value
   ACanvas->Font->Name = Value;

   // Determine the width required to display the string
   AWidth += ACanvas->TextWidth("The quick brown fox jumps over the lazy dog") + 4;

   // Restore the font name
   ACanvas->Font->Name = OldFontName;
}

//------------------------------------------------------------------------------------------//

void __fastcall TDisplayFontNameProperty::ListMeasureHeight(const AnsiString Value,
                                                            Graphics::TCanvas* ACanvas,
                                                            int& AHeight)
{
   // Save the current font
   AnsiString OldFontName = ACanvas->Font->Name;

   // Set the font of Value
   ACanvas->Font->Name = Value;

   // Determine the width required to display the string
   if( (ACanvas->TextHeight("Ag")+2) > AHeight)
   {
      AHeight = ACanvas->TextHeight("Ag") + 2;
   }

   // Restore the font name
   ACanvas->Font->Name = OldFontName;
}
//------------------------------------------------------------------------------------------//

void __fastcall TDisplayFontNameProperty::ListDrawValue(const AnsiString Value,
                                                        Graphics::TCanvas* ACanvas,
                                                        const Types::TRect& ARect,
                                                        bool ASelected)


{
   // Save the current font color
   TColor OldFontColor = ACanvas->Font->Color;

   if(ASelected) ACanvas->Font->Color = clYellow;
   else ACanvas->Font->Color = clNavy;

//   inherited::ListDrawValue(Value, ACanvas, ARect, ASelected);

   // Restore the font color
   ACanvas->Font->Color = OldFontColor;

   // Calculate the offset
   int vRight = ARect.Left + ACanvas->TextWidth(Value) + 4;

   // Save the current font
   AnsiString OldFontName = ACanvas->Font->Name;

   // Set the font of Value
   ACanvas->Font->Name = Value;

   // Display the font name in the same font
   ACanvas->TextRect(Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
                     vRight + 1,
                     ARect.Top + 1,
                     "The quick brown fox jumps over the lazy dog");

   // Restore the font name
   ACanvas->Font->Name = OldFontName;
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TImageListPropertyEditor                                    //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

__fastcall TImageListPropertyEditor::TImageListPropertyEditor
                                     (const _di_IDesigner ADesigner,
                                      int APropCount)
                                     : TComponentProperty(ADesigner, APropCount)
{
   ImageListPropertyImage = new Graphics::TBitmap();

   ImageListPropertyImage->LoadFromResourceName(reinterpret_cast<int>(HInstance),
                                                "RESOURCE_ImageListProperty");
}
//------------------------------------------------------------------------------------------//

__fastcall TImageListPropertyEditor::~TImageListPropertyEditor(void)
{
   delete ImageListPropertyImage;
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageListPropertyEditor::PropDrawName(Graphics::TCanvas* ACanvas,
                                                       const Types::TRect& ARect,
                                                       bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(GetName() != "")
      {
         if(Screen->PixelsPerInch > 96)
         {
            ACanvas->Draw( ARect.Left + 1,
                           ARect.Top + 2,
                           ImageListPropertyImage );
         }
         else
         {
            ACanvas->Draw( ARect.Left + 1,
                           ARect.Top,
                           ImageListPropertyImage );
         }


         ValueRect = Rect( ARect.Left + 16 + 2,
                           ARect.Top,
                           ARect.Right,
                           ARect.Bottom );
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      ACanvas->TextRect( ARect,
                         ARect.Left+1,
                         ARect.Top+1,
                         GetName() );
   }

}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                                TImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

TPropertyAttributes __fastcall TImageIndexProperty::GetAttributes(void)
{
   return (inherited::GetAttributes() << paValueList >> paMultiSelect);
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexProperty::GetValues(Classes::TGetStrProc Proc)
{
   TCustomImageList* ImageList = RemoteImageList;

   if(ImageList != 0)
   {
      for(int i = 0; i<ImageList->Count; ++i) Proc(IntToStr(i));
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexProperty::ListMeasureWidth(const AnsiString Value,
                                                      Graphics::TCanvas* ACanvas,
                                                      int& AWidth)
{
   TCustomImageList* ImageList = RemoteImageList;

   if(ImageList != 0)
   {
      if(ImageList->Width < MaxImageWidth) AWidth += ImageList->Width + Border*2;
      else AWidth += MaxImageWidth + Border*2;
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexProperty::ListMeasureHeight(const AnsiString Value,
                                                       Graphics::TCanvas* ACanvas,
                                                       int& AHeight)
{
   TCustomImageList* ImageList = RemoteImageList;

   if(ImageList != 0)
   {
      if( ImageList->Height < MaxImageHeight && ImageList->Height > AHeight)
      {
         AHeight = ImageList->Height + Border*2;
      }
      else if(ImageList->Height > AHeight) AHeight = MaxImageHeight + Border*2;
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexProperty::ListDrawValue(const AnsiString Value,
                                                   Graphics::TCanvas* ACanvas,
                                                   const Types::TRect& ARect,
                                                   bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      TCustomImageList* ImageList = RemoteImageList;

      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(ImageList != 0 && Value != "")
      {
         int ImageWidth = ImageList->Width;
         int ImageHeight = ImageList->Height;

         if(ImageWidth > MaxImageWidth || ImageHeight > MaxImageHeight)
         {
            std::auto_ptr<Graphics::TBitmap> Image(new Graphics::TBitmap());

            // Set the Bitmap's width and height to that of the image
            Image->Width  = ImageWidth;
            Image->Height = ImageHeight;

            // Draw the image from the ImageList onto the Bitmap's Canvas
            ImageList->Draw(Image->Canvas, 0, 0, StrToInt(Value), true);

            if(ImageWidth > MaxImageWidth) ImageWidth  = MaxImageWidth;
            if(ImageHeight > MaxImageHeight) ImageHeight = MaxImageHeight;

            // Define the area to draw the image into
            TRect ImageRect = Rect( ARect.Left + Border,
                                    ARect.Top + Border,
                                    ARect.Left + Border + ImageWidth,
                                    ARect.Top + Border + ImageHeight );

            // Draw the image onto the canvas using StretchDraw
            ACanvas->StretchDraw(ImageRect, Image.get());
         }
         else
         {
            // Draw the image directly onto the canvas from the ImageList leaving
            // a 2 pixel border
            ImageList->Draw( ACanvas,
                             ARect.Left + Border,
                             ARect.Top + Border,
                             StrToInt(Value),
                             true );
         }

         ValueRect = Rect( ARect.Left + ImageWidth + Border*2,
                           ARect.Top,
                           ARect.Right,
                           ARect.Bottom );
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      ACanvas->TextRect( ARect,
                         ARect.Left+1,
                         ARect.Top+1,
                         GetValue() );
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexProperty::PropDrawValue(Graphics::TCanvas* ACanvas,
                                                   const Types::TRect& ARect,
                                                           bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      TCustomImageList* ImageList = RemoteImageList;

      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(ImageList != 0
         && GetVisualValue() != ""
         && GetVisualValue().ToInt() >= 0
         && GetVisualValue().ToInt() < ImageList->Count)
      {
         int ImageWidth = ImageList->Width;
         int ImageHeight = ImageList->Height;

         // Calculate the MaxSide as we want a square to display our image
         int CurrentMaxSide = ARect.Bottom - ARect.Top - 2;

         if(ImageWidth > CurrentMaxSide || ImageHeight > CurrentMaxSide)
         {
            std::auto_ptr<Graphics::TBitmap> Image(new Graphics::TBitmap());

            // Set the Bitmap's width and height to that of the image
            Image->Width  = ImageWidth;
            Image->Height = ImageHeight;

            // Draw the image from the ImageList onto the Bitmap's Canvas
            ImageList->Draw(Image->Canvas, 0, 0, Value.ToInt(), true);

            if(ImageWidth > CurrentMaxSide) ImageWidth  = CurrentMaxSide;
            if(ImageHeight > CurrentMaxSide) ImageHeight = CurrentMaxSide;

            // Define the area to draw the image into
            TRect ImageRect = Rect( ARect.Left + 2,
                                    ARect.Top + 1,
                                    ARect.Left + 2 + ImageWidth,
                                    ARect.Top + 1 + ImageHeight );

            // Draw the image onto the canvas using StretchDraw
            ACanvas->StretchDraw(ImageRect, Image.get());
         }
         else
         {
            // Draw the image directly onto the canvas from the ImageList leaving
            // a 1 pixel border
            ImageList->Draw( ACanvas,
                             ARect.Left + 2,
                             ARect.Top + 1,
                             Value.ToInt(),
                             true );
         }

         ValueRect = Rect( ARect.Left + ImageWidth + 2,
                           ARect.Top,
                           ARect.Right,
                           ARect.Bottom );
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      ACanvas->TextRect( ARect,
                         ARect.Left+1,
                         ARect.Top+1,
                         GetValue() );
   }
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                   TComponentAccess                                       //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

class TComponentAccess : public TComponent
{
public:
   DYNAMIC Classes::TPersistent* __fastcall GetOwner(void);
};

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                          TPersistentDerivedImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

__fastcall TPersistentDerivedImageIndexProperty::TPersistentDerivedImageIndexProperty
                                                 (const _di_IDesigner ADesigner,
                                                  int APropCount)
                                                  : TImageIndexProperty(ADesigner,
                                                                        APropCount)
{
   FParentImageListName = "Images";
}
//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TPersistentDerivedImageIndexProperty::GetImageList(void)
{
   TPersistent* Parent = static_cast<TComponentAccess*>(GetComponent(0))->GetOwner();

   while( Parent != 0 && !dynamic_cast<TComponent*>(Parent) )
   {
      Parent = static_cast<TComponentAccess*>(Parent)->GetOwner();
   }

   if(Parent == 0) return 0;

   PPropInfo PropInfo = Typinfo::GetPropInfo(static_cast<PTypeInfo>(Parent->ClassInfo()),
                                             FParentImageListName);

   if(PropInfo == 0) return 0;

   return ( reinterpret_cast<TCustomImageList*>(Typinfo::GetOrdProp(Parent, PropInfo)) );
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                          TComponentDerivedImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TComponentDerivedImageIndexProperty::GetImageList(void)
{
   TComponent* Parent = static_cast<TComponentAccess*>(GetComponent(0))->GetParentComponent();

   if(Parent == 0) return 0;

   PPropInfo PropInfo = Typinfo::GetPropInfo(static_cast<PTypeInfo>(Parent->ClassInfo()),
                                             FParentImageListName);

   if(PropInfo == 0) return 0;

   return ( reinterpret_cast<TCustomImageList*>(Typinfo::GetOrdProp(Parent, PropInfo)) );
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TMenuItemImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TMenuItemImageIndexProperty::GetImageList(void)
{
   TMenuItem* Component = dynamic_cast<TMenuItem*>(GetComponent(0));

   if(Component)
   {
      return Component->GetImageList();
   }
   return 0;
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TTabSheetImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TTabSheetImageIndexProperty::GetImageList(void)
{
   TTabSheet* Component = dynamic_cast<TTabSheet*>(GetComponent(0));

   if(Component)
   {
      return Component->PageControl->Images;
   }
   return 0;
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TToolButtonImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TToolButtonImageIndexProperty::GetImageList(void)
{
   TToolButton* Component = dynamic_cast<TToolButton*>(GetComponent(0));

   if(Component)
   {
      TToolBar* ParentToolBar = dynamic_cast<TToolBar*>(Component->Parent);

      if(ParentToolBar)
      {
         if(Component->Enabled) return ParentToolBar->Images;
         else return ParentToolBar->DisabledImages;
      }
   }
   return 0;
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TCoolBandImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TCoolBandImageIndexProperty::GetImageList(void)
{
   TCoolBand* Component = dynamic_cast<TCoolBand*>(GetComponent(0));

   if(Component)
   {
      TCoolBands* CoolBands = dynamic_cast<TCoolBands*>(Component->Collection);

      if(CoolBands)
      {
         return CoolBands->CoolBar->Images;
      }
   }
   return 0;
}
//------------------------------------------------------------------------------------------//
/*
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TListColumnImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

class TCustomListViewAccess : public TCustomListView
{
public:
   __property SmallImages;
};

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TListColumnImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TListColumnImageIndexProperty::GetImageList(void)
{
   TListColumn* Component = dynamic_cast<TListColumn*>(GetComponent(0));

   if(Component)
   {
      TListColumns* ListColumns = dynamic_cast<TListColumns*>(Component->Collection);

      if(ListColumns)
      {
         TCustomListView* CustomListView = dynamic_cast<TCustomListView*>(ListColumns->Owner);

         if(CustomListView)
         {
            return static_cast<TCustomListViewAccess*>(CustomListView)->SmallImages;
         }
      }
   }
   return 0;
}
//------------------------------------------------------------------------------------------//
*/

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TListColumnImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

__fastcall TListColumnImageIndexProperty::TListColumnImageIndexProperty
                                          (const _di_IDesigner ADesigner,
                                           int APropCount)
                                           : TPersistentDerivedImageIndexProperty(ADesigner,
                                                                                  APropCount)
{
   FParentImageListName = "SmallImages";
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TCustomActionImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TCustomActionImageIndexProperty::GetImageList(void)
{
   TCustomAction* Component = dynamic_cast<TCustomAction*>(GetComponent(0));

   if(Component)
   {
      if(Component->ActionList != 0)
      {
         return Component->ActionList->Images;
      }
   }
   return 0;
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                THeaderSectionsAccessImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

class THeaderSectionsAccess : public THeaderSections
{
public:
   DYNAMIC Classes::TPersistent* __fastcall GetOwner(void);
};

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            THeaderSectionImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall THeaderSectionImageIndexProperty::GetImageList(void)
{
   THeaderSection* Component = dynamic_cast<THeaderSection*>(GetComponent(0));

   if(Component)
   {
      THeaderSections* HeaderSections = dynamic_cast<THeaderSections*>(Component->Collection);

      if(HeaderSections)
      {
         TPersistent* Owner = static_cast<THeaderSectionsAccess*>(HeaderSections)->GetOwner();
         THeaderControl* HeaderControl = dynamic_cast<THeaderControl*>(Owner);

         if(HeaderControl)
         {
            return HeaderControl->Images;
         }
      }
   }
   return 0;
}
//------------------------------------------------------------------------------------------//




//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TDisplayCursorProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

void __fastcall TDisplayCursorProperty::PropDrawValue(Graphics::TCanvas* ACanvas,
                                                      const Types::TRect& ARect,
                                                      bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(GetVisualValue() != "")
      {
         int CursorIndex;

         // Get the Cursor Index
         if(!IdentToCursor(GetVisualValue(), CursorIndex))
         {
            CursorIndex = GetVisualValue().ToInt();
         }

         HANDLE CursorHandle = Screen->Cursors[CursorIndex];

         if(CursorHandle != 0)
         {
            // Calculate the MaxSide as we want a square to display our image
            int CurrentMaxSide = ARect.Bottom - ARect.Top - 2;

            DrawIconEx( ACanvas->Handle,
                        ARect.Left + 1,
                        ARect.Top + 1,
                        CursorHandle,
                        CurrentMaxSide,
                        CurrentMaxSide,
                        0,
                        0,
                        DI_NORMAL );

            ValueRect = Rect( ARect.Left + CurrentMaxSide + 2,
                              ARect.Top,
                              ARect.Right,
                              ARect.Bottom );
         }
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      ACanvas->TextRect( ARect,
                         ARect.Left+1,
                         ARect.Top+1,
                         GetValue() );
   }
}
//------------------------------------------------------------------------------------------//




//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                  TUnsignedProperty                                       //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

AnsiString __fastcall TUnsignedProperty::GetValue()
{
   return IntToStr(GetOrdValue());
}
//------------------------------------------------------------------------------------------//

void __fastcall TUnsignedProperty::SetValue(const AnsiString Value)
{
   SetOrdValue(StrToInt(Value));
}
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                 TCharPropertyEditor                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

AnsiString __fastcall TCharPropertyEditor::GetValue()
{
   char ch = static_cast<char>(GetOrdValue());
   if( (ch >= 33) && (ch <= 127) ) return static_cast<char>(ch);
   else return AnsiString().sprintf("#%d", static_cast<int>(ch));
}

//------------------------------------------------------------------------------------------//

void __fastcall TCharPropertyEditor::SetValue(const AnsiString Value)
{
   int charValue;
   if(Value.Length() == 0) charValue = 0;
   else
   {
      if(Value.Length() == 1) charValue = static_cast<int>(Value[1]);
      else
      {
         if(Value[1] == '#') charValue = StrToInt(Value.SubString(2, Value.Length()-1));
         else throw new EPropertyError("Invalid property value"); // Changed from an undefined constant
      }
   }
   if( charValue < -128 || charValue > 255 )
   {
      throw EPropertyError(Consts_SOutOfRange, ARRAYOFCONST((-128, 255)));
   }

   SetOrdValue(static_cast<char>(charValue));
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                 TSignedCharProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

AnsiString __fastcall TSignedCharProperty::GetValue()
{
   signed char ch = static_cast<signed char>(GetOrdValue());
   if( (ch >= 33) && (ch <= 127) ) return static_cast<char>(ch); 
   else return AnsiString().sprintf("#%d", static_cast<int>(ch));
}

//------------------------------------------------------------------------------------------//

void __fastcall TSignedCharProperty::SetValue(const AnsiString Value)
{
   int charValue;
   if(Value.Length() == 0) charValue = 0;
   else
   {
      if(Value.Length() == 1) charValue = static_cast<int>(Value[1]);
      else
      {
         if(Value[1] == '#') charValue = StrToInt(Value.SubString(2, Value.Length()-1));
         else throw new EPropertyError("Invalid property value"); // Added new and changed constant to message MSC
      }
   }
   if( charValue < -128 || charValue > 127 )
   {
      throw EPropertyError(Consts_SOutOfRange, ARRAYOFCONST((-128, 127)));
   }
   SetOrdValue(charValue);
}
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                 TUnsignedCharProperty                                    //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

AnsiString __fastcall TUnsignedCharProperty::GetValue()
{
   unsigned char ch = static_cast<unsigned char>(GetOrdValue());
   if( ch >= 33 && ch <= 127 ) return static_cast<char>(ch);
   else return AnsiString().sprintf("#%d", static_cast<int>(ch));
}

//------------------------------------------------------------------------------------------//

void __fastcall TUnsignedCharProperty::SetValue(const AnsiString Value)
{
   int charValue;
   if(Value.Length() == 0) charValue = 0;
   else
   {
      if(Value.Length() == 1) charValue = static_cast<int>(Value[1]);
      else
      {
         if(Value[1] == '#') charValue = StrToInt(Value.SubString(2, Value.Length()-1));
         else throw new EPropertyError("Invalid property value");
      }
   }
   if( charValue < 0 || charValue > 255 )
   {
      throw EPropertyError(Consts_SOutOfRange, ARRAYOFCONST((0, 255)));
   }
   SetOrdValue(charValue);
}
//------------------------------------------------------------------------------------------//

/*
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//
//                            TMenuItemImageIndexProperty
//
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TMenuItemImageIndexProperty::GetParentImageList(void)
{
   TMenuItem* Component = dynamic_cast<TMenuItem*>(GetComponent(0));

   if(Component)
   {

      TMenuItem* ParentMenuItem = Component->Parent;

      while(ParentMenuItem != 0 && ParentMenuItem->SubMenuImages == 0)
      {
         ParentMenuItem = ParentMenuItem->Parent;
      }

      if(ParentMenuItem != 0) return ParentMenuItem->SubMenuImages;
      else
      {
         TMenu* ParentMenu = Component->GetParentMenu();
         if(ParentMenu != 0) return ParentMenu->Images;
      }

   }

   return 0;
}
//------------------------------------------------------------------------------------------//

TPropertyAttributes __fastcall TMenuItemImageIndexProperty::GetAttributes(void)
{
   return (inherited::GetAttributes() << paValueList >> paMultiSelect);
}
//------------------------------------------------------------------------------------------//

void __fastcall TMenuItemImageIndexProperty::GetValues(Classes::TGetStrProc Proc)
{
   TCustomImageList* ImageList = ParentImageList;

   if(ImageList != 0)
   {
      for(int i = 0; i<ImageList->Count; ++i) Proc(IntToStr(i));
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TMenuItemImageIndexProperty::ListMeasureWidth(const AnsiString Value,
                                                              Graphics::TCanvas* ACanvas,
                                                              int& AWidth)
{
   TCustomImageList* ImageList = ParentImageList;

   if(ImageList != 0)
   {
      if(ImageList->Width < MaxImageWidth) AWidth += ImageList->Width + Border*2;
      else AWidth += MaxImageWidth + Border*2;
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TMenuItemImageIndexProperty::ListMeasureHeight(const AnsiString Value,
                                                               Graphics::TCanvas* ACanvas,
                                                               int& AHeight)
{
   TCustomImageList* ImageList = ParentImageList;

   if(ImageList != 0)
   {
      if( ImageList->Height < MaxImageHeight && ImageList->Height > AHeight)
      {
         AHeight = ImageList->Height + Border*2;
      }
      else if(ImageList->Height > AHeight) AHeight = MaxImageHeight + Border*2;
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TMenuItemImageIndexProperty::ListDrawValue(const AnsiString Value,
                                                           Graphics::TCanvas* ACanvas,
                                                           const Types::TRect& ARect,
                                                           bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      TCustomImageList* ImageList = ParentImageList;

      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(ImageList != 0 && Value != "")
      {
         int ImageWidth = ImageList->Width;
         int ImageHeight = ImageList->Height;

         // Calculate the MaxHeight in case it is not what we expect, i.e. 64
         // ( 4 is subtracted to allow for the 2 pixel border top and bottom, Border = 2 )
         //int CurrentMaxHeight = ARect.Bottom - ARect.Top - Border*2;

         if(ImageWidth > MaxImageWidth || ImageHeight > MaxImageHeight)
         {
            Graphics::TBitmap* Image = new Graphics::TBitmap();

            try
            {
               // Set the Bitmap's width and height to that of the image
               Image->Width  = ImageWidth;
               Image->Height = ImageHeight;

               // Draw the image from the ImageList onto the Bitmap's Canvas
               ImageList->Draw(Image->Canvas, 0, 0, StrToInt(Value), true);

               if(ImageWidth > MaxImageWidth) ImageWidth  = MaxImageWidth;
               if(ImageHeight > MaxImageHeight) ImageHeight = MaxImageHeight;

               // Define the area to draw the image into
               TRect ImageRect = Rect( ARect.Left + Border,
                                       ARect.Top + Border,
                                       ARect.Left + Border + ImageWidth,
                                       ARect.Top + Border + ImageHeight );

               // Draw the image onto the canvas using StretchDraw
               ACanvas->StretchDraw(ImageRect, Image);
            }
            __finally
            {
               delete Image;
            }
         }
         else
         {
            // Draw the image directly onto the canvas from the ImageList leaving
            // a 2 pixel border
            ImageList->Draw( ACanvas,
                             ARect.Left + Border,
                             ARect.Top + Border,
                             StrToInt(Value),
                             true );
         }

         ValueRect = Rect( ARect.Left + ImageWidth + Border*2,
                           ARect.Top,
                           ARect.Right,
                           ARect.Bottom );
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      inherited::ListDrawValue(Value, ACanvas, ValueRect, ASelected);
   }
}
//------------------------------------------------------------------------------------------//

void __fastcall TMenuItemImageIndexProperty::PropDrawValue(Graphics::TCanvas* ACanvas,
                                                           const Types::TRect& ARect,
                                                           bool ASelected)
{
   TRect ValueRect = ARect;

   try
   {
      TCustomImageList* ImageList = ParentImageList;

      // Clear the canvas using the current pen and brush
      ACanvas->FillRect(ARect);

      if(ImageList != 0
         && GetVisualValue() != ""
         && GetVisualValue().ToInt() >= 0
         && GetVisualValue().ToInt() < ImageList->Count)
      {
         int ImageWidth = ImageList->Width;
         int ImageHeight = ImageList->Height;

         // Calculate the MaxSide as we want a square to display our image
         int CurrentMaxSide = ARect.Bottom - ARect.Top - 2;

         if(ImageWidth > CurrentMaxSide || ImageHeight > CurrentMaxSide)
         {
            std::auto_ptr<Graphics::TBitmap> Image(new Graphics::TBitmap());

            // Set the Bitmap's width and height to that of the image
            Image->Width  = ImageWidth;
            Image->Height = ImageHeight;

            // Draw the image from the ImageList onto the Bitmap's Canvas
            ImageList->Draw(Image->Canvas, 0, 0, StrToInt(Value), true);

            if(ImageWidth > CurrentMaxSide) ImageWidth  = CurrentMaxSide;
            if(ImageHeight > CurrentMaxSide) ImageHeight = CurrentMaxSide;

            // Define the area to draw the image into
            TRect ImageRect = Rect( ARect.Left + 2,
                                    ARect.Top + 1,
                                    ARect.Left + 2 + ImageWidth,
                                    ARect.Top + 1 + ImageHeight );

            // Draw the image onto the canvas using StretchDraw
            ACanvas->StretchDraw(ImageRect, Image.get());
         }
         else
         {
            // Draw the image directly onto the canvas from the ImageList leaving
            // a 1 pixel border
            ImageList->Draw( ACanvas,
                             ARect.Left + 2,
                             ARect.Top + 1,
                             StrToInt(Value),
                             true );
         }

         ValueRect = Rect( ARect.Left + ImageWidth + 2,
                           ARect.Top,
                           ARect.Right,
                           ARect.Bottom );
      }
   }
   __finally
   {
      // Whether or not we successfully draw the image we must draw the text
      inherited::PropDrawValue(ACanvas, ValueRect, ASelected);
   }
}
//------------------------------------------------------------------------------------------//
*/


