//------------------------------------------------------------------------------------------//
#ifndef PropertyEditorsH
#define PropertyEditorsH
//------------------------------------------------------------------------------------------//
#include "DesignIntf.hpp"
#include "VCLEditors.hpp"
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TShapeTypePropertyEditor                                    //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TShapeTypePropertyEditor : public TEnumProperty
{
   typedef TEnumProperty inherited;

public:
   DYNAMIC void __fastcall ListMeasureWidth(const AnsiString Value,
                                            Graphics::TCanvas* ACanvas,
                                            int& AWidth);

   DYNAMIC void __fastcall ListMeasureHeight(const AnsiString Value,
                                             Graphics::TCanvas* ACanvas,
                                             int& AHeight);

   DYNAMIC void __fastcall ListDrawValue(const AnsiString Value,
                                         Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

   DYNAMIC void __fastcall PropDrawValue(Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TShapeTypePropertyEditor(const _di_IDesigner ADesigner,
                                                      int APropCount)
                                                      : TEnumProperty(ADesigner,
                                                                      APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TShapeTypePropertyEditor(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                               TDisplayFontNameProperty                                   //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TDisplayFontNameProperty : public TStringProperty
{
   typedef TStringProperty inherited;

public:
   virtual TPropertyAttributes __fastcall GetAttributes(void);

   virtual void __fastcall GetValues(Classes::TGetStrProc Proc);

   DYNAMIC void __fastcall ListMeasureWidth(const AnsiString Value,
                                            Graphics::TCanvas* ACanvas,
                                            int& AWidth);

   DYNAMIC void __fastcall ListMeasureHeight(const AnsiString Value,
                                             Graphics::TCanvas* ACanvas,
                                             int& AHeight);

   DYNAMIC void __fastcall ListDrawValue(const AnsiString Value,
                                         Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TDisplayFontNameProperty(const _di_IDesigner ADesigner,
                                                      int APropCount)
                                                      : TStringProperty(ADesigner,
                                                                        APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TDisplayFontNameProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TImageListPropertyEditor                                    //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TImageListPropertyEditor : public TComponentProperty
{
   typedef TComponentProperty inherited;

private:
   Graphics::TBitmap* ImageListPropertyImage;

public:
   DYNAMIC void __fastcall PropDrawName(Graphics::TCanvas* ACanvas,
                                        const Types::TRect& ARect,
                                        bool ASelected);

public:
   __fastcall virtual TImageListPropertyEditor(const _di_IDesigner ADesigner,
                                               int APropCount);

public:
   __fastcall virtual ~TImageListPropertyEditor(void);

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                 TImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TImageIndexProperty : public TIntegerProperty
{
   typedef TIntegerProperty inherited;

private:
   static const int Border = 2;
   static const int MaxImageWidth = 64;
   static const int MaxImageHeight = 64;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void) = 0;

public:
   virtual TPropertyAttributes __fastcall GetAttributes(void);
   virtual void __fastcall GetValues(Classes::TGetStrProc Proc);

   DYNAMIC void __fastcall ListMeasureWidth(const AnsiString Value,
                                            Graphics::TCanvas* ACanvas,
                                            int& AWidth);

   DYNAMIC void __fastcall ListMeasureHeight(const AnsiString Value,
                                             Graphics::TCanvas* ACanvas,
                                             int& AHeight);

   DYNAMIC void __fastcall ListDrawValue(const AnsiString Value,
                                         Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

   DYNAMIC void __fastcall PropDrawValue(Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

   // This is a read-only property to be used as the pointer to the component's image list
   __property Imglist::TCustomImageList* RemoteImageList = {read=GetImageList};

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TImageIndexProperty(const _di_IDesigner ADesigner,
                                                 int APropCount)
                                                 : TIntegerProperty(ADesigner,
                                                                    APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                             TPersistentDerivedImageIndexProperty                         //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TPersistentDerivedImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);
   AnsiString FParentImageListName;

protected:
   virtual __fastcall TPersistentDerivedImageIndexProperty
                             (const _di_IDesigner ADesigner,
                              int APropCount);

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TPersistentDerivedImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TComponentDerivedImageIndexProperty                         //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TComponentDerivedImageIndexProperty
              : public TPersistentDerivedImageIndexProperty
{
   typedef TPersistentDerivedImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TComponentDerivedImageIndexProperty
                             (const _di_IDesigner ADesigner,
                              int APropCount)
                              : TPersistentDerivedImageIndexProperty(ADesigner,
                                                                     APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TComponentDerivedImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TMenuItemImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TMenuItemImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TMenuItemImageIndexProperty(const _di_IDesigner ADesigner,
                                                         int APropCount)
                                                         : TImageIndexProperty(ADesigner,
                                                                               APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TMenuItemImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                 TTabSheetImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TTabSheetImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TTabSheetImageIndexProperty(const _di_IDesigner ADesigner,
                                                         int APropCount)
                                                         : TImageIndexProperty(ADesigner,
                                                                               APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TTabSheetImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TToolButtonImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TToolButtonImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TToolButtonImageIndexProperty(const _di_IDesigner ADesigner,
                                                           int APropCount)
                                                           : TImageIndexProperty(ADesigner,
                                                                                 APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TToolButtonImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TCoolBandImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TCoolBandImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TCoolBandImageIndexProperty(const _di_IDesigner ADesigner,
                                                         int APropCount)
                                                         : TImageIndexProperty(ADesigner,
                                                                               APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TCoolBandImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

/*
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TListColumnImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TListColumnImageIndexProperty : public TImageIndexProperty
{
   typedef TImageIndexProperty inherited;

protected:
   virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TListColumnImageIndexProperty(const _di_IDesigner ADesigner,
                                                           int APropCount)
                                                           : TImageIndexProperty(ADesigner,
                                                                                 APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TListColumnImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//
*/

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TListColumnImageIndexProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TListColumnImageIndexProperty : public TPersistentDerivedImageIndexProperty
{
   typedef TPersistentDerivedImageIndexProperty inherited;

protected:
   virtual __fastcall TListColumnImageIndexProperty(const _di_IDesigner ADesigner,
                                                    int APropCount);

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TListColumnImageIndexProperty(void)
   { }
   #pragma option pop
};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TCustomActionImageIndexProperty                             //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TCustomActionImageIndexProperty : public TImageIndexProperty
{
  typedef TImageIndexProperty inherited;

protected:
  virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
  #pragma option push -w-inl
  inline __fastcall virtual TCustomActionImageIndexProperty(const _di_IDesigner ADesigner,
                                                            int APropCount)
                                                            : TImageIndexProperty(ADesigner,
                                                                                  APropCount)
  { }
  #pragma option pop

public:
  #pragma option push -w-inl
  inline __fastcall virtual ~TCustomActionImageIndexProperty(void)
  { }
  #pragma option pop

};
//------------------------------------------------------------------------------------------//

// Listing 10.26 shows an implementation of a TIntegerProperty-derived
// THeaderSectionImageIndexProperty property editor. This similar to the TIntegerProperty-
// derived TMenuItemImageIndexProperty property editor commented out at the end of this file.
// The implementation shown here is the implementation actually used. It is part of the
// general solution to image index properties.

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              THeaderSectionImageIndexProperty                             //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE THeaderSectionImageIndexProperty : public TImageIndexProperty
{
  typedef TImageIndexProperty inherited;

protected:
  virtual Imglist::TCustomImageList* __fastcall GetImageList(void);

protected:
 #pragma option push -w-inl
 inline __fastcall virtual THeaderSectionImageIndexProperty(const _di_IDesigner ADesigner,
                                                            int APropCount)
                                                            : TImageIndexProperty(ADesigner,
                                                                                  APropCount)
 { }
 #pragma option pop

public:
 #pragma option push -w-inl
 inline __fastcall virtual ~THeaderSectionImageIndexProperty(void)
 { }
 #pragma option pop

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TDisplayCursorProperty                                      //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TDisplayCursorProperty : public TCursorProperty
{
   typedef TCursorProperty inherited;

public:
   DYNAMIC void __fastcall PropDrawValue(Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TDisplayCursorProperty(const _di_IDesigner ADesigner,
                                                    int APropCount)
                                                    : TCursorProperty(ADesigner,
                                                                      APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TDisplayCursorProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//



//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                  TUnsignedProperty                                       //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TUnsignedProperty : public TOrdinalProperty
{
   typedef TOrdinalProperty inherited;

public:
   virtual AnsiString __fastcall GetValue();
   virtual void __fastcall SetValue(const AnsiString Value);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TUnsignedProperty(const _di_IDesigner ADesigner,
                                               int APropCount)
                                               : TOrdinalProperty(ADesigner,
                                                                  APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TUnsignedProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//


//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TCharPropertyEditor                                       //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TCharPropertyEditor : public TCharProperty
{
   typedef TCharProperty inherited;

public:
   virtual AnsiString __fastcall GetValue();
   virtual void __fastcall SetValue(const AnsiString Value);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TCharPropertyEditor(const _di_IDesigner ADesigner,
                                                 int APropCount)
                                                 : TCharProperty(ADesigner,
                                                                 APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TCharPropertyEditor(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                  TSignedCharProperty                                     //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TSignedCharProperty : public TCharProperty
{
   typedef TCharProperty inherited;

public:
   virtual AnsiString __fastcall GetValue();
   virtual void __fastcall SetValue(const AnsiString Value);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TSignedCharProperty(const _di_IDesigner ADesigner,
                                                 int APropCount)
                                                 : TCharProperty(ADesigner,
                                                                 APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TSignedCharProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                                TUnsignedCharProperty                                     //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TUnsignedCharProperty : public TCharProperty
{
   typedef TCharProperty inherited;

public:
   virtual AnsiString __fastcall GetValue();
   virtual void __fastcall SetValue(const AnsiString Value);

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TUnsignedCharProperty(const _di_IDesigner ADesigner,
                                                   int APropCount)
                                                   : TCharProperty(ADesigner,
                                                                   APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TUnsignedCharProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//




/*

This is the code for Listing 10.24. It is not actually used in this package. The property
editor actually used derives from the TImageIndexProperty property editor - defined earlier
in this file. Both work, however as explained in Chapter 10 the previous approach is part of
a more general solution to image index property editors.

//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
//                                                                                          //
//                              TMenuItemImageIndexProperty                                 //
//                                                                                          //
//------------------------------------------------------------------------------------------//
//------------------------------------------------------------------------------------------//
class PACKAGE TMenuItemImageIndexProperty : public TIntegerProperty
{
   typedef TIntegerProperty inherited;

private:
   static const int Border = 2;
   static const int MaxImageWidth = 64;
   static const int MaxImageHeight = 64;

protected:
   Imglist::TCustomImageList* __fastcall GetParentImageList(void);

public:
   virtual TPropertyAttributes __fastcall GetAttributes(void);
   virtual void __fastcall GetValues(Classes::TGetStrProc Proc);

   DYNAMIC void __fastcall ListMeasureWidth(const AnsiString Value,
                                            Graphics::TCanvas* ACanvas,
                                            int& AWidth);

   DYNAMIC void __fastcall ListMeasureHeight(const AnsiString Value,
                                             Graphics::TCanvas* ACanvas,
                                             int& AHeight);

   DYNAMIC void __fastcall ListDrawValue(const AnsiString Value,
                                         Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

   DYNAMIC void __fastcall PropDrawValue(Graphics::TCanvas* ACanvas,
                                         const Types::TRect& ARect,
                                         bool ASelected);

   // This is a read-only property to be used as the pointer to the component's image list
   __property Imglist::TCustomImageList* ParentImageList = {read=GetParentImageList};

protected:
   #pragma option push -w-inl
   inline __fastcall virtual TMenuItemImageIndexProperty(const _di_IDesigner ADesigner,
                                                         int APropCount)
                                                         : TIntegerProperty(ADesigner,
                                                                            APropCount)
   { }
   #pragma option pop

public:
   #pragma option push -w-inl
   inline __fastcall virtual ~TMenuItemImageIndexProperty(void)
   { }
   #pragma option pop

};
//------------------------------------------------------------------------------------------//
*/


#endif


