//------------------------------------------------------------------------------------------//
#include <vcl.h>
#pragma hdrstop
//------------------------------------------------------------------------------------------//

#include "PropertyEditorsForNewAdditionalComponents.h"
#include "EnhancedImage.h"
#include <memory> // for std::auto_ptr

//------------------------------------------------------------------------------------------//
#pragma package(smart_init)
//------------------------------------------------------------------------------------------//

//------------------------------------------------------------------------------------------//
//                                                                                          //
//         TImageIndexPropertyEditor:: Member Functions                                     //
//                                                                                          //
//------------------------------------------------------------------------------------------//

Imglist::TCustomImageList* __fastcall TImageIndexPropertyEditor::GetComponentImageList(void)
{
   TEnhancedImage* Component = dynamic_cast<TEnhancedImage*>(GetComponent(0));

   if(Component)
   {
      return Component->ImageList;
   }

   return 0;
}

//------------------------------------------------------------------------------------------//

TPropertyAttributes __fastcall TImageIndexPropertyEditor::GetAttributes(void)
{
   return (inherited::GetAttributes() << paValueList >> paMultiSelect);
}

//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexPropertyEditor::GetValues(Classes::TGetStrProc Proc)
{
   TCustomImageList* ImageList = ComponentImageList;

   if(ImageList != 0)
   {
      for(int i = 0; i<ImageList->Count; ++i) Proc(IntToStr(i));
   }
}

//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexPropertyEditor::ListMeasureWidth(const AnsiString Value,
                                                            Graphics::TCanvas* ACanvas,
                                                            int& AWidth)
{
   TCustomImageList* ImageList = ComponentImageList;

   if(ImageList != 0)
   {
      if(ImageList->Width < MaxImageWidth) AWidth += ImageList->Width + Border*2;
      else AWidth += MaxImageWidth + Border*2;
   }
}

//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexPropertyEditor::ListMeasureHeight(const AnsiString Value,
                                                             Graphics::TCanvas* ACanvas,
                                                             int& AHeight)
{
   TCustomImageList* ImageList = ComponentImageList;

   if(ImageList != 0)
   {
      if( ImageList->Height < MaxImageHeight && ImageList->Height > AHeight)
      {
         AHeight = ImageList->Height + Border*2;
      }
      else if(ImageList->Height > AHeight) AHeight = MaxImageHeight + Border*2;
   }
}

//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexPropertyEditor::ListDrawValue(const AnsiString Value,
                                                         Graphics::TCanvas* ACanvas,
                                                         const TRect& ARect,
                                                         bool ASelected)
{
   TRect ValueRect = ARect;

   TCustomImageList* ImageList = ComponentImageList;

   // Clear the canvas using the current pen and brush
   ACanvas->FillRect(ARect);

   if(ImageList != 0 && Value != "")
   {
      int ImageWidth = ImageList->Width;
      int ImageHeight = ImageList->Height;

      // Calculate the MaxHeight in case it is not what we expect, i.e. 64
      // ( 4 is subtracted to allow for the 2 pixel border top and bottom, Border = 2 )
      //int CurrentMaxHeight = ARect.Bottom - ARect.Top - Border*2;

      if(ImageWidth > MaxImageWidth || ImageHeight > MaxImageHeight)
      {
         std::auto_ptr<Graphics::TBitmap> Image(new Graphics::TBitmap());

         // Set the Bitmap's width and height to that of the image
         Image->Width  = ImageWidth;
         Image->Height = ImageHeight;

         // Draw the image from the ImageList onto the Bitmap's Canvas
         ImageList->Draw(Image->Canvas, 0, 0, StrToInt(Value), true);

         if(ImageWidth > MaxImageWidth) ImageWidth  = MaxImageWidth;
         if(ImageHeight > MaxImageHeight) ImageHeight = MaxImageHeight;

         // Define the area to draw the image into
         TRect ImageRect = Rect( ARect.Left + Border,
                                 ARect.Top + Border,
                                 ARect.Left + Border + ImageWidth,
                                 ARect.Top + Border + ImageHeight );

         // Draw the image onto the canvas using StretchDraw
         ACanvas->StretchDraw(ImageRect, Image.get());
      }
      else
      {
         // Draw the image directly onto the canvas from the ImageList leaving
         // a 2 pixel border
         ImageList->Draw( ACanvas,
                          ARect.Left + Border,
                          ARect.Top + Border,
                          StrToInt(Value),
                          true );
      }

      ValueRect = Rect( ARect.Left + ImageWidth + Border*2,
                        ARect.Top,
                        ARect.Right,
                        ARect.Bottom );
   }

}

//------------------------------------------------------------------------------------------//

void __fastcall TImageIndexPropertyEditor::PropDrawValue(Graphics::TCanvas* ACanvas,
                                                         const TRect& ARect,
                                                         bool ASelected)
{
   TRect ValueRect = ARect;

   TCustomImageList* ImageList = ComponentImageList;

   // Clear the canvas using the current pen and brush
   ACanvas->FillRect(ARect);


   if(ImageList != 0
      && GetVisualValue() != ""
      && GetVisualValue().ToInt() >= 0
      && GetVisualValue().ToInt() < ImageList->Count)
   {
      int ImageWidth = ImageList->Width;
      int ImageHeight = ImageList->Height;

      // Calculate the MaxSide as we want a square to display our image
      int CurrentMaxSide = ARect.Bottom - ARect.Top - 2;

      if(ImageWidth > CurrentMaxSide || ImageHeight > CurrentMaxSide)
      {
         std::auto_ptr<Graphics::TBitmap> Image(new Graphics::TBitmap());

         // Set the Bitmap's width and height to that of the image
         Image->Width  = ImageWidth;
         Image->Height = ImageHeight;

         // Draw the image from the ImageList onto the Bitmap's Canvas
         ImageList->Draw(Image->Canvas, 0, 0, StrToInt(Value), true);

         if(ImageWidth > CurrentMaxSide) ImageWidth  = CurrentMaxSide;
         if(ImageHeight > CurrentMaxSide) ImageHeight = CurrentMaxSide;

         // Define the area to draw the image into
         TRect ImageRect = Rect( ARect.Left + 2,
                                 ARect.Top + 1,
                                 ARect.Left + 2 + ImageWidth,
                                 ARect.Top + 1 + ImageHeight );

         // Draw the image onto the canvas using StretchDraw
         ACanvas->StretchDraw(ImageRect, Image.get());
      }
      else
      {
         // Draw the image directly onto the canvas from the ImageList leaving
         // a 1 pixel border
         ImageList->Draw( ACanvas,
                          ARect.Left + 2,
                          ARect.Top + 1,
                          StrToInt(Value),
                          true );
      }

      ValueRect = Rect( ARect.Left + ImageWidth + 2,
                        ARect.Top,
                        ARect.Right,
                        ARect.Bottom );
   }
}

//------------------------------------------------------------------------------------------//

