/*---------------------------------------------------------------------------
  WinManUtilForm.cpp
        C++Builder Developer's Guide
        Chapter 14 - Windows Management Utility
        (c) Paul Gustavson, 2002
        Enumerates and Manages External Windows
---------------------------------------------------------------------------*/
#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "WinManUtilForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormWinMan *FormWinMan;
//---------------------------------------------------------------------------
__fastcall TFormWinMan::TFormWinMan(TComponent* Owner)
        : TForm(Owner)
{
    // define custom message
    WM_MOVE_A_WINDOW = RegisterWindowMessage("WM_MOVE_A_WINDOW");
    WM_STRETCH_A_WINDOW = RegisterWindowMessage("WM_STRETCH_A_WINDOW");
    WindowProc = MyWndProc;
}
//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonAddAppClick(TObject *Sender)
{
  // Browse for app / add to list
  if(OpenDialog1->Execute())   {
     char * filename = new char[OpenDialog1->FileName.Length()];
     sprintf(filename,"%s",OpenDialog1->FileName.c_str());
     ListBoxApps->Items->AddObject(ExtractFileName(OpenDialog1->FileName),
                                   (TObject*)(filename));
     ListBoxApps->Selected[ListBoxApps->Items->Count-1] = true;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonLaunchAppClick(TObject *Sender)
{
   if (ListBoxApps->ItemIndex >=  0) {
      char * data = (char*)ListBoxApps->Items->Objects[ListBoxApps->ItemIndex];
      AnsiString fullfilename  = AnsiString(data);

      EditFullFileName->Text = fullfilename;
      EditWHandle->Text = "0";
      winhandle = 0;
      processid =  0;

      STARTUPINFO  StartupInfo;
      ZeroMemory( &StartupInfo, sizeof(STARTUPINFO));
      PROCESS_INFORMATION ProcessInfo;
      StartupInfo.cb = sizeof(STARTUPINFO);
      if(CreateProcess(fullfilename.c_str(),     // Windows System Service Call
          NULL, //parameters,
          NULL,
          NULL,
          TRUE,
          NORMAL_PRIORITY_CLASS,
          NULL,
          NULL,
          &StartupInfo,
          &ProcessInfo))
      {
          // We must close the handles returned in ProcessInfo. We can
          // close the handle at any time, might as well close it now
          CloseHandle(ProcessInfo.hProcess);     // Windows System Service Call
          CloseHandle(ProcessInfo.hThread);      // Windows System Service Call
          processid =  (unsigned long)ProcessInfo.dwProcessId;
      }

     if (processid != 0)
          TimerGetHandle->Enabled = true; // use timer to get handle
     SetForegroundWindow(Handle);
   }
}

//---------------------------------------------------------------------------
// This code returns the process id which aids in determining the windows handle
unsigned long TFormWinMan::RunProgramEx (char* file, char* parameters)
{
    STARTUPINFO  StartupInfo;
    ZeroMemory( &StartupInfo, sizeof(STARTUPINFO));
    PROCESS_INFORMATION ProcessInfo;
    StartupInfo.cb = sizeof(STARTUPINFO);
    if(CreateProcess(file,       // Windows System Service Call
        parameters,
        NULL,
        NULL,
        TRUE,
        NORMAL_PRIORITY_CLASS,
        NULL,
        NULL,
        &StartupInfo,
        &ProcessInfo))
    {
        // We must close the handles returned in ProcessInfo. We can
        // close the handle at any time, might as well close it now
        CloseHandle(ProcessInfo.hProcess);     // Windows System Service Call
        CloseHandle(ProcessInfo.hThread);      // Windows System Service Call
        return (unsigned long)ProcessInfo.dwProcessId;
    }
    return 0;
}


//---------------------------------------------------------------------------
void __fastcall TFormWinMan::TimerGetHandleTimer(TObject *Sender)
{
   if (processid != 0)
   {
       winhandle = LookForWindowHandle(processid);
       if (winhandle != NULL)
       {
            TimerGetHandle->Enabled = false;
            UpdateAppInfo(winhandle);
       }
   }
}

//---------------------------------------------------------------------------
HWND TFormWinMan::LookForWindowHandle(unsigned long processid)
{
    if (!EnumWindows((WNDENUMPROC)GetWinHandle_Specific,processid))
        return swProcess;
    else
        return 0;
}

//---------------------------------------------------------------------------
BOOL CALLBACK TFormWinMan::GetWinHandle_Specific(HWND hwnd, unsigned long hproc)
{
    unsigned long dwProcessId; 	// address of variable for process identifier
    GetWindowThreadProcessId(
        hwnd,
        &dwProcessId); 	// address of variable for process identifier
    if (dwProcessId != hproc)  return true;    // keep enumerating

    char windowtitle[80];
    char classname[80];

    GetWindowText( hwnd, windowtitle, 80);
    if (windowtitle[0] == NULL) return true;

    GetClassName(hwnd, classname, 80);
    int ptr = strcmp(classname, "TApplication");
    if (ptr == 0) return true;

    FormWinMan->swProcess=hwnd;
    return false; //stop enumeration
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonEnumWindowsClick(TObject *Sender)
{
    TCursor Save_Cursor = Screen->Cursor;

    Screen->Cursor = crHourGlass;    // hourglass cursor

    try
    {
        TreeView1->Items->Clear();                    // clear tree
        EnumWindows((WNDENUMPROC)GetWinHandle_All,0); // enumerate all windows
        TreeView1->AlphaSort();   // sort tree which contains enum results
    }
    __finally
    {
      Screen->Cursor = Save_Cursor; // restore cursor
    }
}

//---------------------------------------------------------------------------

BOOL CALLBACK TFormWinMan::GetWinHandle_All(HWND hwnd, unsigned long hproc)
{
 int correlation = FormWinMan->GetCorrelationChoice();
 FormWinMan->ProcessHandleInformation(hwnd,correlation);

 if (hwnd != NULL) return true;  // keep going (we want them all)
 return false; //stop enumeration
}

//---------------------------------------------------------------------------

TTreeNode* __fastcall TFormWinMan::ProcessHandleInformation(HWND H_Window,
                                              unsigned int correlation_type)
{
    // get the handle of the application instance
    HINSTANCE  hAppInstance = (HINSTANCE)GetWindowLong(H_Window,GWL_HINSTANCE);
    unsigned long dwProcessId = 0;

    // find out who created window
    unsigned long tempID = GetWindowThreadProcessId(
           H_Window,
           &dwProcessId); 	// address of variable for process identifier

    int length = GetWindowTextLength(H_Window);    // get the length
    if ((CheckBoxIgnoreWindowsNoTitles->Checked) && (length == 0))
        return NULL;

    char classname[80];
    char windowtitle[80];

    GetWindowText(H_Window, windowtitle, 80);
    GetClassName(H_Window, classname, 80);
    HWND hwndParent = (HWND)GetWindowLong(H_Window,GWL_HWNDPARENT);

    if ((!CheckBoxIncludeParent->Checked) && (hwndParent == 0))
      return NULL;

    TAppInfo *appinfo = new TAppInfo();
    appinfo->DeviceContext      = GetWindowDC(H_Window);
    appinfo->Title              = AnsiString(windowtitle);
    appinfo->Class              = AnsiString(classname);
    appinfo->WindowHandle       = H_Window;
    appinfo->ParentWindowHandle = hwndParent;
    appinfo->InstanceHandle     = (int)hAppInstance;
    appinfo->ProcessID          = dwProcessId;

    TTreeNode* current_node = NULL;

    // let's double check to make sure we have a node for the parent.
    if ((CheckBoxIncludeParent->Checked) && (appinfo->ParentWindowHandle != 0))
    {
        TTreeNode* parent_node = FormWinMan->GetNode_AppInfoValue(
                  FormWinMan->TreeView1->Items,
                  NULL,
                  WINHANDLE,
                  (unsigned long)appinfo->ParentWindowHandle);

        if (!parent_node)  // need to create node with parent info
             current_node = ProcessHandleInformation(      // recurse
                        appinfo->ParentWindowHandle,
                        correlation_type);
    }

    AnsiString treetext;
    switch (correlation_type)
    {
      case PROCESS :
          current_node =
                  FormWinMan->GetNode_AppInfoValue(
                                    FormWinMan->TreeView1->Items,
                                    NULL,
                                    PROCESSID,
                                    (unsigned long)appinfo->ProcessID);

          treetext = "PID=" + AnsiString(appinfo->ProcessID) + " : ";
          break;
      case INSTANCE :
          current_node =
                   FormWinMan->GetNode_AppInfoValue(
                                    FormWinMan->TreeView1->Items,
                                    NULL,
                                    INSTANCEHANDLE,
                                    (unsigned long)appinfo->InstanceHandle);
          treetext = "INST=" + AnsiString(appinfo->InstanceHandle) + " : ";
          break;

      case WINDOW :
          current_node =
                   FormWinMan->GetNode_AppInfoValue(
                                    FormWinMan->TreeView1->Items,
                                    NULL,
                                    WINHANDLE,
                                    (unsigned long)appinfo->ParentWindowHandle);
          treetext = "HWND=" + AnsiString((unsigned int)appinfo->WindowHandle) +
                     " : ";
          break;
      default :
              if (CheckBoxProcessRootNode->Checked)
                      current_node = FormWinMan->GetNode_Process(
                                FormWinMan->TreeView1->Items,
                                appinfo->ProcessID);
              treetext = "";
    }

    TTreeNode* node = NULL;
    if (CheckBoxProcessRootNode->Checked) // user wants process IDs at 1st level
    {
       TAppInfo *processinfo = new TAppInfo();
       processinfo->ProcessID = appinfo->ProcessID;
       // make sure we have ProcessRootNode for this WindowHandle
       node = FormWinMan->GetNode_Process( FormWinMan->TreeView1->Items,
                                           appinfo->ProcessID);
       if (!node) {
          // need to create Process Node
          AnsiString temptext = "PID=" + AnsiString(appinfo->ProcessID);
          node = FormWinMan->TreeView1->Items->AddObject(NULL,
                                                        temptext,
                                                        processinfo);
          current_node = node;    // reset current_node
       }
    }

    node = FormWinMan->GetNode_AppInfoValue(FormWinMan->TreeView1->Items, NULL,
                                         WINHANDLE,
                                         (unsigned long)appinfo->WindowHandle);
    if (!node)   // make sure we don't already have the WindowHandle
    {
        if (!current_node)  // parent node
        {
            if (FormWinMan->TreeView1->Items->Count > 0)
                  current_node = FormWinMan->TreeView1->Items->Item[0];

            node = FormWinMan->TreeView1->Items->AddObject(current_node,
                      AnsiString(treetext +
                            AnsiString(windowtitle) +  " (" +
                            AnsiString(classname) + ")"),appinfo);
        }
        else // child node
        {
            node = FormWinMan->TreeView1->Items->AddChildObject(
                          current_node,
                          AnsiString(treetext +
                                  AnsiString(windowtitle) + " (" +
                                  AnsiString(classname) + ")"),appinfo);
        }
    }
    return node; // make sure we pass back the node
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::UpdateAppInfo(HWND winhandle)
{
    EditWHandle->Text = (AnsiString)(int)winhandle;

    int length = GetWindowTextLength(winhandle);
    char * title = new char[length+1];
    title[length] = 0;
    GetWindowText(winhandle,title,length+1);
    EditTitle->Text = AnsiString(title);

    char classname[80];
    GetClassName(winhandle, classname, 80);
    EditClass->Text = AnsiString(classname);

    RECT Rect;
    GetWindowRect(winhandle,&Rect);    // get the size for the current window
    EditLeft->Text = AnsiString((unsigned int)Rect.left);
    EditTop->Text = AnsiString((unsigned int)Rect.top);
    EditWidth->Text = AnsiString((unsigned int)(Rect.right - Rect.left));
    EditHeight->Text = AnsiString((unsigned int)(Rect.bottom - Rect.top));

    unsigned long dwProcessId = 0; //(unsigned long) GetWindowLong(H_Window,GWL_ID);
    // find out who created window
    unsigned long tempID = GetWindowThreadProcessId(
           winhandle,
           &dwProcessId); 	// address of variable for process identifier
    EditProcessID->Text = AnsiString(dwProcessId);

    // get the handle of the application instance
    HINSTANCE  hAppInstance = (HINSTANCE)GetWindowLong(winhandle,GWL_HINSTANCE);
    EditIHandle->Text = AnsiString((unsigned int)hAppInstance);

    HWND ParentWindowHandle= (HWND)GetWindowLong(winhandle,GWL_HWNDPARENT);
    EditPWHandle->Text = AnsiString((unsigned int)ParentWindowHandle);
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::Align(HWND winhandle,EAlign align)
{
    RECT avail;
    avail.top    = 0;
    avail.bottom = GetSystemMetrics(SM_CYFULLSCREEN);
    avail.left   = 0;
    avail.right  = GetSystemMetrics(SM_CXFULLSCREEN);

    RECT temp = avail;

    RECT rect;
    int height,width;

    //temp = avail;
    GetWindowRect(winhandle,&rect);  // get the size for the current window
    width = avail.right - avail.left;
    height = (rect.bottom - rect.top);

    if ((align == TOP))
    {
        if (width < 0) width = 0;
        if (height > (avail.bottom - avail.top))
                height = avail.bottom - avail.top;

        if (MoveWindow(winhandle,temp.left,temp.top,width,height,TRUE))
                avail.top = avail.top + height;
    }  // Top
    else if ((align == BOTTOM))
    {
        if (width < 0) width = 0;
        if (height > (avail.bottom - avail.top))
                height = avail.bottom - avail.top;

        temp.top = avail.bottom - height;
        if (temp.top < avail.top)
        {
                temp.top = avail.top;
                height = 0;
        }

        if (MoveWindow(winhandle,temp.left,temp.top,width,height,TRUE))
                avail.bottom = temp.top;
    }  // Bottom

    width = rect.right - rect.left;
    height = (avail.bottom - avail.top);
    if ((align == LEFT))
    {
        if (width > (avail.right - avail.left))
                width = avail.right - avail.left;
        if (height < 0) height = 0;

        if (MoveWindow(winhandle,temp.left,temp.top,width,height,TRUE))
                avail.left = avail.left + width;
    }  // Left
    else if ((align == RIGHT))
    {
        if (width > (avail.right - avail.left))
                width = avail.right - avail.left;
        if (height < 0) height = 0;
        temp.left = avail.right - width;
        if (temp.left < avail.left)
        {
                temp.left = avail.left;
                width = 0;
        }

        if (MoveWindow(winhandle,temp.left,temp.top,width,height,TRUE))
                avail.right = temp.left;
    }  // Right
    //	char data[100];
    if ((align == CLIENT))
    {
        width = (avail.right - avail.left);
        height = (avail.bottom - avail.top);
        if (width > (avail.right - avail.left))
                width = avail.right - avail.left;
        if (height < 0) height = 0;

        temp.left = avail.right - width;
        if (temp.left < avail.left)
        {
                temp.left = avail.left;
                width = 0;
        }
        MoveWindow(winhandle,temp.left,temp.top,width,height,TRUE);
    }   // Client
    SetForegroundWindow(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonAlignLeftClick(TObject *Sender)
{
        Align(winhandle,LEFT);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonAlignRightClick(TObject *Sender)
{
        Align(winhandle,RIGHT);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonAlignTopClick(TObject *Sender)
{
        Align(winhandle,TOP);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonAlignBottomClick(TObject *Sender)
{
       Align(winhandle,BOTTOM);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonAlignClientClick(TObject *Sender)
{
        Align(winhandle,CLIENT);
}


//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonCloseClick(TObject *Sender)
{
  HWND hwnd = (HWND)(EditWHandle->Text.ToInt());
  int result = MessageBox(Handle,"Are you sure?","Destroy Window",MB_YESNO);
  if (result == IDYES)
      SendMessage(hwnd,WM_CLOSE,0,0);
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonMinimizeClick(TObject *Sender)
{
  CloseWindow(winhandle); 	// handle to window to minimize
  SetForegroundWindow(winhandle);
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonMinimize2Click(TObject *Sender)
{
    SendMessage(winhandle,WM_SYSCOMMAND,SC_MINIMIZE,0);
    SetForegroundWindow(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonMaximizeClick(TObject *Sender)
{
    SendMessage(winhandle,WM_SYSCOMMAND,SC_MAXIMIZE,0);
    SetForegroundWindow(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonShowNormalClick(TObject *Sender)
{
    SendMessage(winhandle,WM_SYSCOMMAND,SC_RESTORE,0);
    SetForegroundWindow(Handle);
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonHideClick(TObject *Sender)
{
    ShowWindow(winhandle, SW_HIDE);
    SetForegroundWindow(Handle);
}


//---------------------------------------------------------------------------

void __fastcall TFormWinMan::MyWndProc(Messages::TMessage &Message)
{
  // If the window receives a notification message then
  // pass it to the appropriate windows messaging function. Otherwise let
  // the default processing for the message take place.

  if (Message.Msg == WM_MOVE_A_WINDOW)
        wmMoveAWindow(Message);
  else if (Message.Msg == WM_STRETCH_A_WINDOW)
        wmStretchAWindow(Message);
  else
        WndProc(Message) ;
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::wmMoveAWindow(TMessage Msg)
{
    RECT rect;
    HWND wh = HWND(Msg.WParam);
    GetWindowRect(wh, &rect);  // get the size for the current window
    int width  = rect.right  - rect.left;
    int height = rect.bottom - rect.top;
    int direction = Msg.LParam;
    while (ButtonDown)
    {
        switch (direction)
        {
            case MOVEUP    : rect.top--;  break;
            case MOVEDOWN  : rect.top++;  break;
            case MOVELEFT  : rect.left--; break;
            case MOVERIGHT : rect.left++; break;
        }
        MoveWindow(wh, rect.left, rect.top, width, height, true);
        Application->ProcessMessages(); // process other messages
        UpdateAppInfo(winhandle);
    }
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::wmStretchAWindow(TMessage Msg)
{
    RECT rect;
    HWND wh = HWND(Msg.WParam);
    GetWindowRect(wh, &rect);  // get the size for the current window
    int width  = rect.right  - rect.left;
    int height = rect.bottom - rect.top;
    int direction = Msg.LParam;
    while (ButtonDown)
    {
        switch (direction)
        {
            case STRETCHVERT  : height++;  break;
            case STRETCHHORZ  : width++;  break;
            case SHRINKVERT   : height--; break;
            case SHRINKHORZ   : width--; break;
        }
        MoveWindow(wh, rect.left, rect.top, width, height, true);
        Application->ProcessMessages(); // process other messages
        UpdateAppInfo(winhandle);
    }
}




//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveRightMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_MOVE_A_WINDOW, (unsigned int)winhandle, MOVERIGHT);
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveRightMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    ButtonDown = false;
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonMoveLeftMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_MOVE_A_WINDOW, (unsigned int)winhandle, MOVELEFT);
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonMoveLeftMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveDownMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_MOVE_A_WINDOW, (unsigned int)winhandle, MOVEDOWN);
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveDownMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveUpMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_MOVE_A_WINDOW, (unsigned int)winhandle, MOVEUP);
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonMoveUpMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}

//---------------------------------------------------------------------------

void __fastcall TFormWinMan::ButtonChangeTitleClick(TObject *Sender)
{
        SetWindowText(winhandle, EditTitle->Text.c_str());
        UpdateAppInfo(winhandle);
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::TreeView1Change(TObject *Sender, TTreeNode *Node)
{
   TTreeNode* node  = TreeView1->Selected;

   if ((node->Level == 0) && (CheckBoxProcessRootNode->Checked)) return;

   TAppInfo* info = (TAppInfo*)node->Data;

   winhandle = info->WindowHandle;
   UpdateAppInfo(winhandle);
}


//---------------------------------------------------------------------------
void __fastcall TFormWinMan::Button1Click(TObject *Sender)
{
//    COPYDATASTRUCT *data = new COPYDATASTRUCT;
//    data->dwData = 0;
//    SendMessage(winhandle,WM_COPYDATA,(WPARAM)Handle,(LPARAM)data);

  // bring it to the top - winhandle must be a visible top window
  ButtonBringToFocusClick(Sender);

  unsigned int duration = EditAnimDur->Text.ToIntDef(100);
  // hide it
  AnimateWindow(winhandle, duration, AW_HIDE | AW_SLIDE | AW_VER_POSITIVE);
  // show it
  AnimateWindow(winhandle, duration, AW_ACTIVATE | AW_SLIDE | AW_HOR_POSITIVE);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
unsigned int TFormWinMan::GetCorrelationChoice()
{
  if (RadioButtonProcess->Checked)
        return PROCESS;
  if (RadioButtonInstance->Checked)
        return INSTANCE;
  if (RadioButtonWindow->Checked)
        return WINDOW;
  else
        return 0;
}



//---------------------------------------------------------------------------
TTreeNode*  __fastcall TFormWinMan::GetNode_AppInfoValue(TTreeNodes *tree,
                                                TTreeNode *start = NULL,
                                                unsigned int valuetype = 0,
                                                unsigned long value = 0)
{
     TTreeNode *CurItem = start;
     if (!start) CurItem = tree->GetFirstNode();
     if (CurItem) do
     {
        if (CurItem->Data)
        {
            int id = 0;
            switch (valuetype)
            {
              case WINHANDLE :
                  id =  (unsigned long)PAppInfo(CurItem->Data)->WindowHandle;
                  break;
              case PROCESSID :
                  id =  (unsigned long)PAppInfo(CurItem->Data)->ProcessID;
                  break;
              case DEVICECONTEXT :
                  id =  (unsigned long)PAppInfo(CurItem->Data)->DeviceContext;
                  break;
              case INSTANCEHANDLE :
                  id =  (unsigned long)PAppInfo(CurItem->Data)->InstanceHandle;
                  break;
              default : return NULL;
            }
            if (id == value)
                   return CurItem;
        }
       CurItem = CurItem->GetNext();
     } while (CurItem);
     return start;  // couldn't find anything new
}

//---------------------------------------------------------------------------
TTreeNode*  __fastcall TFormWinMan::GetNode_Process(TTreeNodes *tree,unsigned long value = 0)

{
     TTreeNode *CurItem = tree->GetFirstNode();
     unsigned long pid;
     if (CurItem) do
     {
        if ((CurItem->Data) && (CurItem->Level == 0))
        {
            pid = (unsigned long)PAppInfo(CurItem->Data)->ProcessID;
            if (pid == value)
                   return CurItem;
        }
       CurItem = CurItem->GetNext();
     } while (CurItem);
     return NULL;  // couldn't find anything new
}

//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchRightMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_STRETCH_A_WINDOW, (unsigned int)winhandle, STRETCHHORZ);

}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchRightMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchLeftMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_STRETCH_A_WINDOW, (unsigned int)winhandle, SHRINKHORZ);

}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchLeftMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchUpMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_STRETCH_A_WINDOW, (unsigned int)winhandle, STRETCHVERT);

}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchUpMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchDownMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = true;
     PostMessage(Handle, WM_STRETCH_A_WINDOW, (unsigned int)winhandle, SHRINKVERT);

}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonStretchDownMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
     ButtonDown = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormWinMan::ButtonBringToFocusClick(TObject *Sender)
{
        SendMessage(winhandle,WM_SYSCOMMAND,SC_RESTORE,0);
        SetForegroundWindow(winhandle);
        if (CheckBoxTop->Checked) SetForegroundWindow(Handle);
}
//---------------------------------------------------------------------------






void __fastcall TFormWinMan::ButtonFlashClick(TObject *Sender)
{
  FLASHWINFO flash_info;
  flash_info.cbSize    = sizeof(FLASHWINFO);
  flash_info.hwnd      = winhandle;
  flash_info.dwFlags   = FLASHW_TIMER | FLASHW_ALL;
  flash_info.uCount    = EditCount->Text.ToIntDef(10); // # times to flash
  flash_info.dwTimeout = EditDur->Text.ToIntDef(100); // duration for each flash

  FlashWindowEx(&flash_info);
}
//---------------------------------------------------------------------------


