/*---------------------------------------------------------------------------
  WinSysUtilForm.cpp
        C++Builder Developer's Guide
        Chapter 14 - Windows System Services Viewer
        Displays System, Memory, Drive and File information
        (c) Paul Gustavson, 2002
---------------------------------------------------------------------------*/


#include <vcl.h>
#pragma hdrstop

#include "WinSysUtilForm.h"
#include <stdio.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormSystemService *FormSystemService;
//---------------------------------------------------------------------------
__fastcall TFormSystemService::TFormSystemService(TComponent* Owner)
        : TForm(Owner)
{
}

//---------------------------------------------------------------------------
// this is just a utility to format kilobyte oriented information
AnsiString __fastcall TFormSystemService::FormatSize(double value)   // pass in Kbytes
{
    AnsiString str_to_return;

    if ((value < -1000) || (value > 10000))
    {
       value = value / 1024;
       if ((value < -1000) || (value > 10000))
       {
          value = value / 1024;
          str_to_return = FloatToStrF(value,ffNumber,15,2) + " GB";
       }
       else
          str_to_return = FloatToStrF(value,ffNumber,15,2) + " MB";
    }
    else
    if (value < 1.0)
    {
        value = value * 1024;
        str_to_return = AnsiString((int)value) + " Bytes";
    }
    else
      str_to_return = AnsiString((int)value) + " KB";

    return str_to_return;
}

//---------------------------------------------------------------------------
//  Let's get System Information
//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonSystemInfoClick(TObject *Sender)
{
    // get user
    char name[MAX_PATH];
    DWORD size = MAX_PATH;
    GetUserName(name, &size);   // Win32 API call
    LabelUser->Caption = AnsiString(name);

    // get computer name
    char computername[MAX_COMPUTERNAME_LENGTH];
    size = MAX_COMPUTERNAME_LENGTH;
    GetComputerName(computername, &size);   // Win32 API call
    EditComputer->Text = ComputerName();

    // let's get the windows version
    char os[MAX_PATH];
    char version[MAX_PATH];
    char patch[MAX_PATH];
    bool NT = WindowsVersion(os,version,patch);  // Win32 API "wrapper"
    LabelOS->Caption = AnsiString(os);
    LabelVersion->Caption = AnsiString(version);
    LabelOther->Caption = AnsiString(patch);

    // let's get the processor info
    char processor[MAX_PATH];
    LabelNumProcessors->Caption =
                ProcessorInfo(NT,processor); // Win32 API "wrapper"
    LabelProcessor->Caption = AnsiString(processor);


    LabelScreenSize->Caption = ScreenSize();

    char dir[MAX_PATH];
    dir[0] = '\0';
    GetWindowsDirectory(dir,MAX_PATH);
    LabelWinDir->Caption = AnsiString(dir);

    dir[0] = '\0';
    GetSystemDirectory(dir,MAX_PATH);
    LabelSysDir->Caption = AnsiString(dir);

    dir[0] = '\0';
    GetTempPath(MAX_PATH, dir);
    LabelTempDir->Caption = AnsiString(dir);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TFormSystemService::LoginUserName()
{
    AnsiString Name;
    DWORD size = MAX_PATH+1;
    char name[MAX_PATH+1];
    name[0] = '\0';  // initialize

//    char name[MAX PATH];
    GetUserName(name, &size);   // Win32 API call

    Name = AnsiString(name);
    return Name;
}

//---------------------------------------------------------------------------

AnsiString __fastcall TFormSystemService::ComputerName()
{
    AnsiString Name;
    DWORD size = MAX_COMPUTERNAME_LENGTH + 1;
    char name[MAX_COMPUTERNAME_LENGTH + 1];
    name[0] = '\0';  // initialize

    GetComputerName(name, &size);   // Win32 API call

    Name = AnsiString (name);
    return Name;
}

//---------------------------------------------------------------------------

bool __fastcall TFormSystemService::WindowsVersion(char * os, char * version, char * other)
{
    bool NT = false;

    OSVERSIONINFO version_info;

    version_info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO); // initilize size
    GetVersionEx(&version_info); // now let's get version info - Win32 API call

    if(version_info.dwMajorVersion < 4)  // must be win nt 3.x
    {
        sprintf(os,"Windows NT ");
        sprintf(version,"3.x");
    }
    else
    {
        sprintf(version,"%d.%d.%04d",version_info.dwMajorVersion,
                        version_info.dwMinorVersion,
                        version_info.dwBuildNumber);
    }
    switch (version_info.dwPlatformId)
    {
        case VER_PLATFORM_WIN32s:
            sprintf(os,"%s","Microsoft Win32s");
            break;
        case VER_PLATFORM_WIN32_WINDOWS:
            sprintf(os,"%s","Microsoft Windows 95/98");
            break;
        case VER_PLATFORM_WIN32_NT:
            NT = true;
            switch (version_info.dwMajorVersion)
            {
               case 4: sprintf(os,"%s","Microsoft Windows NT"); break;
               case 5: sprintf(os,"%s","Microsoft Windows 2000"); break;
               case 6: sprintf(os,"%s","Microsoft Windows XP"); break;
               default :
                  sprintf(os,"%s","Microsoft Windows ??");
            }
            break;
        default : // unknown
            sprintf(os,"%s","Microsoft Windows ??");
    }
    sprintf(other,"%s",version_info.szCSDVersion);
    return NT;
}

//---------------------------------------------------------------------------

int __fastcall TFormSystemService::ProcessorInfo(bool NT, char* processor)
{
   SYSTEM_INFO sys_info;

   AnsiString Processor;
   AnsiString Level = "";
   GetSystemInfo(&sys_info); // determine processor type - Win32 API call
   int value;
   if (NT)
   {
        switch (sys_info.wProcessorArchitecture)
        {
            case  PROCESSOR_ARCHITECTURE_INTEL :
                Processor = "Intel";
                switch (sys_info.wProcessorLevel)
                {
                    case  3 : Level = "80386"; break;
                    case  4 : Level = "80486"; break;
                    case  5 : Level = "Pentium"; break;
                    case  6 :
                        value =  HIBYTE(sys_info.wProcessorRevision);
                        switch (value)
                        {
                          case  1         : Level = "Pentium PRO"; break;
                          case  3,5       : Level = "Pentium 2"; break;
                          case  6         : Level = "Celeron"; break;
                          case  7,8,10,11 : Level = "Pentium 3"; break;
                        }
                        break;
                    case 15 : Level = "Pentium 4"; break;
                    default : Level = "unknown";
                }
                break;
            case  PROCESSOR_ARCHITECTURE_ALPHA :
                Processor = "ALPHA";
                switch (sys_info.wProcessorLevel)
                {
                    case  21064 : Level = "21064"; break;
                    case  21066 : Level = "21066"; break;
                    case  21164 : Level = "21164"; break;
                    default :   Level = "unknown";
                }
                break;
            case  PROCESSOR_ARCHITECTURE_MIPS :
                Processor = "MIPS";
                switch (sys_info.wProcessorLevel)
                {
                    case  0004 : Level = "R4000"; break;
                    default :   Level = "unknown";
                }
                break;
            case  PROCESSOR_ARCHITECTURE_PPC :
                Processor = "PPC";
                switch (sys_info.wProcessorLevel)
                {
                    case  1 : Level = "601"; break;
                    case  3 : Level = "603"; break;
                    case  4 : Level = "604"; break;
                    case  6 : Level = "603+"; break;
                    case  9 : Level = "604+"; break;
                    case 20 : Level = "620"; break;
                    default :   Level = "unknown";
                }
                break;
            default :   Processor = "unknown";
        }
   }
   else  // 9x
   {
        switch (sys_info.dwProcessorType)
        {
            case  PROCESSOR_INTEL_386      :
                    Processor = "Intel"; Level = "80386"; break;
            case  PROCESSOR_INTEL_486      :
                    Processor = "Intel"; Level = "80486"; break;
            case  PROCESSOR_INTEL_PENTIUM  :
                    Processor = "Intel"; Level = "Pentium"; break;
            default:   Processor = "unknown";
        }
   }
   sprintf(processor,"%s %s",Processor.c_str(),Level.c_str());
   return sys_info.dwNumberOfProcessors;
}

//---------------------------------------------------------------------------

AnsiString TFormSystemService::ScreenSize()
{
    AnsiString value;

    RECT screen_coords;
    SystemParametersInfo(SPI_GETWORKAREA,0, &screen_coords, 0); //Win32 API call

    int width  = screen_coords.right - screen_coords.left;
    int height = screen_coords.bottom - screen_coords.top;
/*
   // here's another way, but width and height is obsurced by the tray

    int height = GetSystemMetrics(SM_CYFULLSCREEN);
    int width =  GetSystemMetrics(SM_CXFULLSCREEN);
*/
    value = AnsiString(width) + " x " + AnsiString(height) + " pixels";
    return value;
}

//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonChangeComputerClick(TObject *Sender)
{
     SetComputerName(EditComputer->Text.c_str());
     if (MessageBox(Handle,"You must reboot system for change to take affect.",
                "Reboot System?",MB_YESNO) == IDYES)
     {
         bool success = ExitWindowsEx(EWX_REBOOT, 0);
         if  (!success)
            MessageBox(Handle,
                "Unable to shutdown system due to a system restriction.",
                "System Restriction",MB_OK);
         else
            Close();
     }
}

//---------------------------------------------------------------------------
// Now let's get drive info
//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonGetDriveInfoClick(TObject *Sender)
{

  AnsiString temp;
  AnsiString Drive = AnsiString(DriveComboBox1->Drive) + ":\\"; //EditDrive->Text;

  unsigned int drivetype = GetDriveType(Drive.c_str ());

  switch (drivetype)
  {
    case 1	: temp = "No root directory";   return;
    case DRIVE_REMOVABLE : temp = "Removable";  break;
    case DRIVE_FIXED     : temp = "Fixed";      break;
    case DRIVE_REMOTE    : temp = "Remote (network) drive";  break;
    case DRIVE_CDROM     : temp = "CD-ROM";     break;
    case DRIVE_RAMDISK   : temp = "RAM disk";   break;
    default:	temp = "Unknown"; return;
  }

  LabelDriveType->Caption = temp;
  temp = "";
  DWORD VolumeSerialNumber = 0;
  DWORD MaximumComponentLength = 0;
  DWORD FileSystemFlags = 0;
  char * volumeinfo = new char[255];
  volumeinfo[0] = 0;
  char* FileSystemNameBuffer = new char[255];
  FileSystemNameBuffer[0] = 0;
  GetVolumeInformation (Drive.c_str (), volumeinfo,
                            255, &VolumeSerialNumber,
                            &MaximumComponentLength, &FileSystemFlags, FileSystemNameBuffer,255);
    if (strlen(volumeinfo) != 0)
        EditVolumeInfo->Text = volumeinfo ;
    else
        EditVolumeInfo->Text = "- no label -";


    //Translate integer to chars for serial number
    char string1[35];
    char string2[35];
    if (VolumeSerialNumber > 0)
    {
      unsigned int bottom = (LOWORD(VolumeSerialNumber));
      unsigned int top = (HIWORD(VolumeSerialNumber));
      sprintf(string1,"%04X",top);
      sprintf(string2,"%04X",bottom);
      LabelSerialNum->Caption = AnsiString(string1) + "-" + AnsiString(string2);
    }
    else
      LabelSerialNum->Caption = "- unknown -";
    if (MaximumComponentLength > 0)
      LabelMaxComponentLength->Caption = AnsiString(MaximumComponentLength) + " characters";
    else
      LabelMaxComponentLength->Caption = "- unknown -";
    if (strlen(FileSystemNameBuffer) != 0)
        LabelFileSystemNameBuffer->Caption = FileSystemNameBuffer;
    else
        LabelFileSystemNameBuffer->Caption = "- unknown -";

    LabelFileSystemFlags->Caption = ""; //AnsiString(FileSystemFlags);
    if (FileSystemFlags & FS_CASE_IS_PRESERVED)
         temp += AnsiString("Filename case is preserved.\n");
    if (FileSystemFlags & FS_CASE_SENSITIVE)
         temp += AnsiString("Lookup is case-sensitive.\n");
    if (FileSystemFlags & FS_UNICODE_STORED_ON_DISK)
         temp += AnsiString("Supports Unicode in filenames.\n");
    if (FileSystemFlags & FS_PERSISTENT_ACLS)
         temp += AnsiString("Preserves and enforces ACLs.\n");
    if (FileSystemFlags & FS_FILE_COMPRESSION)
         temp += AnsiString("Supports file-based compression.\n");
    if (FileSystemFlags & FS_VOL_IS_COMPRESSED)
         temp += AnsiString("Volume is compressed. (i.e., DoubleSpace).\n");
    LabelFileSystemFlags->Caption = temp;

    DWORD spc = 0;    //Sectors per cluster
    DWORD bps = 0;    //Bytes per cluster
    DWORD cluster = 0;   //clusters
    DWORD freeclust = 0;   //freeclusters

    GetDiskFreeSpace (Drive.c_str (),&spc,&bps,&freeclust,&cluster) ;
    unsigned long v1 = (unsigned long)cluster;
    unsigned long v2 = (unsigned long) spc;
    unsigned long v3 = (unsigned long) bps;

    unsigned long volsize = (v1 * v2)/1024 * v3;
    LabelVolumeSize->Caption = AnsiString(FormatSize(volsize));

    unsigned long free_bytes = (freeclust * spc)/1024 * bps;
    LabelFreeSpace->Caption = AnsiString(FormatSize(free_bytes));

    if (volsize > 0)
        LabelUsed->Caption = AnsiString(((volsize - free_bytes) * 100) / volsize) +  " %";
    else
        LabelUsed->Caption = "n/a";
}

//---------------------------------------------------------------------------

void __fastcall TFormSystemService::DriveComboBox1Change(TObject *Sender)
{
  ButtonGetDriveInfoClick(Sender);
}

//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonChangeVolumeLabelClick(TObject *Sender)
{
  AnsiString Drive = AnsiString(DriveComboBox1->Drive) + ":\\"; //EditDrive->Text;

  bool success = SetVolumeLabel(
                        Drive.c_str(),
                        EditVolumeInfo->Text.c_str());

  if  (!success)
      MessageBox(Handle,
          "Unable to change volume label due to a system restriction.",
          "System Restriction",MB_OK);
  else
      ButtonGetDriveInfoClick(Sender);
}

//---------------------------------------------------------------------------
// Get Memory Information
//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonGetMemInfoClick(TObject *Sender)
{
  MEMORYSTATUS memory ;
  memory.dwLength = sizeof (memory) ;
  GlobalMemoryStatus (&memory) ;

  LabelPTotal->Caption  = FormatSize((memory.dwTotalPhys/1024));
  LabelPFree->Caption   = FormatSize((memory.dwAvailPhys/1024));
  LabelPgTotal->Caption = FormatSize((memory.dwTotalPageFile/1024));
  LabelPgFree->Caption  = FormatSize((memory.dwAvailPageFile/1024));
  LabelVTotal->Caption  = FormatSize((memory.dwTotalVirtual/1024));
  LabelVFree->Caption   = FormatSize((memory.dwAvailVirtual/1024));

  double value1 = double(double(memory.dwAvailPhys*100.0)/memory.dwTotalPhys);
  double value = 100.0-memory.dwMemoryLoad;

  if (value1 > value)
     value = long(value1);

  LabelFree->Caption  = AnsiString(value) + " % free";;
}

//---------------------------------------------------------------------------
// Get File Information
//---------------------------------------------------------------------------

void __fastcall TFormSystemService::ButtonGetFileInfoClick(TObject *Sender)
{
  if(OpenDialog1->Execute())
  {
      EditFilename->Text = ExtractFileName(OpenDialog1->FileName);
      LabelLocation->Caption = ExtractFilePath(OpenDialog1->FileName);

      // clear out everything
      LabelAttrib->Caption = "";
      LabelFileID->Caption = "";
      LabelDateTime->Caption = "";

      // let's open the file
      HANDLE FileHandle = CreateFile(OpenDialog1->FileName.c_str(),
              GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);

      if (FileHandle == INVALID_HANDLE_VALUE)
      {
          MessageBox(Handle, OpenDialog1->FileName.c_str(),
                          "Unable to open file", MB_ICONSTOP | IDOK);
          return;
      }
      else // // get file type
      {
         DWORD FileType = GetFileType(FileHandle);
         switch (FileType)
         {
             case FILE_TYPE_DISK:
                  LabelFileType->Caption = "Disk File";
                  break;
             case FILE_TYPE_CHAR:
                  LabelFileType->Caption = "Character";
                  return;  // can't do much else
             case FILE_TYPE_PIPE:
                  LabelFileType->Caption = "Anonymous Pipe";
                  return;  // can't do much else
             default : LabelFileType->Caption = "Unknown";
         }
      }

      // get more information
      BY_HANDLE_FILE_INFORMATION FileInfo;
      GetFileInformationByHandle(FileHandle, &FileInfo);
      AnsiString temp;
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE)
              temp += "Archive ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_COMPRESSED)
              temp += "Compressed ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
              temp += "Directory ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN)
              temp += "Hidden ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_NORMAL)
              temp += "Normal ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_OFFLINE)
              temp += "Offline ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
              temp += "Readonly ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_SYSTEM)
              temp += "System ";
      if (FileInfo.dwFileAttributes & FILE_ATTRIBUTE_TEMPORARY)
              temp += "Temporary Storage ";
      LabelAttrib->Caption = temp;

      char string1[35];
      char string2[35];
      sprintf(string1,"%08X",FileInfo.nFileIndexHigh);
      sprintf(string2,"%08X",FileInfo.nFileIndexLow);
      LabelFileID->Caption = "0x" + AnsiString(string1) + AnsiString(string2); //hex

      LabelFileSize->Caption =  FormatSize(double(FileInfo.nFileSizeLow / 1024.0));

      SYSTEMTIME SysTime;  // system time and date
      FileTimeToSystemTime(&FileInfo.ftLastWriteTime, &SysTime);
      char date[255];
      char time[255];
      sprintf(date,"%u/%u/%4u",
          SysTime.wMonth, SysTime.wDay, SysTime.wYear);
      sprintf(time,"%2u:%02u.%02u",
          SysTime.wHour, SysTime.wMinute, SysTime.wSecond);
      LabelDateTime->Caption = AnsiString(date) + " " + AnsiString(time);

      // another way to get file size
//      DWORD lo_size = GetFileSize(FileHandle,null);

      CloseHandle(FileHandle);
  }
}

//---------------------------------------------------------------------------


void __fastcall TFormSystemService::Splitter2Moved(TObject *Sender)
{
   GroupBox3->Width = GroupBox2->Width;
}
//---------------------------------------------------------------------------


void __fastcall TFormSystemService::Splitter1Moved(TObject *Sender)
{
   GroupBox2->Width = GroupBox3->Width;

}
//---------------------------------------------------------------------------

