
/****************************************************************************\

    CDDevice.cpp 
    Damon Chandler <dmc27@cornell.edu>

    A CD-audio-tuned MCIDevice descendant class

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

\****************************************************************************/

#include <windows.h>
#pragma hdrstop

#include <math.h>
#include "CDDevice.h"
#pragma link "MCIDevice"
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

int CDDevice::doGetNumDevices(int ADevType) const
{
    return MCIDevice::doGetNumDevices(MCI_DEVTYPE_CD_AUDIO);
}
//---------------------------------------------------------------------------

bool CDDevice::doPlay(long AStart, long AStop)
{
    if (!IsPaused_)
    {
        return MCIDevice::doPlay(MCI_MAKE_TMSF(AStart, 0, 0, 0), -1);
    }
    else
    {
        IsPaused_ = false;
        return MCIDevice::doPlay(-1, -1);
    }
}
//---------------------------------------------------------------------------

bool CDDevice::doStop()
{
    IsPaused_ = false;
    return MCIDevice::doStop();
}
//---------------------------------------------------------------------------

bool CDDevice::doPause()
{
    IsPaused_ = true;
    return MCIDevice::doPause();
}
//---------------------------------------------------------------------------

bool CDDevice::isPaused() const
{
    return IsPaused_;
}
//---------------------------------------------------------------------------


bool CDDevice::open()
{
    if (MCIDevice::doOpen(NULL, "cdaudio"))
    {
        return MCIDevice::doSetTimeFormat(MCI_FORMAT_TMSF);
    }
    return false;
}
//---------------------------------------------------------------------------

bool CDDevice::play()
{
    return doPlay(trackNumber(), -1);
}
//---------------------------------------------------------------------------

bool CDDevice::prev()
{
    bool IsPlaying = playing();
    if (IsPlaying) pause();

    int track = trackNumber();
    if (track > 1) track = track - 1;
    else track = numTracks();

    bool result = doSeek(MCI_MAKE_TMSF(track, 0, 0, 0));
    if (IsPlaying) play();
    return result;
}
//---------------------------------------------------------------------------

bool CDDevice::next()
{
    bool IsPlaying = playing();
    if (IsPlaying) pause();

    int track = trackNumber();
    if (track < numTracks()) track = track + 1;
    else track = 1;

    bool result = doSeek(MCI_MAKE_TMSF(track, 0, 0, 0));
    if (IsPlaying) play();
    return result;
}
//---------------------------------------------------------------------------

bool CDDevice::mediaPresent() const
{
    return MCIDevice::doStatus(
        MCI_STATUS_ITEM, MCI_STATUS_MEDIA_PRESENT, NULL);
}
//---------------------------------------------------------------------------

int CDDevice::trackNumber() const
{
    if (mediaPresent())
    {
        return MCIDevice::doStatus(
            MCI_STATUS_ITEM, MCI_STATUS_CURRENT_TRACK, NULL);
    }
    return 0;
}
//---------------------------------------------------------------------------

int CDDevice::numTracks() const
{
    if (mediaPresent())
    {
        return MCIDevice::doStatus(
            MCI_STATUS_ITEM, MCI_STATUS_NUMBER_OF_TRACKS, NULL);
    }
    return 0;
}
//---------------------------------------------------------------------------

DWORD CDDevice::trackLength() const
{
    DWORD result = 0;
    if (mediaPresent() && MCIDevice::doSetTimeFormat(MCI_FORMAT_MSF))
    {
        result =
            MCIDevice::doStatus(MCI_STATUS_ITEM | MCI_TRACK,
                                MCI_STATUS_LENGTH, trackNumber());
        MCIDevice::doSetTimeFormat(MCI_FORMAT_TMSF);
    }
    return result;
}
//---------------------------------------------------------------------------

double CDDevice::trackPosition() const
{
    double result = 0;
    if (mediaPresent() &&
        MCIDevice::doSetTimeFormat(MCI_FORMAT_MILLISECONDS))
    {
        long pos =
            MCIDevice::doStatus(MCI_STATUS_ITEM, MCI_STATUS_POSITION, 0);
        long track_start =
            MCIDevice::doStatus(MCI_STATUS_ITEM | MCI_TRACK,
                                MCI_STATUS_POSITION, trackNumber());

        result = pos - track_start;
        MCIDevice::doSetTimeFormat(MCI_FORMAT_TMSF);
    }
    return result;
}
//---------------------------------------------------------------------------


//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//    [EOF]
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

 
