
/****************************************************************************\

    MCIDevice.cpp 
    Damon Chandler <dmc27@cornell.edu>

    A simple MCI wrapper class

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

\****************************************************************************/

#include <windows.h>
#pragma hdrstop

#include <math.h>
#include "MCIManip.h"
#include "MCIDevice.h"
#pragma link "MCIManip"
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

MCIDevice::~MCIDevice()
{
    if (DeviceID_)
    {
        if (playing()) doStop();
        doClose();
    }
}
//---------------------------------------------------------------------------

int MCIDevice::doGetNumDevices(int ADevType) const
{
    return mciGetNumDevices(ADevType);
}
//---------------------------------------------------------------------------

bool MCIDevice::doOpen(const char* AFileName, const char* ADeviceType)
{
    return mciOpen(DeviceID_, AFileName, ADeviceType, HCallback_, Report_);
}
//---------------------------------------------------------------------------

bool MCIDevice::doClose()
{
    return (
        DeviceID_ &&
        mciClose(DeviceID_, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::doPlay(long AStart, long AStop)
{
    return (
        DeviceID_ &&
        mciPlay(DeviceID_, AStart, AStop, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::doStop()
{
    return (
        DeviceID_ &&
        mciStop(DeviceID_, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::doPause()
{
    return (
        DeviceID_ &&
        mciPause(DeviceID_, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::doSeek(long APosition)
{
    return (
        DeviceID_ &&
        mciSeek(DeviceID_, APosition, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::doSetTimeFormat(long ATimeFormat)
{
    return (
        DeviceID_ &&
        mciSetTimeFormat(DeviceID_, ATimeFormat, HCallback_, Report_)
        );
}
//---------------------------------------------------------------------------

DWORD MCIDevice::doStatus(long AQueryGroup, long AQueryItem,
    long AQueryTrack) const
{
    DWORD result = 0;
    if (DeviceID_)
    {
        mciStatus(DeviceID_, AQueryGroup, AQueryItem, AQueryTrack,
                  result, Report_);
    }
    return result;
}
//---------------------------------------------------------------------------

bool MCIDevice::isOpened() const
{
    return (
        DeviceID_ &&
        doStatus(MCI_STATUS_ITEM, MCI_STATUS_MODE, 0) == MCI_MODE_OPEN
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::isPlaying() const
{
    return (
        DeviceID_ &&
        doStatus(MCI_STATUS_ITEM, MCI_STATUS_MODE, 0) == MCI_MODE_PLAY
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::isStopped() const
{
    return (
        DeviceID_ &&
        doStatus(MCI_STATUS_ITEM, MCI_STATUS_MODE, 0) == MCI_MODE_STOP
        );
}
//---------------------------------------------------------------------------

bool MCIDevice::isPaused() const
{
    return (
        DeviceID_ &&
        doStatus(MCI_STATUS_ITEM, MCI_STATUS_MODE, 0) == MCI_MODE_PAUSE
        );
}
//---------------------------------------------------------------------------


//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//    [EOF]
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>










