
/****************************************************************************\
    --------------------------------------------------------
     DSPDemo.cpp -- Created 05/08/2000 by Damon Chandler
    --------------------------------------------------------
    
    This sample project demonstrates the use of the AVIFile services
    to read a waveform-audio file from disk.  The audio file can be
    processed (quantized), then played back via the Waveform-Audio
    interface.  This demo uses the supplied waveapi library (waveapi.cpp,
    waveapi.h).  Please refer to the implementation of this library for
    the specifics of the AVIFile and Waveform-Audio interfaces.

\****************************************************************************/


#include <vcl.h>
#pragma hdrstop

#include "waveapi.h"
#include "DSPDemo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------

void QuantizeBuffer(char* ABuffer, long ABufferLength)
{
    /* NOTE: The quantized version will sound bad. */
    /* Scheme assumes stereo -- TODO: verify format. */
        
    short int min_val = 0, max_val = 0;
    for (int index = 0; index < ABufferLength; ++index)
    {
        if (ABuffer[index] < min_val) min_val = ABuffer[index];
        if (ABuffer[index] > max_val) max_val = ABuffer[index];
    }

    for (int index = 0; index < ABufferLength; ++index)
    {
        if (ABuffer[index] < 0) ABuffer[index] = min_val;
        if (ABuffer[index] > 0) ABuffer[index] = max_val;
    }
}
//---------------------------------------------------------------------------

__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    wavInititialize();
}
//---------------------------------------------------------------------------

__fastcall TForm1::~TForm1()
{
    wavUninitialize();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OpenButtonClick(TObject *Sender)
{
    const long MAX_BLOCK_SIZE = 6000 * 1024;
    
    if (!OpenDialog1->Execute()) return;
    const char* filename = OpenDialog1->FileName.c_str();

    PAVIFILE pFile = NULL;
    if (wavOpenFile(pFile, filename, OF_READ))
    {
        PAVISTREAM pStream = NULL;
        if (wavOpenStream(pStream, pFile))
        {
            long block_size = wavCalcBufferSize(pStream);
            if (block_size < MAX_BLOCK_SIZE)
            {
                char* buffer = new char[block_size];
                if (wavReadStream(pStream, 0, block_size, buffer)
                    == block_size)
                {
                    // maniuplate the buffer:
                    // for example...
                    if (RadioGroup1->ItemIndex == 1)
                        QuantizeBuffer(buffer, block_size);

                    WAVEFORMATEX FormatStruct;
                    if (wavReadFormatStruct(pStream, FormatStruct))
                    {
                        HWAVEOUT HWavOut;
                        if (wavPlayOpen(HWavOut, NULL, NULL, NULL,
                                        FormatStruct))
                        {
                            WAVEHDR WavHdr;
                            memset(&WavHdr, 0, sizeof(WAVEHDR));
                            WavHdr.lpData = buffer;
                            WavHdr.dwBufferLength = block_size;

                            if (wavPlayBegin(HWavOut, WavHdr))
                            {
                                ShowMessage("Playing: " +
                                            AnsiString(filename));
                                wavPlayEnd(HWavOut, WavHdr);
                            }
                            wavPlayClose(HWavOut);
                        }
                    }
                }
                delete [] buffer;
            }
            else ShowMessage("file is too large -- increase MAX_BLOCK_SIZE");
            wavCloseStream(pStream);
        }
        wavCloseFile(pFile);
    }
}
//---------------------------------------------------------------------------



