/*---------------------------------------------------------------------------
  FormMain.cpp
        Chapter 15 - Analog Clock Example
        modified by Paul Gustavson, 2002
---------------------------------------------------------------------------*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include <math.h> 
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    BrushStyle = bsSolid;
    Caption = Application->Title;
    InitializeImage();
}
//---------------------------------------------------------------------------
__fastcall TForm1::~TForm1()
{
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    DrawClockToHiddenImage(LastTime,true);
    LastTime = TDateTime::CurrentTime();
    DrawClockToHiddenImage(LastTime,false);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::InitializeImage()
{
    HiddenImage->Width = ClientWidth;
    HiddenImage->Height = ClientHeight;
    Center.x = HiddenImage->Width / 2;
    Center.y = HiddenImage->Height / 2;
    Radius = (3.5*(Center.x < Center.y ? Center.x : Center.y) / 4);
    TRect BoundingBox(Center.x - Radius, Center.y - Radius, Center.x + Radius, Center.y + Radius);
    Radius -= 5;  // we want the radius of the hands to be inside the circle

    // clear the image first
    HiddenImage->Canvas->Pen->Width = 2;
    HiddenImage->Canvas->Pen->Mode = pmCopy;
    HiddenImage->Canvas->Pen->Color = clWhite;
    HiddenImage->Canvas->Brush->Color = clWhite;
    HiddenImage->Canvas->Brush->Style = bsSolid;
    HiddenImage->Canvas->Rectangle(HiddenImage->ClientRect);

    HiddenImage->Canvas->Brush->Style = BrushStyle;
    HiddenImage->Canvas->Brush->Color = clLtGray;
    HiddenImage->Canvas->Pen->Color = clBlack;
    HiddenImage->Canvas->Ellipse(BoundingBox);
    HiddenImage->Canvas->Brush->Style = bsSolid;

    // now write some text - and center using TextExtent()
    AnsiString text = "Right Click for Menu...";
    HiddenImage->Canvas->Font = Font;
    HiddenImage->Canvas->Brush->Color = clWhite;
    TSize textSize = HiddenImage->Canvas->TextExtent(text);
    int x = (HiddenImage->Width - textSize.cx) / 2;
    int y = HiddenImage->Height - textSize.cy - 2;
    HiddenImage->Canvas->TextOut(x, y, text);

    
    LastTime = TDateTime::CurrentTime();
    DrawClockToHiddenImage(LastTime,false);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DrawClockToHiddenImage(const TDateTime &Time, bool erase)
{
    const float TwoPi = 2*3.141592;
    unsigned short hour, min, sec, msec;

    Time.DecodeTime(&hour, &min, &sec, &msec);

    // Compute the location of the hands
    float angleOfSecondHand = (sec * TwoPi) / 60;
    float angleOfMinuteHand = (min * TwoPi) / 60;
    float angleOfHourHand = (hour % 12) * TwoPi/12 + angleOfMinuteHand/12;

    TPoint SecondPoint((Radius)*sin(angleOfSecondHand) + Center.x,
                (-Radius)*cos(angleOfSecondHand) + Center.y);
    TPoint MinutePoint(Radius*sin(angleOfMinuteHand) + Center.x,
                  -Radius*cos(angleOfMinuteHand) + Center.y);
    TPoint HourPoint((Radius/2)*sin(angleOfHourHand) + Center.x,
                (-Radius/2)*cos(angleOfHourHand) + Center.y);

    // Now draw them.  Use the pmNotXor pen drawing mode so that a redraw in the
    // same place will erase the line.

    HiddenImage->Canvas->Pen->Mode = pmNotXor;

    HiddenImage->Canvas->Pen->Color = clNavy;
    HiddenImage->Canvas->Brush->Color = clNavy;
    HiddenImage->Canvas->Pen->Width = 4;
    HiddenImage->Canvas->PenPos = Center;
    HiddenImage->Canvas->LineTo(HourPoint.x, HourPoint.y);

    HiddenImage->Canvas->Pen->Color = clRed;
    HiddenImage->Canvas->Brush->Color = clRed;
    HiddenImage->Canvas->Pen->Width = 4;
    HiddenImage->Canvas->PenPos = Center;
    HiddenImage->Canvas->LineTo(MinutePoint.x, MinutePoint.y);

    HiddenImage->Canvas->Pen->Color = clBlack;
    HiddenImage->Canvas->Brush->Color = clBlack;
    HiddenImage->Canvas->Pen->Width = 2;
    HiddenImage->Canvas->PenPos = Center;
    HiddenImage->Canvas->LineTo(SecondPoint.x, SecondPoint.y);

    HiddenImage->Canvas->Pen->Mode = pmCopy;
    Invalidate();

}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormPaint(TObject *Sender)
{
    // copy the information on the hidden canvas onto the form's canvas.
    Canvas->CopyRect(ClientRect, HiddenImage->Canvas, HiddenImage->ClientRect);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::UncheckAllBrushItems()
{
    bsSolid1->Checked = false;
    bsBDiagonal1->Checked = false;
    bsFDiagonal1->Checked = false;
    bsCross1->Checked = false;
    bsHorizontal1->Checked = false;
    bsVertical1->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsSolid1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsSolid1->Checked = true;
    BrushStyle = bsSolid;
    InitializeImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsBDiagonal1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsBDiagonal1->Checked = true;
    BrushStyle = bsBDiagonal;
    InitializeImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsFDiagonal1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsFDiagonal1->Checked = true;
    BrushStyle = bsFDiagonal;
    InitializeImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsCross1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsCross1->Checked = true;
    BrushStyle = bsCross;
    InitializeImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsHorizontal1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsHorizontal1->Checked = true;
    BrushStyle = bsHorizontal;
    InitializeImage();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::bsVertical1Click(TObject *Sender)
{
    UncheckAllBrushItems();
    bsVertical1->Checked = true;
    BrushStyle = bsVertical;
    InitializeImage();
}
//---------------------------------------------------------------------------
