/*---------------------------------------------------------------------------
  ImageProcessForm.cpp
        Chapter 15 - Image Processing Example
        Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------
#include <vcl.h>
#include <math.h>
#include <jpeg.hpp>
#pragma hdrstop

#include "ImageProcessForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

int Bilinear(BYTE Image[][256], float i, float j)
{
        int x1, y1, x2, y2;
        float Gray;

        x1 = (int)floor(i);
        x2 = x1+1;
        y1 = (int)floor(j);
        y2 = y1+1;
        Gray = (y2-j)*(x2-i)*Image[x1][y1] + (y2-j)*(i-x1)*Image[x2][y1]
                + (j-y1)*(x2-i)*Image[x1][y2] + (j-y1)*(i-x1)*Image[x2][y2];
        return INT(Gray);
}

int Lowpass(BYTE Image[][256], int x, int y)
{
        int Mask[3][3] = {{1, 1, 1,}, {1, 1, 1}, {1, 1, 1}};
        int i, j, sum=0;

        for (j=-1; j<=1; j++)
                for (i=-1; i<=1; i++)
                      sum += Image[x+i][y+j]*Mask[i+1][j+1];
        return INT(sum/9.0);
}
int Edge(BYTE Image[][256], int x, int y)
{
        int Mask[3][3] = {{-1, -1, -1,}, {-1, 9, -1}, {-1, -1, -1}};
        int i, j, sum=0;

        for (j=-1; j<=1; j++)
                for (i=-1; i<=1; i++)
                      sum += Image[x+i][y+j]*Mask[i+1][j+1];
        return INT(sum/9.0);
}

TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
      MouseDown = false;
      BezierSequence = 0;
      NumCurves      = 0;
      CurrentPoint   = 0;

     oX = -1;
     oY = -1;
     lX = -1;
     lY = -1;

}


//---------------------------------------------------------------------------

void __fastcall TForm1::WMEraseBkgnd(TMessage Msg)
{
//        if (CheckBoxEraseBackground->Checked)
//                TForm::
//                WndProc(Msg); //inherited;  //   Erase the backGround
}

//---------------------------------------------------------------------------
void __fastcall TForm1::ButtonLoadClick(TObject *Sender)
{
    //This code requires "jpeg.hpp" to be included in the source file
    OpenDialog1->Filter =
       "Bmp files (*.bmp)|*.BMP| JPEG images (*.jpg) | *.jpg; " ;
    if (OpenDialog1->Execute())
    {
        if (!FileExists(OpenDialog1->FileName))
            return; // make sure it exists, else get out.
        AnsiString temp2 = ExtractFileName(OpenDialog1->FileName);
        AnsiString temp = ExtractFileExt(OpenDialog1->FileName);
        AnsiString Ext = temp.LowerCase();

        if (Ext.AnsiPos("jpg") > 0)  // it's a jpg
        {   //-- Decompress the jpeg image into a bitmap.
            TJPEGImage *myjpeg = new TJPEGImage();
            myjpeg->LoadFromFile(OpenDialog1->FileName);
            myjpeg->DIBNeeded();  // used when jpeg image needs bitmap rep
            Image1->Picture->Bitmap->Assign(myjpeg);
            delete myjpeg;
        }
        else if (Ext.AnsiPos("bmp") > 0)
        {
            Image1->Picture->Bitmap->LoadFromFile(OpenDialog1->FileName);
        }
        EditFile->Text        = ExtractFileName(OpenDialog1->FileName);
        EditWidth->Text       = Image1->Width;
        EditHeight->Text      = Image1->Height;
        EditPixelFormat->Text = Image1->Picture->Bitmap->PixelFormat;
    }
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ButtonSinusoidClick(TObject *Sender)
{
        int x, y;
        int Amplitude, Gray;
        float Period;
        TColor RGB;

        Image1->Picture->Bitmap = new Graphics::TBitmap;
        Image1->Picture->Bitmap->PixelFormat = pf24bit;
        Image1->Picture->Bitmap->Width = 256;
        Image1->Picture->Bitmap->Height = 256;
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                {
                        Amplitude = 64*(255-y)/255;
                        Period = 100*sqrt(1/(1+(exp(0.013*x)*exp(0.027*x)/400)));
                        Gray = Amplitude*sin(2*M_PI/Period*x)+128;
                        RGB = (Graphics::TColor)((Gray << 16) | (Gray << 8) | Gray);
                        Image1->Canvas->Pixels[x][y] = RGB;
                }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonSinusoidFastClick(TObject *Sender)
{
        int x, y;
        int Amplitude;
        float Period;
        BYTE ImageData[256][256], *LinePtr;

        Image1->Picture->Bitmap = new Graphics::TBitmap;
        Image1->Picture->Bitmap->PixelFormat = pf24bit;
        Image1->Picture->Bitmap->Width = 256;
        Image1->Picture->Bitmap->Height = 256;
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                {
                        Amplitude = 64*(255-y)/255;
                        Period = 100*sqrt(1/(1+(exp(0.013*x)*exp(0.027*x)/400)));
                        ImageData[x][y] = Amplitude*sin(2*M_PI/Period*x)+128;
                }
        // Copy the image data to TBitmap
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                {
                        LinePtr[x*3]   = ImageData[x][y];       // Red
                        LinePtr[x*3+1] = ImageData[x][y];       // Green
                        LinePtr[x*3+2] = ImageData[x][y];       // Blue
                }
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ButtonInvertClick(TObject *Sender)
{
   int x, y;
   BYTE* LinePtr;

   // image inverting
   for (y=0; y<=255; y++)
   {
         LinePtr=(BYTE*)Image1->Picture->Bitmap->ScanLine[y];
         for (x=0; x<=255; x++)
             LinePtr[x] = 255 - LinePtr[x];
   }
   Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonSaveClick(TObject *Sender)
{
    //This code requires "jpeg.hpp" to be included in the source file
//    SaveDialog1->Filter =
//       "Bmp files (*.bmp)|*.BMP| JPEG images (*.jpg; *.jpeg) | *.jpg; *;jpeg" ;
  SaveDialog1->Title = "Save Image";
  SaveDialog1->DefaultExt = "jpg";
//  SaveDialog1->Filter = "JPG|*.jpg";
    SaveDialog1->Filter =
       "JPEG images (*.jpg) | *.jpg; | Bmp files (*.bmp)|*.BMP" ;
  SaveDialog1->FilterIndex = 1;
    if (SaveDialog1->Execute())
    {
        AnsiString temp2 = ExtractFileName(SaveDialog1->FileName);
        AnsiString temp = ExtractFileExt(SaveDialog1->FileName);
        AnsiString Ext = temp.LowerCase();

        if (Ext.AnsiPos("jpg") > 0)  // it's a jpg
        {   //-- Decompress the jpeg image into a bitmap.
            TJPEGImage *jp = new TJPEGImage();
            try
            {
              jp->Assign(Image1->Picture->Bitmap);
              jp->SaveToFile(SaveDialog1->FileName);
            }
            __finally
            {
              delete jp;
            }
        }
        else  if (Ext.AnsiPos("bmp") > 0)
        {
           Image1->Picture->Bitmap->SaveToFile(SaveDialog1->FileName);
        }
    }

}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonZoomClick(TObject *Sender)
{
        BYTE ImageData[256][256], *LinePtr;
        BYTE Output[256][256];
        int x, y, i, j;
        int zoom=4, x0=127, y0=127;

        // Copy the image data to TBitmap
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        ImageData[x][y] = LinePtr[x];
        }
        // Calculate the output image
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                {
                        i = INT((x+(zoom-1)*x0) / zoom );
                        j = INT((y+(zoom-1)*y0) / zoom );
                        Output[x][y] = ImageData[i][j];
                }
        // copy the image back for display
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        LinePtr[x] = Output[x][y];
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonBilinearClick(TObject *Sender)
{
        BYTE ImageData[256][256], *LinePtr;
        BYTE Output[256][256];
        int x, y;
        float i, j, zoom=4.0;
        int x0=127, y0=127;

        // Copy the image data to TBitmap
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        ImageData[x][y] = LinePtr[x];
        }
        // Calculate the out image
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                {
                        i = (x+(zoom-1)*x0) / zoom;
                        j = (y+(zoom-1)*y0) / zoom;
                        Output[x][y] = Bilinear(ImageData, i, j);
                }
        // copy the image back for display
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        LinePtr[x] = Output[x][y];
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonLowPassClick(TObject *Sender)
{
        BYTE ImageData[256][256], *LinePtr;
        BYTE Output[256][256];
        int x, y;

        // Copy the image data to TBitmap
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        ImageData[x][y] = LinePtr[x];
        }
        // Calculate the low-pass image
        for (y=1; y<=254; y++)
                for (x=1; x<=254; x++)
                        Output[x][y] = Lowpass(ImageData, x, y);
        // copy the image back for display
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        LinePtr[x] = Output[x][y];
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonEdgeClick(TObject *Sender)
{
        BYTE ImageData[256][256], *LinePtr;
        BYTE Output[256][256];
        int x, y;

        // Copy the image to ImageData
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        ImageData[x][y] = LinePtr[x];
        }
        // Calculate the edge image
        for (y=1; y<=254; y++)
                for (x=1; x<=254; x++)
                        Output[x][y] = Edge(ImageData, x, y);
        // copy the image back for display
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        LinePtr[x] = Output[x][y];
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonHIstogramClick(TObject *Sender)
{
        unsigned int Histogram[256], Transform[256];
        BYTE ImageData[256][256], *LinePtr;
        int x, y, i, j, sum;

        // Copy the image to ImageData
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        ImageData[x][y] = LinePtr[x];
        }
        // initialize the histogram
        for (i=0; i<=255; i++)
                Histogram[i] = 0;
        // Construct the histogram
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                        Histogram[ImageData[x][y]]++;
        // Compute the transformation
        for (i=0; i<=255; i++)
        {
                sum = 0;
                for (j=0; j<=i; j++)
                        sum += Histogram[j];
                Transform[i] = INT(255.0*sum/(256*256));
        }
        // Transform the image
        for (y=0; y<=255; y++)
                for (x=0; x<=255; x++)
                        ImageData[x][y] = Transform[ImageData[x][y]];
        // copy the image back for display
        for (y=0; y<=255; y++)
        {
                LinePtr = (BYTE *) Image1->Picture->Bitmap->ScanLine[y];
                for (x=0; x<=255; x++)
                        LinePtr[x] = ImageData[x][y];
        }
        Image1->Refresh();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonThresholdClick(TObject *Sender)
{
   int x, y;
   BYTE* LinePtr;

   // image thresholding
   for (y=0; y<=255; y++)
   {
         LinePtr=(BYTE*)Image1->Picture->Bitmap->ScanLine[y];
         for (x=0; x<=255; x++)
         {
            if (LinePtr[x] > 128)
               LinePtr[x] = 255;
            else
               LinePtr[x] = 0;
         }
   }
   Image1->Refresh();
}

//---------------------------------------------------------------------------

void __fastcall TForm1::FreeZoomWindow()
{
    if (oX != -1)
    {
       TRect MyRect = TRect(oX,oY,lX,lY);
       Image1->Canvas->Rectangle(MyRect);
    }
    oX = -1;
    oY = -1;
    lX = -1;
    lY = -1;
}

//---------------------------------------------------------------------------

void __fastcall TForm1::Image1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if (Button == mbLeft)
    {

        Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0));
        Image1->Canvas->Pen->Style = psSolid; // solid line
        Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);
        if (RadioGroupMouse->Items->Strings[RadioGroupMouse->ItemIndex] ==
                "Bezier Poly Draw")
        {
           points[CurrentPoint].x = X;
           points[CurrentPoint].y = Y;

           if ((CurrentPoint == 0) || (BezierSequence == 2))
           {
                if (CurrentPoint > 0) NumCurves++;
                Image1->Canvas->Ellipse(X-2,Y-2, X+2, Y+2); // temporary
           }
           else // control point
               Image1->Canvas->Ellipse(X-1,Y-1, X+1, Y+1); // temporary

           BezierSequence++;
           CurrentPoint++;
           if (BezierSequence >= 3)
                BezierSequence = 0;
        }
        else if (RadioGroupMouse->Items->Strings[RadioGroupMouse->ItemIndex] ==
                "Free Pen Draw")// watch mouse move
        {
                if (CheckBoxFlickerFree->Checked)
                        PanelImage->DoubleBuffered = true;
                Image1->Canvas->MoveTo(X,Y);
                MouseDown = true;
                Image1->Canvas->Pen->Mode = pmCopy;
        }
        else if (RadioGroupMouse->Items->Strings[RadioGroupMouse->ItemIndex] ==
                "Crop / Zoom Window")// watch mouse move
        {
           Image1->Canvas->Pen->Color = clBlack;
           Image1->Canvas->Pen->Style = psDash; // dashed line
           Image1->Canvas->Pen->Width = 1;
           Image1->Canvas->Pen->Mode  = pmNotXor;
           Image1->Canvas->Brush->Style = bsClear;
           FreeZoomWindow();
           oX = X;
           oY = Y;
           lX = X;
           lY = Y;
           DrawSelectRegion = true;
           if (CheckBoxFlickerFree->Checked)
               PanelImage->DoubleBuffered = true;
        }
        EditFromX->Text = X;   // this is useful for marking positions to draw
        EditFromY->Text = Y;
    }
    else if (Button == mbRight)
    {
       if (RadioGroupMouse->Items->Strings[RadioGroupMouse->ItemIndex] ==
                "Bezier Poly Draw")  // release and draw Bezier
        {
           Image1->Canvas->MoveTo(points[0].x, points[0].y);
           int size = ((NumCurves)*3);
           Image1->Canvas->PolyBezierTo(points, size-1); //BezierSequence-1);
           BezierSequence = 0;
           NumCurves      = 0;
           CurrentPoint = 0;
        }
        else //  mark the position in the Font Edit boxes
        {
            EditTextX->Text = X;
            EditTextY->Text = Y;
            EditToX->Text = X;
            EditToY->Text = Y;
        }
    }
}

//---------------------------------------------------------------------------

void __fastcall TForm1::Image1MouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
        EditX->Text = X;
        EditY->Text = Y;
        EditPixelValue->Text = Image1->Picture->Bitmap->Canvas->Pixels[X][Y];
        if (MouseDown)     // follow movement of the mouse
        {
           Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0));
           Image1->Canvas->LineTo(X,Y);
//           Image1->Canvas->Pixels[X][Y] = clWhite;
        }
        else if (DrawSelectRegion)
        {
           TRect MyRect = TRect(oX,oY,lX,lY);
           Image1->Canvas->Rectangle(MyRect);
           lX = X;
           lY = Y;
           MyRect = TRect(oX,oY,lX,lY);
           Image1->Canvas->Rectangle(MyRect);
        }
}


//---------------------------------------------------------------------------

void __fastcall TForm1::Image1MouseUp(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
//  if (DrawSelectRegion)
//  {
//     TRect MyRect = TRect(oX,oY,lX,lY);
//     Image1->Canvas->Rectangle(MyRect);
//  }
    DrawSelectRegion = false;
    MouseDown = false;
    PanelImage->DoubleBuffered = false;
    EditToX->Text = X;
    EditToY->Text = Y;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonBorderClick(TObject *Sender)
{
   // let's draw a border around the image
   TRect MyRect(0,0,Image1->Width,Image1->Height);
   Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0)); //clBlue;
   Image1->Canvas->Pen->Style = psSolid; // solid line
   Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);
   Image1->Canvas->Brush->Style = bsClear;
   Image1->Canvas->Rectangle(MyRect);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonColorClick(TObject *Sender)
{
  ColorDialog1->Color = (TColor)EditPenColor->Text.ToIntDef(0);
  if (ColorDialog1->Execute())
  {
    EditPenColor->Text = ColorDialog1->Color;
    PanelPenColor->Color = ColorDialog1->Color;
  }
}

//---------------------------------------------------------------------------


void __fastcall TForm1::Button1Click(TObject *Sender)
{
  ColorDialog1->Color = (TColor)EditBrushColor->Text.ToIntDef(0);
  if (ColorDialog1->Execute())
  {
    EditBrushColor->Text = ColorDialog1->Color;
    PanelBrushColor->Color = ColorDialog1->Color;
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonLineToClick(TObject *Sender)
{
    Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0)); //clBlue;
    Image1->Canvas->Pen->Style = psSolid; // solid line
    Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);

    int x1 = EditFromX->Text.ToIntDef(0);
    int y1 = EditFromY->Text.ToIntDef(0);
    int x2 = EditToX->Text.ToIntDef(0);
    int y2 = EditToY->Text.ToIntDef(0);
    Image1->Canvas->MoveTo(x1,y1);
    Image1->Canvas->LineTo(x2,y2);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonRectangleClick(TObject *Sender)
{
    Image1->Canvas->Brush->Color = TColor(EditBrushColor->Text.ToIntDef(0)); //clBlue;
    if (CheckBoxClearBrush->Checked)
        Image1->Canvas->Brush->Style = bsClear;  // clear rectangle
    else
        Image1->Canvas->Brush->Style = bsSolid;
    Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0)); //clBlue;
    Image1->Canvas->Pen->Style = psSolid; // solid line
    Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);


    int x1 = EditFromX->Text.ToIntDef(0);
    int y1 = EditFromY->Text.ToIntDef(0);
    int x2 = EditToX->Text.ToIntDef(0);
    int y2 = EditToY->Text.ToIntDef(0);
    TRect rect(x1,y1,x2,y2);

    if (CheckBoxFlickerFree->Checked)
       PanelImage->DoubleBuffered = true;
    Image1->Canvas->Rectangle(rect);
    PanelImage->DoubleBuffered = false;
}

//---------------------------------------------------------------------------


void __fastcall TForm1::CheckBoxBezierClick(TObject *Sender)
{
           BezierSequence = 0;    
           NumCurves      = 0;
           CurrentPoint = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonDrawCurveyShapeClick(TObject *Sender)
{
        Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0)); //clBlue;
        Image1->Canvas->Pen->Style = psSolid; // solid line
        Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);

        // let's draw a sine wave...

        TPoint point[30];
        double pi = 3.1514927;
        point[0].x = 0;
        point[0].y = 100;
        point[1].x = 0;
        point[1].y = 100;
        for (int i = 1; i<10; i++)
        {
           int j = i*3 -1;
           point[j].x = i*10;
           point[j].y = (sin(i*(pi/2))*100) + 100;
           point[j+1].x = i*10;
           if (point[j].y > 0)
                   point[j+1].y = 300;
           else if (point[j].y == 0)
                   point[j+1].y = 0;
           else // < 0
                   point[j+1].y = 100;

           point[j+2].x = i*10;
           if (point[j].y > 0)
                   point[j+2].y = 300;
           else if (point[j].y == 0)
                   point[j+2].y = 0;
           else // < 0
                   point[j+2].y = 100;

        }
//        Image1->Canvas->MoveTo(point[0].x,point[1].y);
        Image1->Canvas->PolyBezier(point, 29); //BezierSequence-1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonDrawHexagonClick(TObject *Sender)
{
  Image1->Canvas->Brush->Color = TColor(EditBrushColor->Text.ToIntDef(0));
  if (CheckBoxClearBrush->Checked)
      Image1->Canvas->Brush->Style = bsClear;  // clear shape
  else
      Image1->Canvas->Brush->Style = bsSolid;
  Image1->Canvas->Pen->Style = psSolid; // solid line
  Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);

  int startx = EditFromX->Text.ToIntDef(0);
  int starty = EditFromY->Text.ToIntDef(0);
  int endx   = EditToX->Text.ToIntDef(0);
  int endy   = EditToY->Text.ToIntDef(0);

  int lengthx = abs(endx - startx);
  int lengthy = abs(endy - starty);

  TPoint points[5];
  points[0] = Point(startx, starty + (lengthy*0.4));
  points[1] = Point(startx + (lengthx*0.5), starty);
  points[2] = Point(startx + (lengthx), starty + (lengthy*0.4));
  points[3] = Point(startx + (lengthx*0.7), starty + lengthy);
  points[4] = Point(startx + (lengthx*0.3), starty + lengthy);


  if (CheckBoxFlickerFree->Checked)
     PanelImage->DoubleBuffered = true;
  Image1->Canvas->Polygon(points, 4);
  PanelImage->DoubleBuffered = false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonDrawSineWaveClick(TObject *Sender)
{
    Image1->Canvas->Pen->Color = TColor(EditPenColor->Text.ToIntDef(0)); //clBlue;
    Image1->Canvas->Pen->Style = psSolid; // solid line
    Image1->Canvas->Pen->Width = EditPenWidth->Text.ToIntDef(1);

    const int maxpts = 19;
    TPoint pts[maxpts];
    double pi_3 = 3.1415926535897932384626433832795 / 3.0;

    int startx = EditFromX->Text.ToIntDef(0);
    int starty = EditFromY->Text.ToIntDef(0);

    for (int i = 0; i<maxpts; i++)
    {
       pts[i].x = (i+1)*10 + startx;
       pts[i].y = (sin(i*pi_3)*200) + starty; //300; //get back to axis (pi) every 3rd one
    }

    Image1->Canvas->PolyBezier(EXISTINGARRAY(pts));

    /* Show the points as well as the curve */
    for (int i = 0; i<maxpts; i++)
    {
            Image1->Canvas->Rectangle(pts[i].x-3,pts[i].y-3,pts[i].x+3,pts[i].y+3);
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonFontClick(TObject *Sender)
{
  FontDialog1->Font->Color =   TColor(EditFontColor->Text.ToIntDef(0));
  if (FontDialog1->Execute())
  {
    EditFont->Text = FontDialog1->Font->Name;
    Image1->Canvas->Font->Name = FontDialog1->Font->Name;
    Image1->Canvas->Font->Style = FontDialog1->Font->Style;
    Image1->Canvas->Font->Size = FontDialog1->Font->Size;
    Image1->Canvas->Font->Color = FontDialog1->Font->Color;
  }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonApplyTextClick(TObject *Sender)
{
    Image1->Canvas->Brush->Color = TColor(EditBrushColor->Text.ToIntDef(0)); //clBlue;
    if (CheckBoxClearBrush->Checked)
        Image1->Canvas->Brush->Style = bsClear;  // clear rectangle
    else
        Image1->Canvas->Brush->Style = bsSolid;
    Image1->Canvas->Font->Name  = EditFont->Text;
    Image1->Canvas->Font->Color = TColor(EditFontColor->Text.ToIntDef(0));

    int x = EditTextX->Text.ToIntDef(0);
    int y = EditTextY->Text.ToIntDef(0);
    Image1->Canvas->TextOut(x,y,EditText->Text);
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ButtonRotateClick(TObject *Sender)
{
    //create and setup a temporary source bitmap
    Graphics::TBitmap *source   = new Graphics::TBitmap;
    source->Assign(Image1->Picture->Bitmap);
    source->PixelFormat  =  Image1->Picture->Bitmap->PixelFormat;

    // create and setup a temporary destination bitmap
    Graphics::TBitmap *dest     = new Graphics::TBitmap;
    dest->Width                 = source->Height;
    dest->Height                = source->Width;
    dest->PixelFormat           = source->PixelFormat;

    if (RadioButtonPixelsArray->Checked) //Rotate one pixel at a time
    {
        for (int x=0;x<source->Width;x++)
          for(int y=0;y<source->Height;y++)
            dest->Canvas->Pixels[y][source->Width-1-x] =
                    source->Canvas->Pixels[x][y];
    }
    else  // uses the faster scanline method
    {
      RGBTRIPLE* pixels;
      TColor color;
      for(int y=0;y<source->Height;y++)
      {
        pixels = (RGBTRIPLE*)source->ScanLine[y];
        for (int x=0;x<source->Width;x++)
           dest->Canvas->Pixels[y][source->Width-1-x] =
              TColor(RGB(pixels[x].rgbtRed,
                      pixels[x].rgbtGreen,
                      pixels[x].rgbtBlue));
      }
    }

    //now assign destination bitmap back to Image1 & cleanup
    Image1->Picture->Bitmap = dest; 
    delete dest;
    delete source;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonCropToSelectionClick(TObject *Sender)
{
 // let's crop

    if (lX == -1) return; // no zoom window to work with

    // create and setup a temporary destination bitmap
    Graphics::TBitmap *dest     = new Graphics::TBitmap;
    dest->Width                 = abs(lX - oX);
    dest->Height                = abs(lY - oY);

    TRect  OldOne = Rect(oX,oY,lX,lY);
    TRect  NewOne = Rect(0,0,dest->Width, dest->Height);

    FreeZoomWindow();

    //create and setup a temporary source bitmap
    Graphics::TBitmap *source   = new Graphics::TBitmap;
    source->Assign(Image1->Picture->Bitmap);
    source->PixelFormat  =  Image1->Picture->Bitmap->PixelFormat;

    dest->PixelFormat           = source->PixelFormat;
    dest->Canvas->CopyRect(NewOne,source->Canvas,OldOne);

    //now assign destination bitmap back to Image1 & cleanup
    Image1->Picture->Bitmap->FreeImage();
    Image1->Picture->Bitmap->Assign(dest);
    delete dest;
    delete source;

}
//---------------------------------------------------------------------------


void __fastcall TForm1::CheckBoxStretchImageClick(TObject *Sender)
{
  Image1->Stretch = CheckBoxStretchImage->Checked;
}

//---------------------------------------------------------------------------


void __fastcall TForm1::Button2Click(TObject *Sender)
{
  ColorDialog1->Color = (TColor)EditFontColor->Text.ToIntDef(0);
  if (ColorDialog1->Execute())
  {
    EditFontColor->Text = ColorDialog1->Color;
    PanelFontColor->Color = ColorDialog1->Color;
  }

}
//---------------------------------------------------------------------------

