
/****************************************************************************\

    MCIManip.cpp 
    Damon Chandler <dmc27@cornell.edu>

    A simple MCI wrapper library

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

\****************************************************************************/

#include <windows.h>
#pragma hrdstop

#include "MCIManip.h"
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

bool mciCheck(DWORD AErrorNum, bool AReport)
{
    if (AErrorNum == MMSYSERR_NOERROR) return true;
    if (AReport)
    {
        char buffer[MAXERRORLENGTH];
        mciGetErrorString(AErrorNum, buffer, MAXERRORLENGTH);
        MessageBox(NULL, buffer, "MCI Error", MB_OK | MB_ICONERROR);
    }
    return false;
}
//---------------------------------------------------------------------------

int mciGetNumDevices(WORD ADeviceType)
{
    unsigned long num_devices = 0;
    
    MCI_SYSINFO_PARMS msip;
    memset(&msip, 0, sizeof(MCI_SYSINFO_PARMS));
    msip.lpstrReturn = reinterpret_cast<LPSTR>(&num_devices);
    msip.dwRetSize = sizeof(unsigned long);
    msip.wDeviceType = ADeviceType;

    mciSendCommand(NULL, MCI_SYSINFO, MCI_SYSINFO_QUANTITY,
                   reinterpret_cast<DWORD>(&msip));

    return num_devices;
}
//---------------------------------------------------------------------------

bool mciOpen(MCIDEVICEID& ADevID, const char* AFileName,
    const char* ADevType, HWND AHCallback, bool AReport)
{
    MCI_OPEN_PARMS mop;
    memset(&mop, 0, sizeof(MCI_OPEN_PARMS));
    mop.dwCallback = reinterpret_cast<DWORD>(AHCallback);    
    mop.lpstrElementName = const_cast<char*>(AFileName);
    mop.lpstrDeviceType = const_cast<char*>(ADevType);

    DWORD flags = 0;
    if (AHCallback) flags = flags | MCI_NOTIFY;    
    if (AFileName) flags = flags | MCI_OPEN_ELEMENT;
    if (ADevType) flags = flags | MCI_OPEN_TYPE;
     
    if (mciCheck(mciSendCommand(NULL, MCI_OPEN, flags,
                                reinterpret_cast<DWORD>(&mop)), AReport))
    {
        ADevID = mop.wDeviceID;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------

bool mciClose(MCIDEVICEID ADeviceID, HWND AHCallback, bool AReport)
{
    MCI_GENERIC_PARMS mgp;
    mgp.dwCallback = reinterpret_cast<DWORD>(AHCallback);

    DWORD flags = 0;
    if (AHCallback) flags = flags | MCI_NOTIFY;

    return mciCheck(mciSendCommand(ADeviceID, MCI_CLOSE,
                    flags, reinterpret_cast<DWORD>(&mgp)), AReport);
}
//---------------------------------------------------------------------------

bool mciPlay(MCIDEVICEID ADeviceID, DWORD AStart, DWORD AStop,
    HWND AHCallback, bool AReport)
{
    MCI_PLAY_PARMS mpp;
    memset(&mpp, 0, sizeof(MCI_PLAY_PARMS));
    mpp.dwCallback = reinterpret_cast<DWORD>(AHCallback);
    mpp.dwFrom = AStart;
    mpp.dwTo = AStop;

    DWORD flags = 0;
    if (AHCallback) flags = flags | MCI_NOTIFY;    
    if (static_cast<int>(AStart) >= 0) flags = flags | MCI_FROM;
    if (static_cast<int>(AStop) >= 0) flags = flags | MCI_TO;

    return mciCheck(mciSendCommand(ADeviceID, MCI_PLAY, flags,
                    reinterpret_cast<DWORD>(&mpp)), AReport);
}
//---------------------------------------------------------------------------

bool mciStop(MCIDEVICEID ADeviceID, HWND AHCallback, bool AReport)
{
    MCI_GENERIC_PARMS mgp;
    mgp.dwCallback = reinterpret_cast<DWORD>(AHCallback);

    DWORD flags = 0;
    if (AHCallback) flags = flags | MCI_NOTIFY;

    return mciCheck(mciSendCommand(ADeviceID, MCI_STOP, flags,
                        reinterpret_cast<DWORD>(&mgp)), AReport);
}
//---------------------------------------------------------------------------

bool mciSeek(MCIDEVICEID ADeviceID, DWORD APos, HWND AHCallback,
    bool AReport)
{
    MCI_SEEK_PARMS msp;
    memset(&msp, 0, sizeof(MCI_SEEK_PARMS));
    msp.dwCallback = reinterpret_cast<DWORD>(AHCallback);
    msp.dwTo = APos;

    DWORD flags = MCI_TO;
    if (AHCallback) flags = flags | MCI_NOTIFY;

    return mciCheck(mciSendCommand(ADeviceID, MCI_SEEK, flags,
                        reinterpret_cast<DWORD>(&msp)), AReport);
}
//---------------------------------------------------------------------------

bool mciPause(MCIDEVICEID ADeviceID, HWND AHCallback, bool AReport)
{
    MCI_GENERIC_PARMS mgp;
    mgp.dwCallback = reinterpret_cast<DWORD>(AHCallback);

    DWORD flags = 0;
    if (AHCallback) flags = flags | MCI_NOTIFY;

    return mciCheck(mciSendCommand(ADeviceID, MCI_PAUSE, flags,
                        reinterpret_cast<DWORD>(&mgp)), AReport);
}
//---------------------------------------------------------------------------

bool mciSetTimeFormat(MCIDEVICEID ADeviceID, DWORD ATimeFormat,
    HWND AHCallback, bool AReport)
{
    MCI_SET_PARMS msp;
    memset(&msp, 0, sizeof(MCI_SET_PARMS));
    msp.dwCallback = reinterpret_cast<DWORD>(AHCallback);
    msp.dwTimeFormat = ATimeFormat;

    DWORD flags = MCI_SET_TIME_FORMAT;
    if (AHCallback) flags = flags | MCI_NOTIFY;

    return mciCheck(mciSendCommand(ADeviceID, MCI_SET, flags,
                        reinterpret_cast<DWORD>(&msp)), AReport);
}
//---------------------------------------------------------------------------

bool mciStatus(MCIDEVICEID ADeviceID, DWORD AQueryGroup,
    DWORD AQueryItem, DWORD AQueryTrack, DWORD& AResult, bool AReport)
{
    MCI_STATUS_PARMS msp;
    memset(&msp, 0, sizeof(MCI_STATUS_PARMS));
    msp.dwItem = AQueryItem;
    msp.dwTrack = AQueryTrack;

    DWORD flags = AQueryGroup;
    if (mciCheck(mciSendCommand(ADeviceID, MCI_STATUS, flags,
                    reinterpret_cast<DWORD>(&msp)), AReport))
    {
        AResult = msp.dwReturn;
        return true;
    }
    return false;
}
//---------------------------------------------------------------------------


//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
//    [EOF]
//><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>

