
/****************************************************************************\
    --------------------------------------------------------
     PNGDIB_Demo.cpp -- Created 08/23/2000 by Damon Chandler
    --------------------------------------------------------

    This sample project demonstrates the use of the PNGDIB library
    to read/write a PNG file into/from a TBitmap object.  This
    library requires the libpng library, which requires the zlib
    compression library (both are free, with source).  Both of
    these libraries have been converted to BCB import libraries
    (libpng.lib and zlib.lib, respectively) using IMPLIB.EXE.

    (Note: I modified the file "pngdib.h" to work with BCB5.
     All modifications are indicated by 'DC'.)
    
\****************************************************************************/

#include <vcl.h>
#pragma hdrstop

// include the PNGDIB sources (in 'pngdib' folder)
#include "pngdib.h"
// link with the ZLIB library (in 'zlib' folder)
#pragma link "zlib.lib"
// link with the LIBPNG library (in 'lpng108' folder)
#pragma link "libpng.lib"

#include "PNGDIB_Demo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TForm1 *Form1;
//---------------------------------------------------------------------------

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PNG2BmpButtonClick(TObject *Sender)
{
   if (OpenDialog1->Execute())
   {
      TCHAR filename[MAX_PATH];
      lstrcpyn(filename, OpenDialog1->FileName.c_str(), MAX_PATH);

      // declare and clear the PNGD_P2DINFO structure
      PNGD_P2DINFO png2dib;
	   memset(&png2dib, 0, sizeof(PNGD_P2DINFO));

      // initialize the structure size and filename
   	png2dib.structsize = sizeof(PNGD_P2DINFO);
	   png2dib.pngfn = filename;

      // convert from PNG to DIB
   	if (read_png_to_dib(&png2dib) == PNGD_E_SUCCESS)
      {
         Graphics::TBitmap* Bitmap = Image1->Picture->Bitmap;
         Bitmap->Width = png2dib.lpdib->biWidth;
         Bitmap->Height = png2dib.lpdib->biHeight;

         HBITMAP hBmp = Bitmap->ReleaseHandle();
         HDC hDC = Canvas->Handle;
         try
         {
            //
            // TODO: add palette support...
            //

            // convert from DIB to TBitmap
            SetDIBits(
               hDC, hBmp, 0,
               png2dib.lpdib->biHeight, png2dib.bits,
               reinterpret_cast<LPBITMAPINFO>(png2dib.lpdib),
               DIB_RGB_COLORS
               );
         }
         catch (...)
         {
            Bitmap->Handle = hBmp;
            GlobalFree(png2dib.lpdib);
         }
         Bitmap->Handle = hBmp;         
         GlobalFree(png2dib.lpdib);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Bmp2PNGButtonClick(TObject *Sender)
{
   if (SaveDialog1->Execute())
   {
      TCHAR filename[MAX_PATH];
      lstrcpyn(filename, SaveDialog1->FileName.c_str(), MAX_PATH);

      BITMAPINFO bmi;
      Graphics::TBitmap* Bitmap = Image1->Picture->Bitmap;

      //
      // determine the size of the DIB info
      // (BITMAPINFOHEADER + color table) and the
      // size of the bits (pixels)
      //
      unsigned int info_size = 0, bits_size = 0;
      GetDIBSizes(Bitmap->Handle, info_size, bits_size);

      // allocate memory for the bits
      unsigned char *bits = new unsigned char[bits_size];
      try
      {
         // get the BITMAPINFOHEADER + color table and the bits
         if (GetDIB(Bitmap->Handle, Bitmap->Palette, &bmi, bits))
         {
            // declare and clear the PNGD_D2PINFO structure
            PNGD_D2PINFO dib2png;
            memset(&dib2png, 0, sizeof(PNGD_D2PINFO));

            // initialize the structure            
            dib2png.structsize = sizeof(PNGD_D2PINFO);
            dib2png.flags = PNGD_INTERLACED;
            dib2png.pngfn = filename;
            dib2png.lpdib = &bmi.bmiHeader;
            dib2png.lpbits = bits;

            // convert the DIB to PNG, then save to file
            if (write_dib_to_png(&dib2png) != PNGD_E_SUCCESS)
            {
               throw EInvalidGraphic("Error Saving PNG!");
            }
         }
      }
      catch (...)
      {
         delete [] bits;
      }
      delete [] bits;
   }
}
//---------------------------------------------------------------------------
