
/****************************************************************************\
    --------------------------------------------------------
     VideoDemo.cpp -- Created 05/19/2000 by Damon Chandler
    --------------------------------------------------------
    
    This sample project demonstrates the use of the MCI in creating a
    simple digital video player.  As this demo relies on the MCI, it
    is required that the target platform has the correct codecs installed.
    Refer to the following KB article for more info:
    
    http://support.microsoft.com/support/kb/articles/Q142/7/31.ASP

    This demo uses the supplied TMCIWndFrame class (WndFrame.cpp,
    WndFrame.h).  These latter classes rely on the "MCIWnd Window Class",
    a set of functions and macros provided by Microsoft to simplify use
    of the MCI in developing visually-based applications, especially
    when working with video.  TMCIWndFrame is a TFrame descendant class.

\****************************************************************************/

#include <vcl.h>
#pragma hdrstop

#include "VideoDemo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "WndFrame"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------

__fastcall TForm1::TForm1(TComponent* Owner)
   : TForm(Owner)
{
   //
   // use the OnPosChanged event to perform special
   // processing when the position (frame) changes
   //
   MCIWndFrame1->OnPosChanged = MCIWndFrame1PosChanged;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MCIWndFrame1PosChanged(TObject* ASender,
   long APosition)
{
   // extract the display string
   AnsiString Display(MCIWndFrame1->DisplayString);

   // extract the current frame number
   long current_pos = MCIWndFrame1->Position;
   // extract the total number of frames
   long end_pos = MCIWndFrame1->VideoLength;

   // update the Form caption
   Caption = Display + " -- " +
             IntToStr(current_pos) + " of " +
             IntToStr(end_pos) + " frames";

   // back out of full-screen mode when done
   if (current_pos == end_pos)
   {
      MCIWndFrame1->FullScreen = false;
      ToolPanel->Visible = !MCIWndFrame1->FullScreen;      
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OpenButtonClick(TObject *Sender)
{
   if (OpenDialog1->Execute())
   {
      // assign the name of the file to play
      MCIWndFrame1->FileName = OpenDialog1->FileName;
      if (!MCIWndFrame1->Error)
      {
         // begin playback
         MCIWndFrame1->Play();
         FullScreenButton->Enabled = true;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FullScreenButtonClick(TObject *Sender)
{
   // place the frame into full-screen mode
   MCIWndFrame1->FullScreen = true;
   ToolPanel->Visible = !MCIWndFrame1->FullScreen;
}
//---------------------------------------------------------------------------

//
// use the TApplication::OnMessage event to process the ESC keystroke,
// as the MCI window and its controls are not VCL controls
//
void __fastcall TForm1::ApplicationEvents1Message(tagMSG &Msg,
   bool &Handled)
{
   if (Msg.message == WM_CHAR && Msg.wParam == VK_ESCAPE)
   {
      // ESC keystroke cancels full-screen mode
      MCIWndFrame1->FullScreen = false;
      ToolPanel->Visible = !MCIWndFrame1->FullScreen;
   }
}
//---------------------------------------------------------------------------



