
/****************************************************************************\

    CDDevice.cpp 
    Damon Chandler <dmc27@cornell.edu>

    A MCIWnd-tuned TFrame descendant class

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

\****************************************************************************/

#include <vcl.h>
#pragma hdrstop

#include <vfw.h>
#include "WndFrame.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMCIWndFrame *MCIWndFrame;
//---------------------------------------------------------------------------

__fastcall TMCIWndFrame::TMCIWndFrame(TComponent* Owner)
   : TFrame(Owner), HMCIWnd_(NULL), error_(false), position_(0),
   fullscreen_(false)
{
}
//---------------------------------------------------------------------------

__fastcall TMCIWndFrame::~TMCIWndFrame()
{
   if (HMCIWnd_)
   {
      MCIWndClose(HMCIWnd_);
      MCIWndDestroy(HMCIWnd_);
   }
}
//---------------------------------------------------------------------------

//
// inherited protected member functions
//

void __fastcall TMCIWndFrame::CreateWnd()
{
   TFrame::CreateWnd();

   const DWORD dwStyle =
      MCIWNDF_NOAUTOSIZEWINDOW | MCIWNDF_NOAUTOSIZEMOVIE |
      MCIWNDF_NOOPEN | MCIWNDF_NOTIFYALL |
      MCIWNDF_SHOWMODE | MCIWNDF_SHOWNAME;
      
   HMCIWnd_ = MCIWndCreate(Handle, HInstance, dwStyle, filename_.c_str());
   if (HMCIWnd_)
   {
      SetClassLong(HMCIWnd_, GCL_HBRBACKGROUND,
                   reinterpret_cast<DWORD>(Brush->Handle));
                   
      DoSetTimeFormat();
      DoSetTimers();
      DoResize();
      if (position_)
      {
         DoSeek(position_);
         DoPlay();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DestroyWnd()
{
   MCIWndClose(HMCIWnd_);
   MCIWndDestroy(HMCIWnd_);
   HMCIWnd_ = NULL;

   TFrame::DestroyWnd();   
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::SetBounds(int ALeft, int ATop, int AWidth,
   int AHeight)
{
   TFrame::SetBounds(ALeft, ATop, AWidth, AHeight);
   if (!ComponentState.Contains(csDesigning)) DoResize();
}
//---------------------------------------------------------------------------


//
// message handlers
//

void __fastcall TMCIWndFrame::MCINotifyPos(TMessage& AMsg)
{
   position_ = AMsg.LParam;
   if (OnPosChanged_) OnPosChanged_(this, position_);
}
//---------------------------------------------------------------------------


//
// private member functions
//

AnsiString __fastcall TMCIWndFrame::GetDisplayString()
{
   char buffer[MAX_PATH];
   if (GetWindowText(HMCIWnd_, buffer, MAX_PATH)) return AnsiString(buffer);
   return AnsiString("");   
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::SetFullScreen(bool AValue)
{
   if (fullscreen_ != AValue)
   {
      fullscreen_ = AValue;
      DoSetFullScreen(AValue);
   }
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::SetFileName(AnsiString AValue)
{
   error_ = MCIWndOpen(HMCIWnd_, AValue.c_str(), 0);
   filename_ = AValue;
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::SetPosition(long AValue)
{
   if (position_ != AValue)
   {
      DoSeek(AValue);
      position_ = AValue;
   }
}
//---------------------------------------------------------------------------

long __fastcall TMCIWndFrame::GetLength()
{
   if (HMCIWnd_) return MCIWndGetEnd(HMCIWnd_);
   return -1;
}
//---------------------------------------------------------------------------

// not used by demo, but useful nonetheless
long __fastcall TMCIWndFrame::GetSpeed()
{
   if (HMCIWnd_) return MCIWndGetSpeed(HMCIWnd_);
   return -1;
}
//---------------------------------------------------------------------------

// not used by demo, but useful nonetheless
void __fastcall TMCIWndFrame::SetSpeed(long AValue)
{
   error_ = MCIWndSetSpeed(HMCIWnd_, AValue);
}
//---------------------------------------------------------------------------

// not used by demo, but useful nonetheless
long __fastcall TMCIWndFrame::GetVolume()
{
   if (HMCIWnd_) return MCIWndGetVolume(HMCIWnd_);
   return -1;
}
//---------------------------------------------------------------------------

// not used by demo, but useful nonetheless
void __fastcall TMCIWndFrame::SetVolume(long AValue)
{
   error_ = MCIWndSetVolume(HMCIWnd_, AValue);
}
//---------------------------------------------------------------------------


//
// introduced protected member functions
//

RECT __fastcall TMCIWndFrame::DoCalcDestRect()
{
   RECT RDest = {0, 0, 0, 0};
   if (MCIWndGetDest(HMCIWnd_, &RDest) == MMSYSERR_NOERROR)
   {
      // resize the video display to fit the MCI window
      RECT RMCIClient;
      if (::GetClientRect(HMCIWnd_, &RMCIClient))
      {
         RDest.right = RDest.left + (RMCIClient.right - RMCIClient.left);
         RDest.bottom = RDest.top + (RMCIClient.bottom - RMCIClient.top);
      }

      // account for toolbar, if visible
      RECT RChildClient;
      HWND HChild = GetWindow(HMCIWnd_, GW_CHILD);
      if (HChild && IsWindowVisible(HChild) &&
          ::GetClientRect(HChild, &RChildClient))
      {
         RDest.bottom = RDest.top +
            (RMCIClient.bottom - RMCIClient.top) -
            (RChildClient.bottom - RChildClient.top);
      }
   }
   return RDest;
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoSetTimeFormat()
{
   // set the time format to frames
   if (HMCIWnd_) MCIWndSetTimeFormat(HMCIWnd_, "frames");
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoSetTimers()
{
   // set the interval of the internal timers
   if (HMCIWnd_) MCIWndSetTimers(HMCIWnd_, 500, 500);
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoResize()
{
   // size the MCI window to fit the frame
   MoveWindow(HMCIWnd_, 0, 0, Width, Height, true);
   DoHideToolBar(fullscreen_);

   // size the video display to fit the frame   
   error_ = MCIWndPutDest(HMCIWnd_, &DoCalcDestRect());
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoPlay()
{
   // play the video
   error_ = MCIWndPlay(HMCIWnd_);
   DoResize();   
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoSeek(long APosition)
{
   if (HMCIWnd_)
   {
      // seek to a new position (in frames)
      error_ = MCIWndSeek(HMCIWnd_, APosition);
      if (position_) DoPlay();
   }
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoHideToolBar(bool AHide)
{
   HWND HToolBar = FindWindowEx(HMCIWnd_, NULL, "ToolbarWindow32", NULL);
   if (HToolBar) ShowWindow(HToolBar, AHide ? SW_HIDE : SW_SHOWDEFAULT);

   HWND HTrackBar = FindWindowEx(HMCIWnd_, NULL, "msctls_trackbar32", NULL);   
   if (HTrackBar) ShowWindow(HTrackBar, AHide ? SW_HIDE : SW_SHOWDEFAULT);   
}
//---------------------------------------------------------------------------

void __fastcall TMCIWndFrame::DoSetFullScreen(bool AFullScreen)
{
   TCustomForm* ParentForm =
      static_cast<TCustomForm*>(GetParentForm(this));

   if (ParentForm)
   {
      DoHideToolBar(fullscreen_);   
      if (AFullScreen)
      {
         SavedBoundsRect_ = ParentForm->BoundsRect;
         SavedStyle_ = GetWindowLong(ParentForm->Handle, GWL_STYLE);

         SetWindowLong(ParentForm->Handle, GWL_STYLE,
                       WS_POPUP | WS_VISIBLE);
         SetWindowPos(ParentForm->Handle, HWND_TOPMOST, 0, 0,
                      Screen->Width, Screen->Height,
                      SWP_FRAMECHANGED);
      }
      else
      {
         SetWindowLong(ParentForm->Handle, GWL_STYLE, SavedStyle_);
         SetWindowPos(ParentForm->Handle, HWND_BOTTOM, 0, 0, 0, 0,
                      SWP_FRAMECHANGED | SWP_NOMOVE | SWP_NOSIZE);
         SetWindowPos(ParentForm->Handle, HWND_TOP, 0, 0, 0, 0,
                   SWP_NOMOVE | SWP_NOSIZE);

         ParentForm->BoundsRect = SavedBoundsRect_;
      }
   }
}
//---------------------------------------------------------------------------






