/*---------------------------------------------------------------------------
  FactoryForm.cpp
        C++Builder Developer's Guide
        Chapter 16 - Shared Segmentation DLL Example
        Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FactoryForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormFactory *FormFactory;


#define SHAREDSEG_LINK
//#define VISUALCPP_DLL   //remove comment to load Visual C++ DLL 
                        //rather than BCB DLL - make sure you do this
                        //to DealerForm.cpp too

#ifdef SHAREDSEG_LINK
HINSTANCE SHAREDSEGDLL;


#define all -1
#define accord 0
#define civic 1
#define crv 2
#define pilot 3

typedef int (*CARCOUNT)(int, int);
CARCOUNT CarCount;

typedef int (*BUILDRETURNCAR)(int, int);
BUILDRETURNCAR BuildReturnCar;

typedef int (*OPENFORBUSINESS)(HWND);
OPENFORBUSINESS OpenForBusiness;

typedef int (*NUMDEALERSHIPS)();
NUMDEALERSHIPS NumDealerships;
#endif


//---------------------------------------------------------------------------
__fastcall TFormFactory::TFormFactory(TComponent* Owner)
        : TForm(Owner)
{
        myid = -1;
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::FormClose(TObject *Sender,
      TCloseAction &Action)
{
#ifdef SHAREDSEG_LINK
    if (SHAREDSEGDLL != NULL)
    {
         FreeLibrary(SHAREDSEGDLL);
    }
#endif
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::FormCreate(TObject *Sender)
{
    #ifdef SHAREDSEG_LINK

    #ifdef VISUALCPP_DLL
      SHAREDSEGDLL = LoadLibrary("InventoryVC");
    #else
        SHAREDSEGDLL = LoadLibrary("Inventory");
    #endif
    if (SHAREDSEGDLL != NULL)
    {
        CarCount  = (CARCOUNT)GetProcAddress(SHAREDSEGDLL,"CarCount");
        BuildReturnCar =
           (BUILDRETURNCAR)GetProcAddress(SHAREDSEGDLL,"BuildReturnCar");
        OpenForBusiness=
           (OPENFORBUSINESS)GetProcAddress(SHAREDSEGDLL,"OpenForBusiness");
        NumDealerships =
           (NUMDEALERSHIPS)GetProcAddress(SHAREDSEGDLL,"NumDealerships");

        if ((!CarCount) || (!BuildReturnCar))
        {
          // handle the error
            MessageBeep(MB_OK);
            FreeLibrary(SHAREDSEGDLL);
            SHAREDSEGDLL = NULL;
        }
        else
        {
           myid = OpenForBusiness(Handle);
           // let's get all the dealer information..
            TMessage msg;
            msg.WParam = myid;
            msg.LParam = accord;
            wmCarChange(msg);
            msg.LParam = civic;
            wmCarChange(msg);
            msg.LParam = crv;
            wmCarChange(msg);
            msg.LParam = pilot;
            wmCarChange(msg);
        }
    }
    else
       MessageBox(NULL,"Couldn't find the necessary DLL","Test One",MB_OK);
    #endif
}


//---------------------------------------------------------------------------

void __fastcall TFormFactory::wmCarChange(TMessage Msg)
{
    int id = Msg.WParam;
    int car = (Msg.LParam);

    int count = 0;
    if (id == myid)
    {
        count = CarCount(car, 0);
        // let's figure out what model it is...
        switch (car)
        {
           case accord  : EditFactoryAccords->Text = AnsiString(count); break;
           case civic   : EditFactoryCivics->Text = AnsiString(count); break;
           case crv     : EditFactoryCRVs->Text = AnsiString(count); break;
           case pilot   : EditFactoryPilots->Text = AnsiString(count); break;
        }
    }
    count = CarCount(car, 1);    // all dealers
    switch (car)
    {
           case accord  : EditAllAccords->Text = AnsiString(count); break;
           case civic   : EditAllCivics->Text = AnsiString(count); break;
           case crv     : EditAllCRVs->Text = AnsiString(count); break;
           case pilot   : EditAllPilots->Text = AnsiString(count); break;
     }
    count = CarCount(car, 2);    // factory
    switch (car)
    {
           case accord  : EditFactoryAccords->Text = AnsiString(count); break;
           case civic   : EditFactoryCivics->Text = AnsiString(count); break;
           case crv     : EditFactoryCRVs->Text = AnsiString(count); break;
           case pilot   : EditFactoryPilots->Text = AnsiString(count); break;
     }
    count = CarCount(car, 3);    // sold
    switch (car)
    {
           case accord  : EditSoldAccords->Text = AnsiString(count); break;
           case civic   : EditSoldCivics->Text = AnsiString(count); break;
           case crv     : EditSoldCRVs->Text = AnsiString(count); break;
           case pilot   : EditSoldPilots->Text = AnsiString(count); break;
     }
   Timer1Timer(Application); // update # of dealers
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::Button1Click(TObject *Sender)
{
  BuildReturnCar(accord,EditQuantity->Text.ToIntDef(1));
}

//---------------------------------------------------------------------------
void __fastcall TFormFactory::Button2Click(TObject *Sender)
{
  BuildReturnCar(civic,EditQuantity->Text.ToIntDef(1));
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::Button3Click(TObject *Sender)
{
  BuildReturnCar(crv,EditQuantity->Text.ToIntDef(1));
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::Button4Click(TObject *Sender)
{
  BuildReturnCar(pilot,EditQuantity->Text.ToIntDef(1));
}
//---------------------------------------------------------------------------

void __fastcall TFormFactory::ButtonOpenNewDealershipClick(TObject *Sender)
{
  WinExec ("Dealership.exe", SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::ButtonCloseClick(TObject *Sender)
{
        Close();
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::Timer1Timer(TObject *Sender)
{
  EditDealerships->Text = AnsiString(NumDealerships());
}
//---------------------------------------------------------------------------
void __fastcall TFormFactory::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
  if (NumDealerships() > 0)
  {
        MessageBox(NULL,
                "Can't close factory, there are dealerships still open.",
                "Improper Business Operation",
                MB_OK);
        CanClose = false;
  }
  else
        CanClose = true;

}
//---------------------------------------------------------------------------

