/*---------------------------------------------------------------------------
  InventoryUnitVC.cpp
        C++Builder Developer's Guide
        Chapter 16 - Shared Segmentation DLL Example (Visual C++ example)
        Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------

#include <windows.h>
#pragma hdrstop
#include <stdio.h>
#include <time.h>

#define DLL_LINK

#include "InventoryUnit.h"
#include "InventoryDataTypes.h"


#pragma data_seg(".sdata")	// start of section -- must initialize!

// Here is the initialized data that will be shared.
   int numAutoDealers   = 0;
   SAuto AllDealers     = {0,0,0, 0}; //false};
   SAuto TotalSold      = {0,0,0,0};
   SAuto Factory        = {100, 100, 100, 100}; // start off with 100 cars
   int NextDealerID     = 0;
   SDealerFactory DealerFactory[MaxDealers] = {false, NULL};
   bool	InventoryLock	= false;

#pragma data_seg()

// these will NOT be global/shared to all modules (important only to attached component).

    SAuto LocalDealer = {10, 10, 10, 10};
    int DealerID;

/******************************************************************************/
/*	DLLMain																	  */
/******************************************************************************/
bool WINAPI DllMain(HANDLE hModule, DWORD dwReason, LPVOID lpReserved)
{
	switch(dwReason)
	{
	case DLL_PROCESS_ATTACH:
		{			
            numAutoDealers++;
            DealerID = NextDealerID;
            NextDealerID++;
            for (int model = 0; model < MaxModels; model++) // initialize arrays
            {
                if (DealerID == 0)
                {
                    AllDealers[model] = 0;  // first dealer in, let's initialize
                    TotalSold[model] = 0;
                }
                LocalDealer[model] = 0;
            }
            break;
		}
	case DLL_THREAD_ATTACH:
            break;
	case DLL_THREAD_DETACH:
            break;
	case DLL_PROCESS_DETACH:
		{			
            DealerFactory[DealerID].Open = false;
            for (int model = 0; model < MaxModels; model++)
            {
                AllDealers[model] -= LocalDealer[model];
                Factory[model] += LocalDealer[model]; // factory gets 'em back
                LocalDealer[model] = 0;
                // notify other dealers that dealership and it's cars are gone
                for (int i= 0; i< MaxDealers; i++)
                {
                    if (DealerFactory[i].Open) // tell all dealers...
                           PostMessage(DealerFactory[i].hwnd,
                                    WM_CAR_CHANGE,
                                    (WPARAM) i,	// what dealer id changed
                                    (LPARAM) model);// what car changed
                }
            }
            numAutoDealers--;
            break;
		}
	}
	return TRUE;
}

//---------------------------------------------------------------------------

int OpenForBusiness(HWND hWindow)  // the factory and dealer use this
{
	DealerFactory[DealerID].Open = true;
	DealerFactory[DealerID].hwnd = hWindow;
	return DealerID;
}


//---------------------------------------------------------------------------

int CarCount(int model, int inventory) // 0 = local, 1 = all_dealers, 2 = factory
{
        int value = 0;
        if (model != -1)
        {
            switch (inventory)
            {
               case 0 : value = LocalDealer[model]; break; 
               case 1 : value = AllDealers[model]; break;
               case 2 : value = Factory[model]; break;
               case 3 : value = TotalSold[model]; break;
            }
        }
        else // count is wanted on all models
        {

			//int i = 0;
            switch (inventory)
            {
               case 0 :  // dealer quantity
			   {
                 for (int i = 0; i<MaxModels; i++)
                    value += LocalDealer[model];
                 break;
			   }
               case 1 :  // total dealership quantity
			   {
                 for (int i = 0; i<MaxModels; i++)
                    value += AllDealers[model];
                 break;
			   }
               case 2 :  // factory quantity
			   {
                 for (int i = 0; i<MaxModels; i++)
                    value += Factory[model];
                 break;
			   }
               case 3 :   // sold quantity
			   {
                 for (int i = 0; i<MaxModels; i++)
                    value += TotalSold[model];
                 break;
			   }
            }
        }
// -------------- begin debug code --------------
/*
        char dummy[MAX_PATH];
        sprintf(dummy,"inventory %d - model# %d - total %d",inventory,model,value);
        MessageBox(NULL,dummy,"DLL",MB_OK);
*/
// -------------- end debug code --------------
        return value;  // Car count
}

//---------------------------------------------------------------------------

int BuySellCar(int model,int quantity)
{
    unsigned int StartTime = clock();
    while (InventoryLock)
    {
        if (StartTime - clock() > MaxWaitTime)
                break;
    }
    InventoryLock = true;

    // dealer can't sell if no cars are available from dealer or factory
    if (quantity > 0)  // dealer can't sell back to factory
    {
      int value = Factory[model];
      if (value >= quantity)   // can't deplete the factory below zero
      {
          LocalDealer[model] += quantity;
          AllDealers[model] += quantity;
          Factory[model] -= quantity;
      }
      else     // can't deplete the factory below zero
      {
          LocalDealer[model] += value;
          AllDealers[model] += value;
          Factory[model] -= value;
          MessageBeep(MB_OK);    // factory doesn't have enough vehicles to ship
      }
    }
    else // dealer is selling a car
    {
      int value = LocalDealer[model];
      if (value >= abs(quantity))   // can't deplete all the dealer's  vehicles
      {
          LocalDealer[model] += quantity;
          AllDealers[model] += quantity;
          TotalSold[model] -= quantity;
      }
      else
      {
          LocalDealer[model] -= value;
          AllDealers[model] -= value;
          TotalSold[model] -= value;
          MessageBeep(MB_OK);    // dealer doesn't have enough cars to sell
      }
    }

    // notify other Dealers of local dealer's change Status

    for (int i= 0; i< MaxDealers; i++)
    {
        if (DealerFactory[i].Open) // tell all dealers...
               PostMessage(DealerFactory[i].hwnd,
                        WM_CAR_CHANGE,
                        (WPARAM) i,	// what dealer id changed
                        (LPARAM) model);// what car changed
    }
    InventoryLock = false;
    return DealerID; // no error, return store id
}

//---------------------------------------------------------------------------

int BuildReturnCar(int model, int quantity)  // used by the factory
{
    unsigned int StartTime = clock();
    while (InventoryLock)
    {
        if (StartTime - clock() > MaxWaitTime)
                break;
    }
    InventoryLock = true;
    int value = quantity;

    // dealer can't sell if no cars are available from dealer or factory
    if (quantity > 0)  // dealer can't sell back to factory
    {
          Factory[model] += quantity;
    }
    else if (DealerID != 0) // make sure factory is not returning to self
    {
      value = LocalDealer[model];
      if (value >= abs(quantity))
         value = abs(quantity);
      Factory[model] +=value;
      LocalDealer[model] -=value;
    }

    // notify other Dealers of factory's change Status

    for (int i= 0; i< MaxDealers; i++)
    {
        if (DealerFactory[i].Open) // tell all dealers...
               PostMessage(DealerFactory[i].hwnd,
                        WM_CAR_CHANGE,
                        (WPARAM) i,	// what dealer id changed
                        (LPARAM) model);// what car changed
    }
    InventoryLock = false;
    return value;
}

//---------------------------------------------------------------------------

int  NumDealerships()
{
   return numAutoDealers-1;
}