/*---------------------------------------------------------------------------
  Simple.cpp
        C++Builder Developer's Guide
        Chapter 16 - Simple DLL example - class based
        Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------
#include <vcl.h>
#include <windows.h>
#include <stdio.h>
#include "simple.h"
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#pragma argsused
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fwdreason, LPVOID lpvReserved)
//int WINAPI DllEntryPoint(HINSTANCE hinstDLL, unsigned long fwdreason, void* lpvReserved)
{
    switch(fwdreason)
    {
	case DLL_PROCESS_ATTACH:
            if (lpvReserved)
                  MessageBox(NULL,"Process has attached to DLL (Static Load)",
                      "DLLMain",MB_OK);
            else
                  MessageBox(NULL,"Process has attached to DLL (Dynamic Load)",
                        "DLLMain",MB_OK);
            break;
	case DLL_THREAD_ATTACH:
            MessageBox(NULL,"Thread has attached to DLL","DLLMain",MB_OK);
            break;
	case DLL_THREAD_DETACH:
            MessageBox(NULL,"Thread has detached to DLL","DLLMain",MB_OK);
            break;
	case DLL_PROCESS_DETACH:
            MessageBox(NULL,"Process has detached to DLL","DLLMain",MB_OK);
            break;
    }
    return 1; // always return true;
}

//---------------------------------------------------------------------------

__fastcall TSimpleClass::TSimpleClass()
{
   FVersion = 1.1;
}

//---------------------------------------------------------------------------

__fastcall TSimpleClass::~TSimpleClass()
{

}

//---------------------------------------------------------------------------

__fastcall TDistance::TDistance()
{
   FVersion = 1.1;
   FFeet = 0.0;
   FMeters = 0.0;
}

//---------------------------------------------------------------------------

__fastcall TDistance::~TDistance()
{

}

//---------------------------------------------------------------------------

double TDistance::GetLibVersion()  //Retrieves the current library version of the DLL.
{
	return FVersion;
}

//---------------------------------------------------------------------------

void __fastcall TDistance::SetMeters(double value)
{
   FMeters = value;
   last_set = meters;
}

//---------------------------------------------------------------------------

void __fastcall TDistance::SetFeet(double value)
{
   FFeet = value;
   last_set = feet;
}

//---------------------------------------------------------------------------


double __fastcall  TDistance::feet_to_meters()
{
   if (last_set == meters)
        return FMeters;
   else
	return (FFeet * 0.3048);
}

//---------------------------------------------------------------------------

double __fastcall  TDistance::meters_to_feet()
{
   if (last_set == feet)
        return FFeet;
   else
	return (FMeters * 3.2808);
}

