// INVENTORYKEEPERIMPL : Implementation of TInventoryKeeperImpl (CoClass: InventoryKeeper, Interface: IAutoInventory)

#include <vcl.h>
#pragma hdrstop
#include <stdio.h>
#include <time.h>

#include "INVENTORYKEEPERIMPL.H"
#include "InventoryKeeperUnit.h"
#pragma link "Inventory_OCX"

/////////////////////////////////////////////////////////////////////////////
// TInventoryKeeperImpl

STDMETHODIMP TInventoryKeeperImpl::BuildReturnCar(int model, int quantity)
{ 
    unsigned int StartTime = clock();
    while (Form1->InventoryLock)
    {
        if (StartTime - clock() > MaxWaitTime)
                break;
    }
    Form1->InventoryLock = true;
    int value = quantity;

    // dealer can't sell if no cars are available from dealer or factory
    if (quantity > 0)  // dealer can't sell back to factory
    {
          Form1->Factory[model] += quantity;
    }
    else if (Form1->DealerID != 0) // make sure factory is not returning to self
    {
      value = Form1->LocalDealer[Form1->DealerID][model];
      if (value >= abs(quantity))
         value = abs(quantity);
      Form1->Factory[model] +=value;
      Form1->LocalDealer[Form1->DealerID][model] -=value;
    }

    // notify other Dealers of factory's change Status

    for (int i= 0; i< MaxDealers; i++)
    {
        if (Form1->DealerFactory[i].Open) // tell all dealers...
               PostMessage(Form1->DealerFactory[i].hwnd,
                        WM_CAR_CHANGE,
                        (WPARAM) i,	// what dealer id changed
                        (LPARAM) model);// what car changed
    }
    Form1->InventoryLock = false;
//    return value;
    return S_OK;
}

STDMETHODIMP TInventoryKeeperImpl::BuySellCar(int model, int quantity)
{ 
    unsigned int StartTime = clock();
    while (Form1->InventoryLock)
    {
        if (StartTime - clock() > MaxWaitTime)
                break;
    }
    Form1->InventoryLock = true;

    // dealer can't sell if no cars are available from dealer or factory
    if (quantity > 0)  // dealer can't sell back to factory
    {
      int value = Form1->Factory[model];
      if (value >= quantity)   // can't deplete the factory below zero
      {
          Form1->LocalDealer[Form1->DealerID][model] += quantity;
          Form1->AllDealers[model] += quantity;
          Form1->Factory[model] -= quantity;
      }
      else     // can't deplete the factory below zero
      {
          Form1->LocalDealer[Form1->DealerID][model] += value;
          Form1->AllDealers[model] += value;
          Form1->Factory[model] -= value;
          MessageBeep(MB_OK);    // factory doesn't have enough vehicles to ship
      }
    }
    else // dealer is selling a car
    {
      int value = Form1->LocalDealer[Form1->DealerID][model];
      if (value >= abs(quantity))   // can't deplete all the dealer's  vehicles
      {
          Form1->LocalDealer[Form1->DealerID][model] += quantity;
          Form1->AllDealers[model] += quantity;
          Form1->TotalSold[model] -= quantity;
      }
      else
      {
          Form1->LocalDealer[Form1->DealerID][model] -= value;
          Form1->AllDealers[model] -= value;
          Form1->TotalSold[model] -= value;
          MessageBeep(MB_OK);    // dealer doesn't have enough cars to sell
      }
    }

    // notify other Dealers of local dealer's change Status

    for (int i= 0; i< MaxDealers; i++)
    {
        if (Form1->DealerFactory[i].Open) // tell all dealers...
               PostMessage(Form1->DealerFactory[i].hwnd,
                        WM_CAR_CHANGE,
                        (WPARAM) i,	// what dealer id changed
                        (LPARAM) model);// what car changed
    }
    Form1->InventoryLock = false;
    return Form1->DealerID; // no error, return store id
}

STDMETHODIMP TInventoryKeeperImpl::CarCount(int model, int inventory)
{ 
        int value = 0;
        if (model != -1)
        {
            switch (inventory)
            {
               case 0 : value = Form1->LocalDealer[Form1->DealerID][model]; break;
               case 1 : value = Form1->AllDealers[model]; break;
               case 2 : value = Form1->Factory[model]; break;
               case 3 : value = Form1->TotalSold[model]; break;
            }
        }
        else // count is wanted on all models
        {

            switch (inventory)
            {
               case 0 :  // dealer quantity
                 for (int i = 0; i<MaxModels; i++)
                    value += Form1->LocalDealer[Form1->DealerID][model];
                 break;
               case 1 :  // total dealership quantity
                 for (int i = 0; i<MaxModels; i++)
                    value += Form1->AllDealers[model];
                 break;
               case 2 :  // factory quantity
                 for (int i = 0; i<MaxModels; i++)
                    value += Form1->Factory[model];
                 break;
               case 3 :   // sold quantity
                 for (int i = 0; i<MaxModels; i++)
                    value += Form1->TotalSold[model];
                 break;
            }
        }
// -------------- begin debug code --------------
/*
        char dummy[MAX_PATH];
        sprintf(dummy,"inventory %d - model# %d - total %d",inventory,model,value);
        MessageBox(NULL,dummy,"DLL",MB_OK);
*/
// -------------- end debug code --------------
        return value;  // Car count
}

STDMETHODIMP TInventoryKeeperImpl::Join()
{
        Form1->EditDealerCount->Text = AnsiString(NumDealerships());
        return Form1->Attach();
}

STDMETHODIMP TInventoryKeeperImpl::Leave()
{
        Form1->EditDealerCount->Text = AnsiString(NumDealerships());
        return Form1->Detach();
}

STDMETHODIMP TInventoryKeeperImpl::NumDealerships()
{
   return Form1->numAutoDealers-1;

}

STDMETHODIMP TInventoryKeeperImpl::OpenForBusiness(unsigned hWindow)
{ 
	Form1->DealerFactory[Form1->DealerID].Open = true;
	Form1->DealerFactory[Form1->DealerID].hwnd = (HWND)hWindow;
        Form1->EditDealerCount->Text = AnsiString(NumDealerships());
	return Form1->DealerID;
}

