/*---------------------------------------------------------------------------
  CustomerForm.cpp
        Chapter 17 - COM Automation Controller Example w/ Event Firing
        created by Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "CustomerForm.h"
#include "MenuForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm2 *Form2;
//---------------------------------------------------------------------------
__fastcall TForm2::TForm2(TComponent* Owner)
        : TForm(Owner)
{
  myid = -1;
}
//---------------------------------------------------------------------------


void __fastcall TForm2::FormCreate(TObject *Sender)
{
  Restaurant = Restaurantwaiter_tlb::CoCustomerAction::Create();

  // Connect dispatch events:
  FRestaurantSink.OnOrderReceived = TForm2::OnOrderReceived;
  FRestaurantSink.OnCheckReceived = TForm2::OnCheckReceived;
  FRestaurantSink.OnChangeReturned = TForm2::OnChangeReturned;
  FRestaurantSink.OnSeatingReady   = TForm2::OnSeatingReady;
  FRestaurantSink.OnReadyToOrderQuery = TForm2::OnReadyToOrderQuery;
  FRestaurantSink.Connect(Restaurant);

//  Restaurant->EnterRestaurant(&myid);
//  OnSeatingReady(myid);
  Restaurant->EnterRestaurant(&myid);
  Caption = "Customer #" + AnsiString(myid);
}



//---------------------------------------------------------------------------
void __fastcall TForm2::ButtonRequestSeatingClick(TObject *Sender)
{
   Restaurant->RequestSeating(myid);
   LabelStatus->Caption = "Waiting for Seating...";
   ButtonRequestSeating->Enabled = false;
}

//---------------------------------------------------------------------------
void __fastcall TForm2::OnSeatingReady(long customerID)
{
  LabelStatus->Caption = "Seated. Your menu is on the table."
                         "Pick it up and look at it";
  ButtonMenu->Enabled = true;
//  FormMenu->Show();
}

//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonMenuClick(TObject *Sender)
{
    FormMenu->Caption = "Menu (Customer #" +
                        AnsiString(myid) + ")";
    FormMenu->Show();
    LabelStatus->Caption = "Looking over menu";
}
//---------------------------------------------------------------------------
void __fastcall TForm2::OnReadyToOrderQuery()
{
  LabelStatus->Caption = "You've been asked if you're ready to order.";
  ButtonPlaceOrder->Enabled = true;
}

//---------------------------------------------------------------------------
void __fastcall TForm2::ButtonPlaceOrderClick(TObject *Sender)
{
   TOrder order;

   order.Drink          = FormMenu->RadioGroupDrink->ItemIndex;
   order.Appetizer      = FormMenu->RadioGroupAppetizer->ItemIndex;
   order.Dessert        = FormMenu->RadioGroupDessert->ItemIndex;
   order.Entree         = FormMenu->RadioGroupEntree->ItemIndex;
   order.SideDish       = FormMenu->RadioGroupSideDish->ItemIndex;

   Restaurant->PlaceOrder(myid,order);
   LabelStatus->Caption = "Placed Order.  Waiting for meal...";
   FormMenu->Close();  // close menu
   ButtonPlaceOrder->Enabled = false;
   ButtonMenu->Enabled = false;
}

//---------------------------------------------------------------------------
void __fastcall TForm2::OnOrderReceived()
{
  LabelStatus->Caption = "Meal has arrived. You are now eating.";
  ButtonRequestBill->Enabled = true;
}

//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonRequestBillClick(TObject *Sender)
{
///
   Restaurant->RequestCheck(myid);
   LabelStatus->Caption = "Waiting for Check...";
   ButtonRequestBill->Enabled = false;
}

//---------------------------------------------------------------------------
void __fastcall TForm2::OnCheckReceived(double amount)
{
  LabelStatus->Caption = "You have the bill.  Pay when you're ready.";
  LabelAmount->Caption = FloatToStrF(amount, ffCurrency, 0, 2);
  EditTendered->Text   = FloatToStrF(amount, ffFixed, 0, 2);

  double tip = amount * 0.15; // 15 percent
  EditTip->Text   = FloatToStrF(tip, ffFixed, 0, 2);

  ButtonPayBill->Enabled = true;
  bill = amount;
  PanelBill->Enabled = true;
}

//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonPayBillClick(TObject *Sender)
{
    PaymentType paymenttype = (PaymentType)RadioGroupPaymentType->ItemIndex;
    double amount =  StrToFloatDef(EditTendered->Text, 0.00);
    if (amount >= bill)
    {
        Restaurant->PayBill(myid,paymenttype,amount);
        ButtonPayBill->Enabled = false;
    }
    else
    {
        MessageBox(NULL,"This is not enough to cover amount of bill.",
                "Waiter Response",MB_OK);
    }
}

//---------------------------------------------------------------------------
void __fastcall TForm2::OnChangeReturned(double amount)
{
  LabelStatus->Caption = "Change / Credit Card has been returned.";
  LabelAmountReturned->Caption = FloatToStrF(amount, ffCurrency, 0, 2);
  ButtonLeaveTip->Enabled = true;
}


//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonLeaveTipClick(TObject *Sender)
{
   LabelStatus->Caption = "Left Tip.  Still hanging around restaurant.";
   double amount =  StrToFloatDef(EditTip->Text, 0.00);
   Restaurant->LeaveTip(myid, amount);
   ButtonLeaveTip->Enabled = false;
   ButtonExit->Enabled = true;
}


//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonExitClick(TObject *Sender)
{
        Close();
}

//---------------------------------------------------------------------------

void __fastcall TForm2::FormClose(TObject *Sender, TCloseAction &Action)
{
        Restaurant->ExitRestaurant(myid);
}
//---------------------------------------------------------------------------

void __fastcall TForm2::ButtonOpenNewCustomerClick(TObject *Sender)
{
  WinExec ("Customer.exe", SW_SHOWNORMAL);
}

//---------------------------------------------------------------------------

