/*---------------------------------------------------------------------------
  RestaurantForm.cpp
        Chapter 17 - COM Automation Server Example w/ Event Firing
        created by Paul Gustavson, 2002
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "RestaurantForm.h"
#include "RestaurantWaiter_TLB.h"
#include "time.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
  nextcustomerid = 1;
  numcustomers = 0;
  multiplier = 30;
  numseats = 25;
  time_ToSeat =  2; // minutes
  time_ToServeMeal =  12; // minutes
  time_ToGetCheck =  2; // minutes
  time_ToReturnChange =  2; // minutes
  time_ReadyToOrder = 2; // minutes
  lasttimesec = 0; // seconds elapsed
  StringGridCustomers->Cells[0][0] = "Customer";
  StringGridCustomers->Cells[1][0] = "Status";
  StringGridCustomers->Cells[2][0] = "Amount";
  StringGridCustomers->Cells[3][0] = "Tip";
  StringGridCustomers->Cells[4][0] = "Time In";
  StringGridCustomers->Cells[5][0] = "Time Out";
  StringGridCustomers->Cells[6][0] = "Duration";
}

//---------------------------------------------------------------------------
AnsiString TForm1::SimulationTime(unsigned long timesec)
{
     AnsiString value;

     unsigned int min = timesec / 60;
     unsigned int hour = min/24;
     min = min - (hour * 24);
     unsigned int sec = timesec - ((hour * 24 * 60) + (min * 60));
     
     AnsiString minstr;
     AnsiString hourstr;
     AnsiString secstr;
     if (min < 10)
       minstr = "0" + AnsiString(min);
     else
       minstr = AnsiString(min);
     if (hour < 10)
       hourstr = "0" + AnsiString(hour);
     else
       hourstr = AnsiString(hour);
     if (sec < 10)
       secstr = "0" + AnsiString(sec);
     else
       secstr = AnsiString(sec);
     value = hourstr + ":" + minstr+ ":" + secstr;
     return value;
}

//---------------------------------------------------------------------------
long __fastcall TForm1::AddCustomer(TCustomerActionImpl* ca)
{
//  MessageBeep(MB_OK);
  long customerID = nextcustomerid;
  nextcustomerid++;
  numcustomers++;

  customer[customerID-1] = TCustomer(lasttimesec); // initialize;

  customer[customerID-1].status = Entered;
  customer[customerID-1].customerID  = customerID;
  customer[customerID-1].customeraction = ca;

  LabelNumCustomers->Caption = AnsiString(numcustomers);
  StringGridCustomers->Cells[0][customerID] = AnsiString(customerID);
  StringGridCustomers->Cells[1][customerID] = "Entered";
  StringGridCustomers->Cells[4][customerID] =
                SimulationTime(customer[customerID-1].timein);

//  if (numseats > 0)
//          StringGridCustomers->Cells[1][customerID] = "Seated";

  return customerID;
}

//---------------------------------------------------------------------------
void __fastcall TForm1::RequestSeating(long customerID)
{
   if (!customer[customerID-1].timer)
   {
        customer[customerID-1].timer = new TTimer(Form1);
        customer[customerID-1].timer->OnTimer =
            customer[customerID-1].timerevent;
   }
   customer[customerID-1].timer->Interval = (time_ToSeat * 60000) / multiplier;
   customer[customerID-1].timer->Enabled  = true;
   customer[customerID-1].status = WaitingToBeSeated;
   StringGridCustomers->Cells[1][customerID] = "Waiting To Be Seated";
}

//---------------------------------------------------------------------------
void __fastcall TForm1::PlaceOrder(long customerID, TOrder order)
{
 if (customerID >= 1)
 {
  StringGridCustomers->Cells[1][customerID] = "Waiting For Meal";
  customer[customerID-1].order  = order;
  customer[customerID-1].status = WaitingForFood;

   if (!customer[customerID-1].timer)
   {
        customer[customerID-1].timer = new TTimer(Form1);
        customer[customerID-1].timer->OnTimer =
            customer[customerID-1].timerevent;
   }
   customer[customerID-1].timer->Interval = (time_ToServeMeal * 60000) / multiplier;
   customer[customerID-1].timer->Enabled  = true;
 }

 // click on StringGrid for this customer - show the order
  StringGridCustomers->Row = customerID;
  StringGridCustomersSelectCell(Application,0,customerID,NULL);
}

//---------------------------------------------------------------------------
void __fastcall TForm1::RequestCheck(long customerID)
{
  StringGridCustomers->Cells[1][customerID] = "Ready For Check";
  customer[customerID-1].status = WaitingForBill;
   if (!customer[customerID-1].timer)
   {
        customer[customerID-1].timer = new TTimer(Form1);
        customer[customerID-1].timer->OnTimer =
            customer[customerID-1].timerevent;
   }
   customer[customerID-1].timer->Interval = (time_ToGetCheck * 60000) / multiplier;
   customer[customerID-1].timer->Enabled  = true;
}

//---------------------------------------------------------------------------
void __fastcall TForm1::PayBill(long customerID, PaymentType Type,
                    double Amount)
{
  StringGridCustomers->Cells[1][customerID] = "Waiting For Change or Charge Card";
  customer[customerID-1].status = WaitingForChange_Card;

  customer[customerID-1].change = Amount - customer[customerID-1].bill;

   if (!customer[customerID-1].timer)
   {
        customer[customerID-1].timer = new TTimer(Form1);
        customer[customerID-1].timer->OnTimer =
            customer[customerID-1].timerevent;
   }
   customer[customerID-1].timer->Interval = (time_ToReturnChange * 60000) / multiplier;
   customer[customerID-1].timer->Enabled  = true;
}

//---------------------------------------------------------------------------

void __fastcall TForm1::LeaveTip(long customerID, double Amount)
{
  customer[customerID-1].status = AfterDinnerChat;
  customer[customerID-1].tip    = Amount;
  StringGridCustomers->Cells[1][customerID] = "After Dinner Chat";
  StringGridCustomers->Cells[3][customerID] =
         FloatToStrF(Amount, ffCurrency, 0, 2);
}

//---------------------------------------------------------------------------

void __fastcall TForm1::RemoveCustomer(long customerID)
{
  numcustomers--;
  customer[customerID-1].status = Departed;
  customer[customerID-1].timeout = lasttimesec; // initialize;
  customer[customerID-1].duration =
          customer[customerID-1].timeout -
          customer[customerID-1].timein;

  LabelNumCustomers->Caption = AnsiString(numcustomers);
  LabelCustomersServed->Caption = AnsiString(nextcustomerid-1);
  StringGridCustomers->Cells[1][customerID] = "Departed";
  StringGridCustomers->Cells[5][customerID] =
        SimulationTime(customer[customerID-1].timeout);
  StringGridCustomers->Cells[6][customerID] =
        SimulationTime(customer[customerID-1].duration);


  // cleanup timer
  if (customer[customerID-1].timer)
  {
    delete customer[customerID-1].timer;
    customer[customerID-1].timer = NULL;
  }
}

//---------------------------------------------------------------------------

void __fastcall  TCustomer::timerevent(TObject *Sender)
{
  // if time up
  timer->Enabled = false;
  if (status == WaitingToBeSeated)
  {
     Form1->StringGridCustomers->Cells[1][customerID] = "Seated with menu";
     status = PreparingToOrder;
     customeraction->Fire_OnSeatingReady(customerID);

     // set up timer for waiter to check to see if customer is ready to order
     timer->Interval = (Form1->time_ReadyToOrder * 60000) / Form1->multiplier;
     timer->Enabled  = true;

  }
  else if (status == PreparingToOrder)
  {
     Form1->StringGridCustomers->Cells[1][customerID] = "Ready To Order Query";
     customeraction->Fire_OnReadyToOrderQuery();
  }
  else if (status == WaitingForFood)
  {
     Form1->StringGridCustomers->Cells[1][customerID] = "Meal Has Arrived";
     status = Eating;
     customeraction->Fire_OnOrderReceived();
  }
  else if (status == Eating)
  {
  }
  else if (status == WaitingForBill)
  {
     Form1->StringGridCustomers->Cells[1][customerID] = "Bill On Table";
     Form1->StringGridCustomers->Cells[2][customerID] =
         FloatToStrF(bill, ffCurrency, 0, 2);
     customeraction->Fire_OnCheckReceived(bill);
  }
  else if (status == WaitingForChange_Card)
  {
     Form1->StringGridCustomers->Cells[1][customerID] = "Change / Charge Card Returned";
     status = AfterDinnerChat;
     customeraction->Fire_OnChangeReturned(change);
  }
  else if (status == AfterDinnerChat)
  {
  }
  else if (status == Departed)
  {
  }
}

//---------------------------------------------------------------------------

AnsiString TForm1::GetEntreeName(EntreeType entreetype)
{
   AnsiString name;
   switch (entreetype)
   {
      case Steak:       name = "Steak"; break;
      case Chicken:     name = "Chicken"; break;
      case Fish:        name = "Steak"; break;
      case Shrimp:      name = "Shrimp"; break;
      case Lobster:     name = "Lobster"; break;
   }
   return name;
}
//---------------------------------------------------------------------------

AnsiString TForm1::GetAppetizerName(AppetizerType appetizertype)
{
   AnsiString name;
   switch (appetizertype)
   {
      case Nachos:       name = "Nachos"; break;
      case Breadsticks:  name = "Breadsticks"; break;
      case Wings:        name = "Wings"; break;
      case Soup:         name = "Soup"; break;
   }
   return name;

}
//---------------------------------------------------------------------------

AnsiString TForm1::GetDrinkName(DrinkType drinktype)
{
   AnsiString name;
   switch (drinktype)
   {
      case Nachos:       name = "Water"; break;
      case Breadsticks:  name = "Soda"; break;
      case Wings:        name = "Iced Tea"; break;
      case Soup:         name = "Coffee"; break;
   }
   return name;
}
//---------------------------------------------------------------------------

AnsiString TForm1::GetSideDishName(SideOrderType sidedish)
{
   AnsiString name;
   switch (sidedish)
   {
      case Veggies:      name = "Veggies"; break;
      case Fries:        name = "Fries"; break;
      case BakedPotato:  name = "Baked Potato"; break;
      case OnionRings:   name = "Onion Rings"; break;
   }
   return name;
}
//---------------------------------------------------------------------------

AnsiString TForm1::GetDessertName(DessertType dessert)
{
   AnsiString name;
   switch (dessert)
   {
      case Restaurantwaiter_tlb::Pie:      name = "Pie"; break;
      case Cake:     name = "Cake"; break;
      case IceCream: name = "Ice Cream"; break;
      case Cookies:  name = "Cookies"; break;
   }
   return name;
}

//---------------------------------------------------------------------------
void __fastcall TForm1::StringGridCustomersSelectCell(TObject *Sender,
      int ACol, int ARow, bool &CanSelect)
{
    // find out what cell they clicked
    long customerID = ARow;
    if (nextcustomerid <= customerID)
    {
        CanSelect = false;
        return;
    }
    if (ARow == 0)
    {
        CanSelect = false;
        return;
    }
    CanSelect = true;
    // display what they ordered
    LabelEntree->Caption = GetEntreeName(customer[customerID-1].order.Entree);
    LabelDrink->Caption = GetDrinkName(customer[customerID-1].order.Drink);
    LabelDessert->Caption = GetDessertName(customer[customerID-1].order.Dessert);
    LabelSideDish->Caption = GetSideDishName(customer[customerID-1].order.SideDish);
    LabelAppetizer->Caption = GetAppetizerName(customer[customerID-1].order.Appetizer);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ButtonOpenNewCustomerClick(TObject *Sender)
{
  WinExec ("Customer.exe", SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditMultiplierChange(TObject *Sender)
{
  // calculate
  long value = EditMultiplier->Text.ToIntDef(multiplier);
  LabelSimTime->Caption = AnsiString(60000 / value);
  multiplier = value;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditSeatsChange(TObject *Sender)
{
  long value = EditSeats->Text.ToIntDef(numseats);
  numseats = value; // minutes (simulation time)
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditSeatTimeChange(TObject *Sender)
{
  long value = EditSeatTime->Text.ToIntDef(time_ReadyToOrder);
  time_ReadyToOrder = value; // minutes (simulation time)
}

//---------------------------------------------------------------------------

void __fastcall TForm1::EditOrderTimeChange(TObject *Sender)
{
  long value = EditSeatTime->Text.ToIntDef(time_ToSeat);
  time_ToSeat = value; // minutes (simulation time)
}

//---------------------------------------------------------------------------

void __fastcall TForm1::EditServeFoodTimeChange(TObject *Sender)
{
  long value = EditServeFoodTime->Text.ToIntDef(time_ToServeMeal);
  time_ToServeMeal = value; // minutes (simulation time)
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditGetCheckTimeChange(TObject *Sender)
{
  long value = EditGetCheckTime->Text.ToIntDef(time_ToGetCheck);
  time_ToGetCheck = value; // minutes (simulation time)

}
//---------------------------------------------------------------------------

void __fastcall TForm1::EditReturnChangeTimeChange(TObject *Sender)
{
  long value = EditReturnChangeTime->Text.ToIntDef(time_ToReturnChange);
  time_ToReturnChange = value; // minutes (simulation time)
}

//---------------------------------------------------------------------------

void __fastcall TForm1::TimerClockTimer(TObject *Sender)
{
     unsigned long timesec = lasttimesec + multiplier; // seconds
     lasttimesec = timesec;
     LabelClock->Caption = SimulationTime(timesec);
}
//---------------------------------------------------------------------------


