/*---------------------------------------------------------------------------
  RestaurantSink.h
        Chapter 17 - COM Automation Server Example w/ Event Firing
        created by Paul Gustavson, 2002
---------------------------------------------------------------------------*/
#if !defined(RESTAURANTSINK_H__)
#define RESTAURANTSINK_H__

#include <atlvcl.h>
#include <atlbase.h>
#include <atlcom.h>
#include <ComObj.HPP>
#include <utilcls.h>
#include "RestaurantWaiter_TLB.h"

typedef void __fastcall (__closure * TOrderReceivedEvent)(void);
typedef void __fastcall (__closure * TCheckReceivedEvent)(double Amount);
typedef void __fastcall (__closure * TChangeReturnedEvent)(double Amount);
typedef void __fastcall (__closure * TSeatingReadyEvent)(long customerID);
typedef void __fastcall (__closure * TReadyToOrderQuery)(void);

//---------------------------------------------------------------------------
// Create a class that handles ICustomerActionEvents
class TRestaurantSink :
   public TEventDispatcher<TRestaurantSink, &DIID_ICustomerActionEvents>
{
protected:
  // Event field
  TOrderReceivedEvent  FOnOrderReceived;
  TCheckReceivedEvent  FOnCheckReceived;
  TChangeReturnedEvent FOnChangeReturned;
  TSeatingReadyEvent   FOnSeatingReady;
  TReadyToOrderQuery   FOnReadyToOrderQuery;

  // Event dispatcher
  HRESULT InvokeEvent(DISPID id, TVariant* params)
  {
      if ((id == 1) && (FOnOrderReceived != NULL)) // OnOrderReceived
        FOnOrderReceived();
      else if ((id == 2) && (FOnCheckReceived != NULL)) // OnCheckReceived
        FOnCheckReceived(params[0]);
      else if ((id == 3) && (FOnChangeReturned != NULL)) // OnChangeReturned
        FOnChangeReturned(params[0]);
      else if ((id == 4) && (FOnSeatingReady != NULL)) // OnReadyToOrderQuery
        FOnSeatingReady(params[0]);
      else if ((id == 5) && (FOnReadyToOrderQuery != NULL)) // OnReadyToOrderQuery
        FOnReadyToOrderQuery();
      return S_OK;
  }

  // Reference to the event sender
  CComPtr<IUnknown> m_pSender;

public:
  __property TOrderReceivedEvent OnOrderReceived =
    { read = FOnOrderReceived, write = FOnOrderReceived };

  __property TCheckReceivedEvent OnCheckReceived =
    { read = FOnCheckReceived, write = FOnCheckReceived };

  __property TChangeReturnedEvent OnChangeReturned =
    { read = FOnChangeReturned, write = FOnChangeReturned };

  __property TSeatingReadyEvent OnSeatingReady =
    { read = FOnSeatingReady, write = FOnSeatingReady };

  __property TReadyToOrderQuery OnReadyToOrderQuery =
    { read = FOnReadyToOrderQuery, write = FOnReadyToOrderQuery };

public:
  TRestaurantSink() :
     m_pSender(NULL),
     FOnOrderReceived(NULL),
     FOnCheckReceived(NULL),
     FOnSeatingReady(NULL),
     FOnReadyToOrderQuery(NULL)
  {
  }

  virtual ~TRestaurantSink()
  {
     Disconnect();
  }

  // Connect to Server
  void Connect(IUnknown* pSender)
  {
     if (pSender != m_pSender)
        m_pSender = pSender;
     if (NULL != m_pSender)
       ConnectEvents(m_pSender);
  }

  // Disconnect from Server
  void Disconnect()
  {
    if (NULL != m_pSender)
    {
      DisconnectEvents(m_pSender);
      m_pSender = NULL;
    }
  }
};

#endif //RESTAURANTSINK_H__

