/*---------------------------------------------------------------------------
  wizard_memstatus.cpp
        Paul Gustavson,  (c) 2002
        Developed for...
                C++Builder Developer's Guide
                Chapter 23 - OTA Wizard Example
---------------------------------------------------------------------------*/

//---------------------------------------------------------------------------

#include <Windows.hpp>
#include <SysUtils.hpp>
#include <ToolsAPI.hpp>
#pragma hdrstop

#pragma resource "*.res"

#include "wizard_memstatus.h"
#include "ToolsAPIEx.h"
#include <stdio.h>
#include <cstdarg>
#include "MemStatusForm.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)


ULONG   __stdcall MemStatusWizard::AddRef()  { return inherited::AddRef(); }
ULONG   __stdcall MemStatusWizard::Release() { return inherited::Release(); }
HRESULT __stdcall MemStatusWizard::QueryInterface(const GUID& iid, void** obj)
{
  QUERY_INTERFACE(IOTAMenuWizard, iid, obj);
  QUERY_INTERFACE(IOTAWizard, iid, obj);
  return inherited::QueryInterface(iid, obj);
}

void __fastcall MemStatusWizard::AfterSave()  {}
void __fastcall MemStatusWizard::BeforeSave() {}
void __fastcall MemStatusWizard::Destroyed()  {}
void __fastcall MemStatusWizard::Modified()   {}

//---------------------------------------------------------------------------

AnsiString __fastcall MemStatusWizard::GetIDString()
{
  return "CBuilderDevelopersGuide.MemStat Wizard";
}

//---------------------------------------------------------------------------

AnsiString __fastcall MemStatusWizard::GetName()
{
  return "MemStat Wizard";
}

//---------------------------------------------------------------------------

TWizardState __fastcall MemStatusWizard::GetState()
{
  TWizardState result;
  result << wsEnabled;
  return result;
}

//---------------------------------------------------------------------------

AnsiString __fastcall MemStatusWizard::GetMenuText()
{
  return "MemStat Wizard...";
}

//---------------------------------------------------------------------------
_di_IOTAProject      MemStatusWizard::FindCurrentProject(_di_IOTAModuleServices ModServices)
{
    //To obtain the active IOTAProjectGroup reference,
    //iterate over all open modules and find the one module that
    //supports the IOTAProjectGroup interface.

    if (!ModServices) return NULL; // get out, no modules to search

    _di_IOTAModule module;
    _di_IOTAProjectGroup projectgroup = NULL;
    _di_IOTAProject project = NULL;

    bool done = false;
    int index = 0;
    MessageServices->AddTitleMessage(
       AnsiString(GetName() + " - Locating Active Project..."),MessageGroup);
    while (!done)
    {
       if (index < ModServices->ModuleCount)
       {
          module = ModServices->Modules[index];
          // cast module into project (if you can)
          projectgroup = (_di_IOTAProjectGroup)module;
          if (projectgroup)
          {
              //done = true;
              project = projectgroup->ActiveProject;
              MessageServices->AddTitleMessage(
                AnsiString(GetName() + " - Project Found = " +
                        project->FileName),MessageGroup);
              done = true;
          }
          else
          index++;
       }
       else
       {
         done = true;
          MessageServices->AddTitleMessage(
            AnsiString(GetName() + " -  Unable to locate project"),MessageGroup);
       }
    }  // while
    return project;
}

//---------------------------------------------------------------------------

TMenuItem * MemStatusWizard::FindMenuItemCaption(TMenuItem* topmenu,
                AnsiString Caption)
{
    TMenuItem *menuitem = NULL;
    bool done = false;
    if (!topmenu) return menuitem; // get out, no menu to search
    int index = 0;
    while (!done)
    {
       if (index < topmenu->Count)
       {
          menuitem = topmenu->Items[index];
          if (menuitem->Caption.AnsiPos(Caption))
          {
                  done = true;
          }
          index ++;
       }
       else
       {
          done = true;
          menuitem = NULL;
       }
    }  // while
    return menuitem;
}

//---------------------------------------------------------------------------


void __fastcall MemStatusWizard::SetupMessageViewAccess()
{
    BorlandIDEServices->Supports(MessageServices); // get access to Message View
    MessageGroup = MessageServices->AddMessageGroup(GetName());
    MessageServices->ClearMessageGroup(MessageGroup); // need clean canvas
    MessageServices->ShowMessageView(MessageGroup);   // make it visible
    MessageServices->AddTitleMessage(GetName() + " - Activated",MessageGroup);
}

//---------------------------------------------------------------------------

void __fastcall MemStatusWizard::Execute()
{
  Application->ProcessMessages();  // let menu processing complete

  bool launch = true;  // unless instruected otherwise, run the application
  TFormMemStat* FormMemStat = NULL;

  SetupMessageViewAccess();

  _di_IOTAModuleServices ModServices;
  BorlandIDEServices->Supports(ModServices); // get access to Modules)
  _di_IOTAProject project = FindCurrentProject(ModServices);
  if (project)
  {
      _di_IOTAProjectBuilder projectbuilder = project->ProjectBuilder;
     if (projectbuilder->ShouldBuild)
     {
          MessageServices->AddTitleMessage(
                AnsiString(GetName() + " - Project needs to be built first"),
                MessageGroup);

          AnsiString filename = ExtractFileName(project->FileName);
          AnsiString message = "MemStatus Wizard detected that the " +
                               filename + " project needs to be built first.\n"
                               "Do you wish to continue?";
          int result = MessageBox(NULL,message.c_str(),
                              "MemStat Wizard - Build Project?",
                              MB_YESNO);
          if (result == IDYES)
          {
              launch = projectbuilder->BuildProject(cmOTAMake,true,true);
          }
          else launch = false; // user doesn't want to run now
     }
  }
  else  // could not find project
  {
      AnsiString message = GetName() + " - Project not loaded. Unable to run.";
      MessageServices->AddTitleMessage(message,MessageGroup);
      MessageBox(NULL,message.c_str(),"MemStat Wizard - Build Project?", MB_OK);
      launch = false;
  }

  if (launch)  // if launch is still a go
  {
    MessageServices->AddTitleMessage(
        AnsiString(GetName() + " - Project is built and ready to run"),MessageGroup);

    _di_INTAServices NativeServices;
    BorlandIDEServices->Supports(NativeServices); // get access to IDE (menu bar)

    // let's find the Run top menu item...
    TMenuItem* MenuItem =
        FindMenuItemCaption(NativeServices->MainMenu->Items,"Run");
    if (MenuItem)
    {
        // now let's find the Run (F9) menu item
        TMenuItem* temp = FindMenuItemCaption(MenuItem,"Run");
        MenuItem = temp;
        if (MenuItem)
        {
            if (MenuItem->Enabled)
            {
                int result = MessageBox(NULL,
                        "MemStat Wizard is ready to launch active project and "\
                        "measure memory performance. \n\n" \
                        "Although not required, it's recommended that you "\
                        "close all other applications with the exception of "\
                        "C++Builder before running the memory test. This will "\
                        "allow the MemStat Wizard to more precisely measure "\
                        "memory performance of the application under test.\n\n"\
                        "Do you wish to continue?",
                        "MemStat Wizard - Ready to Run",MB_YESNO);
                if (result == IDYES)
                {
                    FormMemStat = new TFormMemStat(0);// instantiate MemStat form
                    FormMemStat->Show();  // show the wizard
                    Application->ProcessMessages(); //let FormMemStat complete
                    FormMemStat->SpeedButtonStartClick(0); // measure memory first
                    MessageServices->AddTitleMessage(
                        AnsiString(GetName() + " - " +
                        FormMemStat->GetMemoryStartFree()),MessageGroup);
                    MenuItem->OnClick(0);  // run the app
                }
                else
                {
                    MessageServices->AddTitleMessage(
                        AnsiString(GetName() +
                        " - User aborted run."),MessageGroup);
                }

            }
        }
    }
    if (!MenuItem)
    {
                MessageServices->AddTitleMessage(
                   AnsiString(GetName() + " - Unable to run application. " \
                   "Project not loaded."),MessageGroup);
    }
  }

  // little loop processing here - but don't tie up system (check and get out)
  if (FormMemStat)
  {
      while (FormMemStat->Visible)  // check
      {
           Application->HandleMessage(); //get out (process current messages)
      }
      MessageServices->AddTitleMessage(
          AnsiString(GetName() + " - " +
          FormMemStat->GetMemoryStopFree()),MessageGroup);
      MessageServices->AddTitleMessage(
          AnsiString(GetName() + " - " +
          FormMemStat->GetMemoryFinalUsage()),MessageGroup);
      MessageServices->AddTitleMessage(
          AnsiString(GetName() + " - " +
          FormMemStat->GetMemoryPeakUsage()),MessageGroup);
      delete FormMemStat;
  }
  MessageServices->AddTitleMessage(GetName() + " - Completed",MessageGroup);
}

//---------------------------------------------------------------------------

void __fastcall MemStatusWizard::action_execute(TObject* Sender)
{
  Execute();
}


void __fastcall MemStatusWizard::action_update(TObject* Sender)
{
  // Enable the action item only if a file is open.
  action->Enabled = interface_cast<IOTAModuleServices>(BorlandIDEServices)->CurrentModule() != 0;
}



//---------------------------------------------------------------------------

__fastcall MemStatusWizard::MemStatusWizard()
: action(0), menu_item(0)
{

  _di_INTAServices services;
  Cast(BorlandIDEServices, services);
/*
  // Add an image to the image list.
  Graphics::TBitmap* bitmap(new Graphics::TBitmap());
  bitmap->LoadFromResourceID(reinterpret_cast<unsigned>(HInstance), 1);
  int image = services->AddMasked(bitmap, bitmap->TransparentColor, "Tempest Software.intro wizard image");
  delete bitmap;

  // Create an action item for the demo action.
  action = new TAction(0);
  action->ActionList = services->ActionList;
  action->Caption = GetMenuText();
  action->Hint = "Display a silly dialog box";
  action->ImageIndex = image;
  action->Name = "WizDemoCommand";
  action->OnExecute = action_execute;
  action->OnUpdate = action_update;

  // Create a menu item at the top of the View menu.
  for (int i = 0; i < services->MainMenu->Items->Count; ++i)
  {
    TMenuItem* item = services->MainMenu->Items->Items[i];
    if (CompareText(item->Name, "ViewsMenu") == 0)
    {
      menu_item = new TMenuItem(0);
      menu_item->Action = action;
      menu_item->Name = "WizDemoItem";
      item->Insert(0, menu_item);
    }
  }
*/
}

static void remove_action_from_toolbar(TAction* action, TToolBar* toolbar)
{
  for (int i = toolbar->ButtonCount; --i >= 0; )
  {
    TToolButton* button = toolbar->Buttons[i];
    if (button->Action == action)
    {
      toolbar->Perform(CM_CONTROLCHANGE, WPARAM(button), 0);
      delete button;
    }
  }
}

//---------------------------------------------------------------------------

__fastcall MemStatusWizard::~MemStatusWizard()
{
/*
  _di_INTAServices services;
  Cast(BorlandIDEServices, services);
  // Check all the toolbars, and remove any buttons that use this action.
  remove_action_from_toolbar(action, services->ToolBar[sCustomToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sDesktopToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sStandardToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sDebugToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sViewToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sInternetToolBar]);
  remove_action_from_toolbar(action, services->ToolBar[sCORBAToolBar]);

  delete menu_item;
  delete action;
*/
}

//---------------------------------------------------------------------------

#ifndef DLL  // it's a package
namespace Wizard_memstatus
{
  void __fastcall PACKAGE Register()
  {
    RegisterPackageWizard(new MemStatusWizard());
  }
}
#endif




