/*---------------------------------------------------------------------------
  ToolsAPIEx.h
        Paul Gustavson,  2002
        Developed for...
                C++Builder Developer's Guide
                Chapter 23 - OT Wizard Example
---------------------------------------------------------------------------*/
//---------------------------------------------------------------------------

#ifndef ToolsAPIExH
#define ToolsAPIExH
//---------------------------------------------------------------------------

#include <ToolsAPI.hpp>
#include <typeinfo>

// macro for implementing interfaces
#define QUERY_INTERFACE(T, iid, obj)   \
  if ((iid) == __uuidof(T))            \
  {                                    \
    *(obj) = static_cast<T*>(this);    \
    static_cast<T*>(*(obj))->AddRef(); \
    return S_OK;                       \
  }

#define DLL  // enable this if building a DLL

#ifdef DLL   // we'll need this for building a DLL
#define BorlandIDEServices LocalIDEServices
extern _di_IBorlandIDEServices LocalIDEServices;
#endif


class PACKAGE TNotifierObject : public IOTANotifier {
public:
  __fastcall TNotifierObject() : ref_count(0) {}
  virtual __fastcall ~TNotifierObject();
  void __fastcall AfterSave();
  void __fastcall BeforeSave();
  void __fastcall Destroyed();
  void __fastcall Modified();
protected:
  // IInterface
  virtual HRESULT __stdcall QueryInterface(const GUID&, void**);
  virtual ULONG __stdcall AddRef();
  virtual ULONG __stdcall Release();
private:
  long ref_count;
};



// TSERVICESFILE
// This is a convenient(!) way to do IOTAFiles. Create an object of this
// type, passing a string in the constructor. You can then use it as
// an IOTAFile interface, for anything else that needs one.
class TServicesFile : public IOTAFile
{
private:
  AnsiString s; TDateTime dt; long ref;
public:
  TServicesFile(AnsiString as) : IOTAFile(), s(as), dt(-1), ref(0) {}
  // IUnknown
  HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppv) {if (riid==IID_IUnknown || riid==IID_IOTAFile) {*ppv=this; AddRef(); return S_OK;} else return E_NOINTERFACE;}
  ULONG STDMETHODCALLTYPE AddRef() {return InterlockedIncrement(&ref);}
  ULONG STDMETHODCALLTYPE Release() {int tmp = InterlockedDecrement(&ref); if (tmp==0) delete this; return tmp;}
  // IOTAFile
  AnsiString __fastcall GetSource() {return s;}
  TDateTime __fastcall GetAge() {return dt;}
};


template<typename T>
void Cast(_di_IInterface instance, T& intf)
{
  if (! instance->Supports(intf))
    throw EIntfCastError(AnsiString("Interface not supported: ") + typeid(T).name());
}

template<typename T>
DelphiInterface<T> interface_cast(_di_IInterface instance)
{
  DelphiInterface<T> result;
  Cast(instance, result);
  return result;
}
#endif
 