/*---------------------------------------------------------------------------
  wizard_memstatus.h
        Paul Gustavson,  (c) 2002
        Developed for...
                C++Builder Developer's Guide
                Chapter 23 - OTA Wizard Example
---------------------------------------------------------------------------*/
#ifndef wizard_memstatusH
#define wizard_memstatusH
//---------------------------------------------------------------------------
#include <typeinfo>
#include "ToolsAPIEx.h"



class PACKAGE MemStatusWizard : public TNotifierObject, public IOTAMenuWizard
{
  typedef TNotifierObject inherited;
public:
  __fastcall MemStatusWizard();
  __fastcall ~MemStatusWizard();

  // override IOTAWizard Methods
  virtual AnsiString __fastcall GetIDString();
  virtual AnsiString __fastcall GetName();
  virtual TWizardState __fastcall GetState();
  virtual void __fastcall Execute();

  // override IOTAMenuWizard method
  virtual AnsiString __fastcall GetMenuText();

  // override NotifierObjectObject  methods
  void __fastcall AfterSave();
  void __fastcall BeforeSave();
  void __fastcall Destroyed();
  void __fastcall Modified();

  void __fastcall SetProcessActive(bool value);  // used by debug notifier

  // expose these properties
  _di_IOTAMessageServices  MessageServices;
  _di_IOTAMessageGroup     MessageGroup;

  AnsiString projectname;
  //results;
  AnsiString memtotal;
  AnsiString memstartfree;
  AnsiString memstopfree;
  AnsiString mempeakusage;
  AnsiString memfinalusage;
protected:
  // override IInterface  methods
  virtual HRESULT __stdcall QueryInterface(const GUID&, void**);
  virtual ULONG   __stdcall AddRef();
  virtual ULONG   __stdcall Release();

private:
  bool                     ProcessActive;   // for SetProcessActive

  void __fastcall SetupMessageViewAccess();
  TMenuItem* FindMenuItemCaption(TMenuItem* topmenu,  AnsiString Caption);
  _di_IOTAProject FindCurrentProject(_di_IOTAModuleServices ModServices);

  AnsiString __fastcall ExtractFileNameNoExt(const AnsiString FileLocation,
                                                bool keeppath=false);

//  TList* notifierlist;    // need to keep track of all notifiers we add
//  IOTADebuggerNotifier* debugnotifier; // keep track of debugnotifier

  void __fastcall action_execute(TObject* Sender);
  void __fastcall action_update(TObject* Sender);

  TMenuItem* menu_item;
  TAction* action;
};

//---------------------------------------------------------------------------

class DebugNotifier: public TNotifierObject, public IOTADebuggerNotifier
{
  typedef TNotifierObject inherited;
public:
  __fastcall DebugNotifier(const _di_IOTADebuggerServices debugger,
                           const MemStatusWizard*   wizard);
  __fastcall ~DebugNotifier();

  // IOTADebuggerNotifer
  virtual void __fastcall BreakpointAdded(_di_IOTABreakpoint Breakpoint);
  virtual void __fastcall BreakpointDeleted(_di_IOTABreakpoint Breakpoint);
  virtual void __fastcall ProcessCreated(_di_IOTAProcess Process);
  virtual void __fastcall ProcessDestroyed(_di_IOTAProcess Process);

  // override NotifierObjectObject  methods
  void __fastcall AfterSave();
  void __fastcall BeforeSave();
  void __fastcall Destroyed();  // implement this
  void __fastcall Modified();


protected:
  // override IInterface  methods
  virtual HRESULT __stdcall QueryInterface(const GUID&, void**);
  virtual ULONG   __stdcall AddRef();
  virtual ULONG   __stdcall Release();

private:
  const MemStatusWizard*   wizard; // keep track of the wizard that owns this notifier
  _di_IOTADebuggerServices debugger; // keep track of debuggerservice that added me as a notifier
  AnsiString name; // remember the debugger's old name
  int index; // Notifier index
};

//---------------------------------------------------------------------------

class File : public IOTAFile {
public:
  __fastcall File(const AnsiString source);
  virtual __fastcall ~File();
  AnsiString __fastcall GetSource();
  System::TDateTime __fastcall GetAge();
protected:
  // override IInterface  methods
  virtual HRESULT __stdcall QueryInterface(const GUID&, void**);
  virtual ULONG __stdcall AddRef();
  virtual ULONG __stdcall Release();
private:
  long ref_count;
  System::TDateTime age;
  AnsiString source;
};

//---------------------------------------------------------------------------

class Creator: public IOTAModuleCreator
{
public:

  __fastcall Creator(const AnsiString creator_type,
                     const MemStatusWizard*   wizard);

  virtual __fastcall ~Creator();

  // IOTAModuleCreator
  virtual AnsiString __fastcall GetAncestorName();
  virtual AnsiString __fastcall GetImplFileName();
  virtual AnsiString __fastcall GetIntfFileName();
  virtual AnsiString __fastcall GetFormName();
  virtual bool __fastcall GetMainForm();
  virtual bool __fastcall GetShowForm();
  virtual bool __fastcall GetShowSource();
  virtual _di_IOTAFile __fastcall NewFormFile(
        const AnsiString FormIdent, const AnsiString AncestorIdent);
  virtual _di_IOTAFile __fastcall NewImplSource(
        const AnsiString ModuleIdent, const AnsiString FormIdent,
        const AnsiString AncestorIdent);
  virtual _di_IOTAFile __fastcall NewIntfSource(
        const AnsiString ModuleIdent, const AnsiString FormIdent,
        const AnsiString AncestorIdent);
  virtual void __fastcall FormCreated(
        const _di_IOTAFormEditor FormEditor);

  // IOTACreator
  virtual AnsiString __fastcall GetCreatorType();
  virtual bool __fastcall GetExisting();
  virtual AnsiString __fastcall GetFileSystem();
  virtual _di_IOTAModule __fastcall GetOwner();
  virtual bool __fastcall GetUnnamed();

protected:
  // override IInterface  methods
  virtual HRESULT __stdcall QueryInterface(const GUID&, void**);
  virtual ULONG   __stdcall AddRef();
  virtual ULONG   __stdcall Release();

private:
  long ref_count;
  const AnsiString creator_type;
  const MemStatusWizard*   wizard; // keep track of the wizard that owns this notifier
};
#endif



